﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;

using System.Drawing;
using MyGeometry;
using MyCholmodSolver;

using System.IO;


using Accord.Statistics.Analysis;

using Tao.OpenGl;

namespace ShapeGraph
{	
	public enum ShapeType2D { TRIANGLE, QUAD, POLYGON, CIRCLLE, NONE };
	public enum ProxyConstrutionMode { AXISALIGNED, SPECIFYHEIGHT, OTHERS, NONE }
	public enum ProxyType { CUBOID, CYLINDER, GC, POLY };

	public unsafe sealed class HelperFunction
	{
		/// static members
		public static double cos3 = Math.Cos(Math.PI / 60);
		public static double cos5 = Math.Cos(Math.PI / 36);
		public static double sin5 = Math.Sin(Math.PI / 36);
		public static double sin10 = Math.Sin(Math.PI / 18);
		public static double cos10 = Math.Cos(Math.PI / 18);

		public static PointF Vector2PointF(Vector2d v)
		{
			return new PointF((float)v.x, (float)v.y);
		}
		public static Vector2d PointF2Vecter2d(PointF p)
		{
			return new Vector2d(p.X, p.Y);
		}
		public static double Distance2Line(Vector2d p, Vector2d u, Vector2d v)
		{
			Vector2d uv = (v - u).Normalize();
			Vector2d up = p - u;
			return (up - up.Dot(uv) * uv).Length();
		}
		public static double Distance2LineSegment(Vector2d p, Vector2d u, Vector2d v)
		{
			bool isProjectionIn = !((p - u).Dot(v - u) < 0 || (p - v).Dot(u - v) < 0);
			if (!isProjectionIn)
			{
				return Math.Min((p - u).Length(), (p - v).Length());
			}
			else
			{
				Vector2d uv = (v - u).Normalize();
				Vector2d up = p - u;
				return (up - up.Dot(uv) * uv).Length();
			}
		}
		public static double Distance2LineSegment(Vector3d p, Vector3d u, Vector3d v, out double r)
		{
			bool isProjectionIn = !((p - u).Dot(v - u) < 0 || (p - v).Dot(u - v) < 0);
			Vector3d uv = (v - u).Normalize();
			Vector3d up = p - u;
			double l = up.Dot(uv);
			r = l / (v - u).Length();
			if (!isProjectionIn)
			{
				return Math.Min(up.Length(), (p - v).Length());
			}
			else
			{
				return (up - l * uv).Length();
			}
		}
		public static double Distance2LineSegment(Vector2d p, Vector2d u, Vector2d v, out double projectionRatio)
		{
			Vector2d uv = (v - u).Normalize();
			Vector2d up = (p - u);
			double l = up.Dot(uv);
			projectionRatio = l / (v - u).Length();

			return (up - l * uv).Length();

		}
		public static Vector2d FindLinesIntersectPoint(LineSegment l1, LineSegment l2)
		{
			Vector3d u1 = new Vector3d(l1.u.pos, 1), v1 = new Vector3d(l1.v.pos, 1);
			Vector3d u2 = new Vector3d(l2.u.pos, 1), v2 = new Vector3d(l2.v.pos, 1);
			Vector3d uv = u1.Cross(v1).Cross(u2.Cross(v2)).HomogenousNormalize();
			return uv.ToVector2d();
		}
		public static Matrix4d ReflectionalMatrix(Vector3d plane_normal)
		{
			// create an coordinates sys, with plane_normal as x-axis
			Vector3d x = plane_normal;
			Vector3d y;
			if (x.x == 0 && x.y == 0)
				y = new Vector3d(1, 0, 0);
			else
				y = (new Vector3d(-x.y, x.x, 0)).Normalize();
			Vector3d z = x.Cross(y).Normalize();
			Matrix3d R = new Matrix3d(x, y, z).Transpose();
			Matrix3d InvR = R.Inverse();
			Matrix4d U = new Matrix4d(R);
			Matrix4d V = new Matrix4d(InvR);
			Matrix4d I = Matrix4d.IdentityMatrix();
			I[0, 0] = -1; // reflect matrix along yz plane
			return V * I * U;
		}
		public static Vector3d GetMirrorSymmetryPoint(Vector3d pt, Vector3d normal, Vector3d center)
		{
			Matrix4d R = ReflectionalMatrix(normal);
			return center + (R * new Vector4d(pt - center, 0)).XYZ();
		}
		public static Vector3d GetMirrorSymmetryVector(Vector3d vec, Vector3d normal)
		{
			Matrix4d R = ReflectionalMatrix(normal);
			return (R * new Vector4d(vec, 0)).XYZ();
		}
		public static Matrix3d ComputeHomographyMatrix(Vector2d[] imgpts, Vector2d[] refpts)
		{
			// compute mapping from imgpts to refpts
			if (refpts.Length != imgpts.Length || imgpts.Length < 4) return null;

			int n = imgpts.Length * 2;
			double[] b = new double[n];
			double[] h = new double[n + 1];

			CholmodSolver linearsolver = new CholmodSolver();
			linearsolver.InitializeMatrixA(n, 8);
			int index = 0, j = 0;
			foreach (Vector2d p in imgpts)
			{
				double x = p.x, y = p.y, u = refpts[index].x, v = refpts[index].y;

				linearsolver.Add_Coef(j, 0, x);
				linearsolver.Add_Coef(j, 1, y);
				linearsolver.Add_Coef(j, 2, 1);
				linearsolver.Add_Coef(j, 6, -x * u);
				linearsolver.Add_Coef(j, 7, -y * u);

				int jj = j + 1;
				linearsolver.Add_Coef(jj, 3, x);
				linearsolver.Add_Coef(jj, 4, y);
				linearsolver.Add_Coef(jj, 5, 1);
				linearsolver.Add_Coef(jj, 6, -x * v);
				linearsolver.Add_Coef(jj, 7, -y * v);

				b[j] = u;
				b[jj] = v;

				++index; j += 2;
			}

			linearsolver.InitializeSolver();
			linearsolver.SetFinalPack(0);
			linearsolver.Factorize();

			fixed (double* _b = b)
			{
				linearsolver.InitializeMatrixB(_b, b.Length, 1);
			}
			fixed (double* _x = h)
			{
				linearsolver.Linear_Solve(_x, false);
			}

			h[8] = 1.0;
			linearsolver.Release();

			return new Matrix3d(h);
		}
		public static PolyProxy FindBoundingBox(List<PolyProxy> boxes)
		{
			List<Vector3d> pts3 = new List<Vector3d>();
			foreach (PolyProxy box in boxes)
			{
				foreach (Point3 pt in box.points3)
					pts3.Add(pt.pos);
			}

			return FindBoundingBox(pts3);
		}
		public static PolyProxy FindBoundingBox(List<Vector3d> points3)
		{
			int vertexCount = points3.Count;

			Vector3d maxCoord = new Vector3d(double.MinValue, double.MinValue, double.MinValue);
			Vector3d minCoord = new Vector3d(double.MaxValue, double.MaxValue, double.MaxValue);
			for (int i = 0; i < vertexCount; i++)
			{
				Vector3d v = points3[i];
				maxCoord = Vector3d.Max(maxCoord, v);
				minCoord = Vector3d.Min(minCoord, v);
			}

			// the other six points
			Vector3d u1 = new Vector3d(minCoord.x, maxCoord.y, minCoord.z);
			Vector3d u2 = new Vector3d(maxCoord.x, minCoord.y, minCoord.z);
			Vector3d u3 = new Vector3d(maxCoord.x, maxCoord.y, minCoord.z);
			Vector3d v1 = new Vector3d(minCoord.x, maxCoord.y, maxCoord.z);
			Vector3d v2 = new Vector3d(maxCoord.x, minCoord.y, maxCoord.z);
			Vector3d v3 = new Vector3d(minCoord.x, minCoord.y, maxCoord.z);

			Point3[] botpts = new Point3[4] {
				new Point3(minCoord),
				new Point3(u1),
				new Point3(u3),
				new Point3(u2),
			};

			Point3[] toppts = new Point3[4] {
				new Point3(v3),
				new Point3(v1),
				new Point3(maxCoord),
				new Point3(v2),
			};

			Polygon top = new Polygon(toppts);
			Polygon bot = new Polygon(botpts);

			PolyProxy boundingBox = new PolyProxy(bot, top);

			return boundingBox;
		}
		public static bool check(int[] a, int[] b, int n, int m)
		{
			b = new int[b.Length];
			for (int i = 0; i < n; ++i)
				b[a[i]]++;
			for (int i = 0; i < m; ++i)
				if (b[i] == 0)
					return false;
			return true;
		}
		public static void dfs(int step, int k, int[] a, int[] b, int n, int m, List<List<List<int>>> groups)
		{
			if (step == n)
			{
				if (check(a, b, n, m))
				{
					List<List<int>> grp = new List<List<int>>();
					for (int i = 0; i < m; ++i)
					{
						List<int> g = new List<int>();
						for (int j = 0; j < n; ++j)
						{
							if (a[j] == i)
							{
								g.Add(j);
							}
						}
						grp.Add(g);
					}
					groups.Add(grp);
				}
				return;
			}
			for (int i = k; i < m; ++i)
			{
				a[step] = i;
				dfs(step+1,k,a,b,n,m, groups);
			}
		}
		
		public static void output(int k, List<List<int>> v, List<List<int[]>> results)
		{
			List<int[]> groups = new List<int[]>();
			for (int i = 0; i < k; ++i) {
				groups.Add(v[i].ToArray());
			}
			results.Add(groups);
		}
		public static void zju_dfs(int n, int k, int ind, int ngroup, List<List<int>> v, List<List<int[]>> results)
		{
			if (ind >= n) {
				output(k, v, results);
			}
			if (ngroup < k) {
				v[ngroup].Add(ind);
				zju_dfs(n, k, ind + 1, ngroup + 1,v,results);
				v[ngroup].RemoveAt(v[ngroup].Count-1);
			}
			if (ngroup + n - ind > k) {
				for (int i = 0; i < ngroup; ++i) {
					v[i].Add(ind);
					zju_dfs(n, k, ind + 1, ngroup, v, results);
					v[i].RemoveAt(v[i].Count-1);
				}
			}
		}
		public static void gao(int n, int k, List<List<int>> v, List<List<int[]>> results)
		{
			for (int i = 0; i < k; ++i) {
				v.Add(new List<int>());
			}
			zju_dfs(n, k, 0, 0, v, results);
		}
		public static int division(int n, int k, out List<List<int[]>> results)
		{
			List<List<int>> v = new List<List<int>>();
			
			results = new List<List<int[]>>();

			gao(n, k, v, results);

			return 0;
		}

		// select from each of the arrays, form a combination (correspondence)
		// this is the same as for an array A[1,2,..,n], each A[i] has k_i values to choose
		// enumerate through all possible combinations, the complexity if k_1*k_2*...*k_n.
		// bArray should have size of n, each entry k is the limit of an index, i.e., k_i, a[] is a tmp array
		public static void perms(int[] a, int[] bArray, int n, int temp, List<int[]> _perms)
		{
			if (temp < n)
			{
				for (int j = 0; j < bArray[temp];)
				{
					perms(a, bArray, n, temp+1, _perms);

					a[temp] = ++j;
					if (a[temp] == bArray[temp])
						a[temp] = 0;

				}
			}
			else
			{
				_perms.Add(a.Clone() as int[]);
			}
		}

		// select n elements out of N elements, n <= N, b[] is array of index -- temparary use only
		public static void combs(int N, int K,  int[] b, List<int[]> _combs)
		{ 
			for(int i=N; i>=K; i--)
			{
				b[K-1] = i - 1;
				if (K > 1)
					combs(i-1,K-1,b,_combs);
				else
				{
					_combs.Add(b.Clone() as int[]);
				}
			}
		}

		// permuation of an array of size a[N]
		public static void permutation(int[] a, int k, List<int[]> _perms)
		{
			int j;  
			if(k == a.Length)  
			{  
				_perms.Add(a.Clone() as int[]); 
			}  
			else  
			{  
				for(j=k;j<a.Length;j++)  
				{  
					swap(a,j,k);  
					permutation(a,k+1,_perms);  
					swap(a,j,k);  
				}  
			}
		}
		public static void swap(int[] b, int i, int j)
		{
			int tmp = b[i];
			b[i] = b[j];
			b[j] = tmp;
		}

		public static void FlipMatrixRows(double[,] mat)
		{
			// flip the matrix row i becomes row n-i
			int nrows = mat.GetLength(0);
			for (int i = 0; i < nrows/2; ++i)
			{
				SwapMatrixRows(mat, i, nrows-i-1);
			}
		}
		public static double[,] GetSubMatrix(double[,] mat, int[] array)
		{
			// sort the matrix by the specified array, array contains the new indices of each row
			// the mat is assumed to be symmetric, NxN
			// the output is also symmetric
			int N = array.Length;
			double[,] mat2 = new double[N,N];
			for (int i = 0; i < N; ++i)
			{
				for (int j = 0; j < N; ++j)
				{
					int I = array[i], J = array[j];
					mat2[i,j] = mat[I,J];
				}
			}
			return mat2;
		}
		public static void SwapMatrixRows(double[,] mat, int k, int l)
		{
			// swap the k-th row with l-th row of matirx mat
			int N = mat.GetLength(0);
			for (int i = 0; i < N; ++i) // swap rows
			{
				double tmp = mat[i, k];
				mat[i, k] = mat[i, l];
				mat[i, l] = tmp;
			}
		}
		public static void SwapMatrixRowCols(double[,] mat, int k, int l)
		{
			// swap the k-th row with l-th row of matirx mat
			int N = mat.GetLength(1);
			for (int i = 0; i < N; ++i) // swap cols
			{
				double tmp = mat[i, k];
				mat[i,k] = mat[i,l];
				mat[i, l] = tmp;
			}
			for (int i = 0; i < N; ++i) // swap rows
			{
				double tmp = mat[k, i];
				mat[k, i] = mat[l,i];
				mat[l, i] = tmp;
			}
		}
		public static double[,] MultiplyMat(double[,] mat, double[,] mat2)
		{
			// multiply mat with mat2, results are stored in res
			int M = mat.GetLength(0), N = mat.GetLength(1), L = mat2.GetLength(1);
			if (N != mat2.GetLength(0)) throw new Exception();
			double[,] res = new double[M, L];
			for (int i = 0; i < M; ++i)
			{
				for (int j = 0; j < L; ++j)
				{
					double sum = 0;
					for (int k = 0; k < N; ++k)
					{
						sum += mat[i, k] * mat2[k, j];
					}
					res[i,j] = sum;
				}
			}
			return res;
		}

		public static void NormalizeData(double[,] data)
		{
			double min = double.MaxValue;
			double max = double.MinValue;
			int M = data.GetLength(0), N = data.GetLength(1);
			for (int i = 0; i < M; ++i)
			{
				for (int j = 0; j < N; ++j)
				{
					double val = data[i, j];
					if (max < val) max = val;
					if (min > val) min = val;
				}
			}
			if (Math.Abs(max - min) < 1e-5)
			{
				Program.OutputText("max == min, normalization failed", true);
				return;
			}
			for (int i = 0; i < M; ++i)
			{
				for (int j = 0; j < N; ++j)
				{
					data[i, j] = (data[i, j] - min) / (max - min);
				}
			}
		}
		public static void NormalizeData(double[][] data)
		{
			double min = double.MaxValue;
			double max = double.MinValue;
			int M = data.GetLength(0), N = data[0].GetLength(0);
			for (int i = 0; i < M; ++i)
			{
				for (int j = 0; j < N; ++j)
				{
					double val = data[i][j];
					if (max < val) max = val;
					if (min > val) min = val;
				}
			}
			for (int i = 0; i < M; ++i)
			{
				for (int j = 0; j < N; ++j)
				{
					data[i][j] = (data[i][j] - min) / (max - min);
				}
			}
		}
		public static double NormalizeData(double[] dat)
		{
			double sum = 0;
			for (int i = 0; i < dat.Length; ++i)
			{
				sum += dat[i] * dat[i];
			}
			sum = Math.Sqrt(sum);
			for (int i = 0; i < dat.Length; ++i)
			{
				dat[i] = dat[i] / sum;
			}
			return sum;
		}
		public static void LinearizeDataTo01(double[] dat)
		{
			double max = double.MinValue;
			double min = double.MaxValue;
			for (int i = 0; i < dat.Length; ++i)
			{
				double d = dat[i];
				if (d > max) max = d;
				if (d < min) min = d;
			}
			double dd = max - min;
			if (dd < 1e-6) return;
			for (int i = 0; i < dat.Length; ++i)
			{
				dat[i] = (dat[i]-min) / dd;
			}
		}
		public static double[] ComputeD2Descriptor(List<Vector3d> samplepoints, int dim)
		{
			int N = samplepoints.Count;
			List<double> _dis_mat = new List<double>();
			for (int i = 0; i < N - 1; ++i)
			{
				Vector3d p = samplepoints[i];
				for (int j = i + 1; j < N; ++j)
				{
					Vector3d q = samplepoints[j];
					double dis = (q - p).Length();
					_dis_mat.Add(dis);
				}
			}
			double[] data = _dis_mat.ToArray();


			LinearizeDataTo01(data);


			double itval = 1.0 / dim;
			double[] _descriptor = new double[dim];
			foreach (double dist in data)
			{
				int J = (int)(dist / itval);
				if (J >= dim) J = dim - 1;
				_descriptor[J] += 1.0;
			}

			NormalizeData(_descriptor);

			return _descriptor;

		}

		// point to box distance (box is axis-aligned)
		public static double Point2BoxDistance(Vector3d point, Vector3d box_center, Vector3d box_scale)
		{
			Vector3d r = point - box_center;

			double dx = Math.Sign(r.x) * (r.x - Math.Sign(r.x) * box_scale.x);
			double dy = Math.Sign(r.y) * (r.y - Math.Sign(r.y) * box_scale.y);
			double dz = Math.Sign(r.z) * (r.z - Math.Sign(r.z) * box_scale.z);

			return Math.Abs(Math.Max(dx, Math.Max(dy, dz)));
		}

		// point to box distance (box is oriented)
		public static double Point2BoxDistance(Vector3d point, Vector3d box_center, Vector3d box_scale, Vector3d axis_x, Vector3d axis_y, Vector3d axis_z)
		{
			Vector3d r = point - box_center;

			double rx = r.Dot(axis_x);
			double ry = r.Dot(axis_y);
			double rz = r.Dot(axis_z);
			double dx = Math.Sign(rx) * (rx - Math.Sign(rx) * box_scale.x);
			double dy = Math.Sign(ry) * (ry - Math.Sign(ry) * box_scale.y);
			double dz = Math.Sign(rz) * (rz - Math.Sign(ry) * box_scale.z);

			return Math.Abs(Math.Max(dx, Math.Max(dy, dz)));
		}

		// randomly select k elements from N elements (A)
		public static int[] ReservoirSampling(int[] A, int k)
		{
			int N = A.Length;

			int[] R = new int[k];
			int i, j;
			for (i = 0; i < k; ++i)
				R[i] = A[i];

			Random random = new Random();
			for (i = k; i < N; ++i)
			{
				j = random.Next(0, i-1);
				if (j < k)
					R[j] = A[i];
			}

			return R;
		}

		/// distance from p to line qr in 3d
		public static double Dist2Line(Vector3d p, Vector3d q, Vector3d r)
		{
			Vector3d u = q - p;
			Vector3d v = r - p;
			double s = u.Cross(v).Length();
			return s / (q - r).Length();
		}
		/// obtain reflection vector for any give vector
		public static Vector3d ReflectionVector(Vector3d u, Vector3d planeNormal)
		{
			Matrix4d R = ReflectionalMatrix(planeNormal); // n is the plane normal
			return (R * new Vector4d(u, 0)).XYZ();
		}
		/// update axis of p according to the new principle axis
		public static void UpdateProxyAxis(Proxy p, Vector3d newAxis, int whichAxis)
		{
			Vector3d axis = p.z;

			if (whichAxis == 0)
			{
				axis = p.x;
			}
			else if (whichAxis == 1)
			{
				axis = p.y;
			}

			double cos = axis.Dot(newAxis);

			if (cos < 0)
			{
				newAxis = new Vector3d() - newAxis;
				cos = -cos;
			}

			if (cos > 1) return;

			Vector3d rot_axis = axis.Cross(newAxis).Normalize();

			if (double.IsNaN(rot_axis.x)) return;

			double rot_angle = Math.Acos(Math.Abs(cos));
			Matrix4d R = Matrix4d.RotationMatrix(rot_axis, rot_angle);


			p.Axis = (R * new Vector4d(p.Axis, 0)).XYZ();
			p.PrinAxis1 = (R * new Vector4d(p.PrinAxis1, 0)).XYZ();
			p.PrinAxis2 = (R * new Vector4d(p.PrinAxis2, 0)).XYZ();

		}
		
		// get up rotational axis and rotational angle (from uAxis to vAxis)
		public static void GetRotationalAxisAngle(Vector3d uAxis, Vector3d vAxis, out Vector3d rot_axis, out double rot_angle)
		{
			Vector3d uA = uAxis.Normalize();
			Vector3d uB = vAxis.Normalize();
			rot_axis = uA.Cross(uB).Normalize();
			if (double.IsNaN(rot_axis.x) || double.IsNaN(rot_axis.y) || double.IsNaN(rot_axis.z))
			{
				rot_axis = new Vector3d(0, 0, 1);
				rot_angle = 0;
				return;
			}
			double cos = uA.Dot(uB);
			if (cos > 1) cos = 1;
			if (cos < -1) cos = -1;
			rot_angle = Math.Acos(cos);
		}
	}

	public sealed class GLUtil
	{
		public static void drawSphere(Vector3d p, double radius, float[] material)
		{
			Gl.glEnable(Gl.GL_LIGHTING);
			Gl.glEnable(Gl.GL_NORMALIZE);

			Gl.glMaterialfv(Gl.GL_FRONT_AND_BACK, Gl.GL_DIFFUSE, material);

			Glu.GLUquadric quad = Glu.gluNewQuadric();

			Gl.glPushMatrix();
			Gl.glTranslated(p.x, p.y, p.z);
			Glu.gluSphere(quad, radius, 40, 40);
			Gl.glPopMatrix();

			Glu.gluDeleteQuadric(quad);

			Gl.glDisable(Gl.GL_LIGHTING);
			Gl.glDisable(Gl.GL_NORMALIZE);
		}
		public static void drawCylinder(Vector3d u, Vector3d v, double radius, float[] material)
		{
			Gl.glEnable(Gl.GL_LIGHTING);
			Gl.glEnable(Gl.GL_NORMALIZE);

			double len1 = (v - u).Length();
			Vector3d dir = (v - u).Normalize();

			double r1 = radius, r2 = r1 / 2;
			double angle = Math.Acos(dir.z / dir.Length()) * 180 / Math.PI;


			Gl.glMaterialfv(Gl.GL_FRONT_AND_BACK, Gl.GL_DIFFUSE, material);


			Glu.GLUquadric quad = Glu.gluNewQuadric();

			Gl.glPushMatrix();
			Gl.glTranslated(u.x, u.y, u.z);
			Glu.gluSphere(quad, r2, 40, 40);
			Gl.glPopMatrix();


			Gl.glPushMatrix();
			Gl.glTranslated(u.x, u.y, u.z);
			Gl.glRotated(angle, -dir.y, dir.x, 0);
			Glu.gluCylinder(quad, r1, r1, len1, 40, 40);
			Gl.glPopMatrix();

			Gl.glPushMatrix();
			Gl.glTranslated(v.x, v.y, v.z);
			Gl.glRotated(angle, -dir.y, dir.x, 0);
			Glu.gluSphere(quad, r2, 40, 40);
			Gl.glPopMatrix();

			Glu.gluDeleteQuadric(quad);

			Gl.glDisable(Gl.GL_LIGHTING);
			Gl.glDisable(Gl.GL_NORMALIZE);
		}
	}
	  
	// contacts that connecting components
	public class Contact
	{
		public bool updated = false;
		public Vector3d oldpos, pos;
		public List<MeshComponent> conntectingComponents = null; // in regular case, should be 2 components
		// connected node type + edge Index (orientation order)
		public Tuple<int, int>[] edgeType = new Tuple<int,int>[2];// each edge only has two connecting components/type
		public DualEdge[] dualEdges = new DualEdge[2];
		public List<Contact> adjContacts = null;


		public Contact(Vector3d _pos)
		{
			this.oldpos = this.pos = _pos;
		}
		public void AddComponent(MeshComponent component)
		{
			if (this.conntectingComponents == null)
			{
				this.conntectingComponents = new List<MeshComponent>();
			}
			if (!this.conntectingComponents.Contains(component))
			{
				this.conntectingComponents.Add(component);
				component.AddContact(this);
			}
		}
		public void UpdatePos()
		{
			this.oldpos = this.pos;
		}
		public void RestorePos()
		{
			this.pos = this.oldpos;
		}
		public void Transform(Matrix4d Q)
		{
			Vector4d _pos = new Vector4d(this.oldpos, 1);
			this.pos = (Q * _pos).XYZ();
			this.updated = true;
		}
		public void TransformCurrent(Matrix4d Q)
		{
			Vector4d _pos = new Vector4d(this.pos, 1);
			this.pos = (Q * _pos).XYZ();
			this.updated = true;
		}
		public void Transform(Matrix4d Q, Vector3d center)
		{
			Vector4d _pos = new Vector4d(this.oldpos - center, 1);
			this.pos = (Q * _pos).XYZ() + center;
		}
		public void TransformCurrent(Matrix4d Q, Vector3d center)
		{
			Vector4d _pos = new Vector4d(this.pos - center, 1);
			this.pos = (Q * _pos).XYZ() + center;
		}
	}
	
	public class CoordinateFrame
	{
		public Vector3d old_o, old_x, old_y, old_z;
		public Vector3d o, x, y, z;
		public Matrix3d XYZFrame = null, oldXYZFrame = null;
		public CoordinateFrame(Vector3d o_, Vector3d x_, Vector3d y_, Vector3d z_)
		{
			this.old_o = this.o = o_;
			this.old_x = this.x = x_;
			this.old_y = this.y = y_;
			this.old_z = this.z = z_;
			this.oldXYZFrame = new Matrix3d(x_,y_,z_);
			this.XYZFrame = new Matrix3d(x_, y_, z_);
		}
		
		public Vector3d GetPointLocalCoord(Vector3d p)
		{
			Vector3d po = (p - this.o);
			return new Vector3d(po.Dot(this.x), po.Dot(this.y), po.Dot(this.z));
		}
		public Vector3d GetPointSpaceCoord(Vector3d p)
		{
			return this.XYZFrame * p + this.o;
		}
		public Vector3d PointAtCoord(Vector2d coord)
		{
			return this.XYZFrame * new Vector3d(coord, 0) + this.o;
		}

		public void TransformOld(Matrix4d T)
		{
			this.o = (T * new Vector4d(this.old_o, 1)).XYZ();
			this.x = (T * new Vector4d(this.old_x, 0)).XYZ().Normalize();
			this.y = (T * new Vector4d(this.old_y, 0)).XYZ().Normalize();
			this.z = (T * new Vector4d(this.old_z, 0)).XYZ().Normalize();
			this.XYZFrame = new Matrix3d(this.x, this.y, this.z);
		}
		public void Update()
		{
			this.old_o = this.o;
			this.old_x = this.x;
			this.old_y = this.y;
			this.old_z = this.z;
			this.oldXYZFrame = new Matrix3d(this.XYZFrame);
		}
	}
	public class Point2 : ICloneable
	{
		public bool occluded = false;
		public int index = -1;
		public Vector2d pos, oldpos;
		public Vector2d texcoord;
		public Point2(Vector2d pt)
		{
			this.oldpos = this.pos = pt;
			this.texcoord = this.pos;
		}
		public Point2()
		{
			this.oldpos = this.pos = new Vector2d();
			this.texcoord = this.pos;
		}
		public void Translate(Vector2d off)
		{
			this.pos = this.oldpos + off;
		}
		public void UpdateOldPos()
		{
			this.oldpos = this.pos;
		}
		public object Clone()
		{
			Point2 another = new Point2(this.pos);
			another.index = this.index;
			another.texcoord = this.texcoord;
			another.occluded = this.occluded;
			return another;
		}
	}
	public class Point3 : ICloneable
	{
		public int index = -1;
		public List<Point3> symmpoints = null;
		public Matrix3d R = null; // local frame encoding for manipulation, R does not change unless rotated
		public Vector3d pos, oldpos;
		public Vector3d normal;
		public Point3(Vector3d pt)
		{
			this.oldpos = this.pos = pt;
		}
		public Point3()
		{
			this.pos = new Vector3d();
		}
		public void UpdateOldPos()
		{
			this.oldpos = this.pos;
		}
		public object Clone()
		{
			Point3 another = new Point3(this.pos);
			if (this.R != null)
				another.R = new Matrix3d(this.R);
			another.index = this.index;
			another.normal = this.normal;
			return another;
		}
	}
	public class LineSegment : IComparable<LineSegment>
	{
		public Point2 u, v;
		public Point3 u3 = null, v3 = null;
		public int vanishingindex = -1;			// which vanishing direction it corresponds
		public double length = 0;
		public double fiterror = 0;
		public LineSegment(Point2 _u, Point2 _v)
		{
			this.u = _u;
			this.v = _v;
			this.vanishingindex = -1;
			this.length = this.GetLength();
		}
		public LineSegment(Point2 _u, Point2 _v, int _i)
		{
			this.u = _u;
			this.v = _v;
			this.vanishingindex = _i;
			this.length = this.GetLength();
		}
		public void Translate(Vector2d off)
		{
			this.u.Translate(off);
			this.v.Translate(off);
		}
		public double GetLength()
		{
			return (this.v.pos - this.u.pos).Length();
		}
		#region IComparable<LineSegment> Members
		public int CompareTo(LineSegment other)
		{
			return Math.Sign(length - other.length);
		}

		#endregion
	}
	public class Shape2D
	{
		public Vector2d center2;
		public Vector3d normal;
		public double radius2 = 0;
		public int textureindex = -1;
		public bool visible = true;
		public ShapeType2D shaptype = ShapeType2D.NONE;
		public List<Point2> points = null;
		public List<int> boxptindice = null;
		public List<LineSegment> linesegments = new List<LineSegment>();
		public Shape2D()
		{
		}
		public void AddPoint(Point2 pt)
		{
			if (this.points == null)
			{
				this.points = new List<Point2>();
				this.linesegments = new List<LineSegment>();
			}
			this.points.Add(pt);
			int n = this.points.Count;
			if (n < 2) return;
			this.linesegments.Add(
				new LineSegment(this.points[n - 2], this.points[n - 1])
			);
		}
		public void Close()
		{
			this.linesegments.Add(
				new LineSegment(this.points[this.points.Count - 1], this.points[0])
			);
			this.ComputeCenter();
		}
		public void ComputeCenter()
		{
			Vector2d c = new Vector2d();
			foreach (Point2 p in this.points)
			{
				c += p.pos;
			}
			this.center2 = c / this.points.Count;
		}
		public void ComputeRadius()
		{
			Vector2d c = new Vector2d();
			foreach (Point2 p in this.points)
			{
				c += p.pos;
			}
			this.center2 = c / this.points.Count;
			double sum = 0;
			foreach (Point2 p in this.points)
			{
				sum += (p.pos - center2).Length();
			}
			this.radius2 = sum / this.points.Count;
		}

		public static bool PointInPoly(Vector2d p, Shape2D shape)
		{
			return PointInPoly(p, shape.points);
		}
		public static bool PointInPoly(Vector2d p, List<Point2> points)
		{
			bool c = false;
			int n = points.Count;
			for (int i = 0, j = n - 1; i < n; j = i++)
			{
				if (((points[i].pos.y > p.y) != (points[j].pos.y > p.y)) &&
					(p.x < (points[j].pos.x - points[i].pos.x) * (p.y - points[i].pos.y) / (points[j].pos.y - points[i].pos.y) + points[i].pos.x))
					c = !c;
			}
			return c;
		}
		public static bool PointInPoly(Vector2d p, Point2[] points)
		{
			bool c = false;
			int n = points.Length;
			for (int i = 0, j = n - 1; i < n; j = i++)
			{
				if (((points[i].pos.y > p.y) != (points[j].pos.y > p.y)) &&
					(p.x < (points[j].pos.x - points[i].pos.x) * (p.y - points[i].pos.y) / (points[j].pos.y - points[i].pos.y) + points[i].pos.x))
					c = !c;
			}
			return c;
		}
		public static bool PointInPoly(Vector2d p, Vector2d[] shape)
		{
			int n = shape.Length;
			Vector2d o = shape[0];
			for (int i = 1; i < n - 1; ++i)
			{
				if (PointInTriangle(p, o, shape[i], shape[i + 1]))
					return true;
			}
			return false;
		}
		//public static bool PointInPoly(Vector2d p, Shape2D shape)
		//{
		//    int n = shape.points.Count;
		//    Vector2d o = shape.points[0].pos;
		//    for (int i = 1; i < n-1; ++i)
		//    {
		//        if (PointInTriangle(p,o,shape.points[i].pos,shape.points[i+1].pos))
		//            return true;
		//    }
		//    return false;
		//}
		public static bool PointInTriangle(Vector2d p, Vector2d a, Vector2d b, Vector2d c)
		{
			if (SameSide(p, a, b, c) && SameSide(p, b, a, c) && SameSide(p, c, a, b))
				return true;
			return false;
		}
		public static bool SameSide(Vector2d p1, Vector2d p2, Vector2d a, Vector2d b)
		{
			Vector3d A = new Vector3d(a);
			Vector3d B = new Vector3d(b);
			Vector3d P = new Vector3d(p1);
			Vector3d Q = new Vector3d(p2);
			Vector3d E = B - A;
			Vector3d cp1 = E.Cross(P - A);
			Vector3d cp2 = E.Cross(Q - A);
			if (cp1.Dot(cp2) >= 0)
				return true;
			return false;
		}
	}
	public class Quad : Shape2D
	{
		public int vanishingindex = -1;
		public Point3[] points3 = new Point3[4];
		public Vector3d center;
		public Quad()
		{
			this.shaptype = ShapeType2D.QUAD;
		}
		public Quad(Vector2d a, Vector2d b, Vector2d c, Vector2d d)
		{
			this.shaptype = ShapeType2D.QUAD;
			this.points = new List<Point2>();
			this.points.Add(new Point2(a));
			this.points.Add(new Point2(b));
			this.points.Add(new Point2(c));
			this.points.Add(new Point2(d));
		}
		public Quad(Point2 a, Point2 b, Point2 c, Point2 d)
		{
			this.shaptype = ShapeType2D.QUAD;
			this.points = new List<Point2>();
			this.points.Add(a);
			this.points.Add(b);
			this.points.Add(c);
			this.points.Add(d);
		}
		public Quad(Point2[] pts)
		{
			this.shaptype = ShapeType2D.QUAD;
			this.points = new List<Point2>();
			this.points.AddRange(pts);
		}
	}
	public class Polygon : Shape2D, ICloneable
	{
		public Polygon opposite = null;
		public Polygon assignedpolygon = null;
		public Vector2d[] assignedtextures = new Vector2d[4] {
			new Vector2d(0,0),
			new Vector2d(1,0),
			new Vector2d(1,1),
			new Vector2d(0,1)
		};
		public uint[] textid = null;

		public static double propagationpointdensity = 0;
		public CoordinateFrame localframe = null;
		public Matrix3d H = null; // homography matrix
		public Point3 center;
		public int prevattachindex = -1, attachindex = -1;
		public double depth = 0;
		public double sliceratio = 0; // for decomposition of a box
		public int sliceindex = -1; // for decomposition of a box
		public int index = -1;
		public int globalindex = -1;
		public int occludcount = 0;
		public PolyProxy hostproxy = null;
		public List<Polygon> hostpolygons = new List<Polygon>(); // the polygons that contains it
		public List<Polygon> mirrorsymmetries = null;

		public List<int> groupindex = new List<int>();
		public bool visited = false;
		public List<Point3> points3 = new List<Point3>();
		public List<Point3> propagatepoints3 = null; // use for propagation, much denser than the original points
		public List<int> propagateKs = null; // use for propagation
		public Polygon()
		{
			this.shaptype = ShapeType2D.POLYGON;
		}
		public Polygon(List<Point2> pts)
		{
			this.shaptype = ShapeType2D.POLYGON;
			this.points = pts;
			this.linesegments = new List<LineSegment>();
			int n = pts.Count;
			for (int i = 0; i < n; ++i)
			{
				this.linesegments.Add(
					new LineSegment(pts[i], pts[(i + 1) % n])
				);
			}
			this.ComputeCenter();
		}
		public Polygon(Point2[] pts)
		{
			this.shaptype = ShapeType2D.POLYGON;

			if (pts == null) return;

			this.points = new List<Point2>();
			this.points.AddRange(pts);
			this.linesegments = new List<LineSegment>();
			int n = pts.Length;
			for (int i = 0; i < n; ++i)
			{
				this.linesegments.Add(
					new LineSegment(pts[i], pts[(i + 1) % n])
				);
			}
			this.ComputeCenter();
		}
		public Polygon(Point3[] pts)
		{
			this.shaptype = ShapeType2D.POLYGON;
			this.points3.AddRange(pts);
		}
		public void Compute()
		{
			this.ComputeCenter3();
			this.ComputeNormal();
		}
		public void ComputeCenter3()
		{
			Vector3d cc = new Vector3d();
			foreach (Point3 p in this.points3)
			{
				cc += p.pos;
			}
			cc /= this.points3.Count;
			if (this.center == null)
				this.center = new Point3(cc);
			else
				this.center.pos = cc;
		}
		public void ComputeNormal()
		{
			if (this.points3.Count < 2) return;
			this.normal = (this.points3[0].pos - this.points3[1].pos).Cross(
					this.points3[2].pos - this.points3[1].pos).Normalize();
		}
		public void ComputeLocalFrame()
		{
			if (this.points3 == null) return;
			Vector3d o = this.points3[0].pos;
			Vector3d u = this.points3[1].pos;
			Vector3d v = this.points3[this.points3.Count-1].pos;
			Vector3d z = (v - o).Cross(u - o).Normalize();
			if (double.IsNaN(z.x)) z = new Vector3d(0, 0, 1);
			Vector3d x = (u - o).Normalize();
			Vector3d y = z.Cross(x).Normalize();
			this.localframe = new CoordinateFrame(o, x, y, z);
		}
		public void ComputeLinesegmentsP3()
		{
			int i = 0, n = this.points3.Count;
			foreach (LineSegment line in this.linesegments)
			{
				line.u3 = this.points3[i];
				line.v3 = this.points3[(i + 1) % n];
				i++;
			}
		}
		public void CreatePropagatingPoints()
		{
			this.propagatepoints3 = new List<Point3>();
			this.propagateKs = new List<int>();
			int n = this.points3.Count;
			for (int i = 0; i < n; ++i)
			{
				Vector3d p = this.points3[i].pos, q = this.points3[(i + 1) % n].pos;
				double l = (p - q).Length();
				int K = (int)(l / propagationpointdensity);
				if (K < 2)
				{
		//			Program.OutputText("@CreatePropagatingPoints, K < 1",true);
					K = 2;
				}
		//		if (K > 10) K = 10;
				this.propagateKs.Add(K);
				double step = 1.0 / (K - 1);
				for (int j = 0; j < K; ++j)
				{
					double r = step * j;
					Vector3d pos = q * r + p * (1 - r);
					this.propagatepoints3.Add(new Point3(pos));
				}
			}
		}
		public void UpdatePropagatingPoints()
		{
			int n = this.points3.Count; int index = 0;
			for (int i = 0; i < n; ++i)
			{
				Vector3d p = this.points3[i].pos, q = this.points3[(i + 1) % n].pos;
				double l = (p - q).Length();
				int K = this.propagateKs[i];
				double step = 1.0 / (K - 1);
				for (int j = 0; j < K; ++j)
				{
					double r = step * j;
					Vector3d pos = q * r + p * (1 - r);
					this.propagatepoints3[index++].pos = pos;
				}
			}
		}
		public void PropagationUpdate()
		{
			this.ComputeCenter3();
			this.UpdatePropagatingPoints();
		}
		public object Clone()
		{
			List<Point3> points3 = new List<Point3>();
			foreach (Point3 p in this.points3)
			{
				points3.Add(p.Clone() as Point3);
			}
			Polygon another = new Polygon();
			another.points3 = points3;

			// normals & textures
			another.normal = this.normal;
			another.textureindex = this.textureindex;
			if (this.textid != null)
				another.textid = this.textid;

			// centers, points3 & propagating points
			if (this.center != null) another.center = this.center.Clone() as Point3;
			if (this.points3.Count > 0)
			another.ComputeLinesegmentsP3();
			another.CreatePropagatingPoints();
			return another;
		}

		public void Transform(Matrix4d tran, Vector3d c)
		{
			Vector4d cc = new Vector4d(c, 1);
			for (int i = 0; i < this.points3.Count; ++i)
			{
				Vector3d pos = this.points3[i].pos;
				Vector4d v = new Vector4d(pos, 1.0);
				v = tran * (v - cc) + cc;
				this.points3[i].pos[0] = v.x;
				this.points3[i].pos[1] = v.y;
				this.points3[i].pos[2] = v.z;
			}
			this.Compute();
		}
		public void Transform(Matrix4d tran)
		{
			for (int i = 0; i < this.points3.Count; ++i)
			{
				Vector3d pos = this.points3[i].pos;
				Vector4d v = new Vector4d(pos, 1.0);
				v = tran * v;
				this.points3[i].pos[0] = v.x;
				this.points3[i].pos[1] = v.y;
				this.points3[i].pos[2] = v.z;
			}
			this.Compute();
		}

		public static Polygon CreateCircle(int nslices, double zcoord)
		{
			double delta_theta = Math.PI * 2 / nslices;
			List<Point3> points3 = new List<Point3>();
			for (int i = 0; i < nslices; ++i)
			{
				double t = delta_theta * i;
				double x = 1.0 * Math.Cos(t);
				double y = 1.0 * Math.Sin(t);
				points3.Add(new Point3(
					new Vector3d(x, y, zcoord)
					)
				);
			}
			return new Polygon(points3.ToArray());
		}
	}

	public class PolyProxy : ICloneable, IComparable<PolyProxy>
	{
		public List<int> enclosingindex = null; // currently not used any more
		public List<Vector2d> srctxtpos = null;

		// mesh component adapt to it
		public Mesh mesh = null;

		public ProxyType type = ProxyType.CUBOID;

		public bool isfixed = false;
		public bool deleted = false;
		public bool occuluded = false;
		public PolyProxy jointproxy = null;
		public List<PolyProxy> mirrorsymmetry = new List<PolyProxy>();

		public void Transform(Matrix4d tran, Vector3d c)
		{
			Vector4d cc = new Vector4d(c, 1);
			for (int i = 0; i < this.points3.Count; ++i)
			{
				Vector3d pos = this.points3[i].pos;
				Vector4d v = new Vector4d(pos, 1.0);
				v = tran * (v-cc) + cc;
				this.points3[i].pos[0] = v.x;
				this.points3[i].pos[1] = v.y;
				this.points3[i].pos[2] = v.z;
			}
			this.Compute3DInfo();
		}
		public void Transform(Matrix4d tran)
		{
			for (int i = 0; i < this.points3.Count; ++i)
			{
				Vector3d pos = this.points3[i].pos;
				Vector4d v = new Vector4d(pos, 1.0);
				v = tran * v;
				this.points3[i].pos[0] = v.x;
				this.points3[i].pos[1] = v.y;
				this.points3[i].pos[2] = v.z;
			}
			this.Compute3DInfo();
		}
		public void TransFromOld(Matrix4d tran, Vector3d c)
		{
			Vector4d cc = new Vector4d(c, 1);
			for (int i = 0; i < this.points3.Count; ++i)
			{
				Vector3d pos = this.points3[i].oldpos;
				Vector4d v = new Vector4d(pos, 1.0);
				v = tran * (v - cc) + cc;
				this.points3[i].pos[0] = v.x;
				this.points3[i].pos[1] = v.y;
				this.points3[i].pos[2] = v.z;
			}
			this.Compute3DInfo();
		}
		public void TransFromOld(Matrix4d tran)
		{
			// update vertex positions
			for (int i = 0; i < this.points3.Count; ++i)
			{
				Vector3d pos = this.points3[i].oldpos;
				Vector4d v = new Vector4d(pos, 1.0);
				v = tran * v;
				this.points3[i].pos[0] = v.x;
				this.points3[i].pos[1] = v.y;
				this.points3[i].pos[2] = v.z;
			}
			
			// update polygon centers
			this.Compute3DInfo();

		}
		public void Update3DPos()
		{
			this.center.UpdateOldPos();
			foreach (Point3 pt3 in this.points3)
				pt3.UpdateOldPos();
		}
		public void Update()
		{
			// update vertex position
			this.Update3DPos();
			// update polygons, scales, etc
			this.Compute3DInfo();
		}
		public double GetVolume()
		{
			return this.scale.x * this.scale.y * this.scale.z;
		}


		//  for MRF selection
		public int maxlevel = 0;
		public List<PolyProxy> kids = null;
		public void AddKid(PolyProxy px)
		{
			if (this.kids == null) this.kids = new List<PolyProxy>();
			this.kids.Add(px);
		}
		public void RemoveKid(PolyProxy px)
		{
			if (this.kids != null)
				this.kids.Remove(px);
		}
		public int groupindex = -1; // for inter-obejct propagation
		public int index = -1;
		public int segindex = -1;
		public int refindex = -1;
		public int tmpindex = -1;
		public int level = 0;
		public double fittingcost = 0;

		public List<Polygon> bendingslices = null; // for bending operation
		public List<PolyProxy> slicingboxes = null; // for bending operation

		public double xlen, ylen, zlen;
		public bool IntersectRoom(PolyProxy room)
		{
			double x = room.points3[0].pos.x;
			double y = room.points3[2].pos.y;
			double z = room.points3[4].pos.z;
			foreach (Point3 p in this.points3)
			{
				if (p.pos.x * x < 0 || Math.Abs(p.pos.x) > Math.Abs(x) + 1e-4 ||
					p.pos.y * y < 0 || Math.Abs(p.pos.y) > Math.Abs(y) + 1e-4 ||
					p.pos.z * z < 0 || Math.Abs(p.pos.z) > Math.Abs(z) + 1e-4)
					return true;
			}
			return false;
		}
		public bool Intersect(PolyProxy another)
		{
			foreach (Point3 p in this.points3)
			{
				if (another.Contains(p.pos))
					return true;
			}
			foreach (Point3 p in another.points3)
			{
				if (this.Contains(p.pos))
					return true;
			}
			return false;
		}
		public int IntersectPoly(PolyProxy another)
		{
			foreach (Point3 p in this.points3)
			{
				if (another.Contains(p.pos))
					return 1;
			}
			foreach (Point3 p in another.points3)
			{
				if (this.Contains(p.pos))
					return 2;
			}
			return 0;
		}
		// 2013-12-27
		public bool Intersect_new(PolyProxy another)
		{
			foreach (Point3 p in this.sampledPointsOnBox)
			{
				if (another.Contains(p.pos))
					return true;
			}
			foreach (Point3 p in another.sampledPointsOnBox)
			{
				if (this.Contains(p.pos))
					return true;
			}
			return false;
		}
		public CoordinateFrame frame = null;
		public bool Contains(Vector3d p)
		{
			Vector3d abc = this.frame.GetPointLocalCoord(p);
			double e1 = Math.Abs(abc.x) - this.xlen / 2;
			double e2 = Math.Abs(abc.y) - this.ylen / 2;
			double e3 = Math.Abs(abc.z) - this.zlen / 2;
			return  e1 < -1e-4 && e2 < -1e-4 && e3 < -1e-4;
		}

		public Vector2d[] vps = null;

		public int attachindex = 0;
		public int prevattachindex = 0;
		public bool visited = false;
		public double volume = -1;
		public double depth = 0;
		public Vector3d scale;


		public Point3 center;
		public Point2 center2;
		public List<int> hexagonindice = null;
		public List<Polygon> txtquads = null;
		public List<LineSegment> enclosingimglines = null;
		public Quad srcenclosingtxtquad = null; // texture mapping, source enclosing quad, for cylindric/sphere objects
		public Quad tarenclosingtxtquad = null; // texture mapping, target enclosing quad
		public List<Polygon> polygons = new List<Polygon>();
		public List<Point2> points = new List<Point2>();
		public List<Point3> points3 = new List<Point3>();
		// for intersection -- add 2013-12-27
		public List<Point3> sampledPointsOnBox = new List<Point3>();
		public List<Vector3d> sampledPointsOnBox_visual = new List<Vector3d>();

		public List<Point[]> objprofile = null;
		public PolyProxy onproxy = null; // object that it lies on
		public Polygon attachingFace = null;
		public MeshComponent meshComponent = null;
		public List<PolyProxy> onobjects = null; // objects that lies on it
		public void AddOnObject(PolyProxy px)
		{
			if (this.onobjects == null) this.onobjects = new List<PolyProxy>();
			this.onobjects.Add(px);
		}
		public PolyProxy() { }
		public PolyProxy(Point2[] pts, Point3[] pts3)
		{
			this.Create(pts, pts3);
		}
		public PolyProxy(List<Point2> pts, List<Point3> pts3)
		{
			this.Create(pts, pts3);
		}
		public PolyProxy(Polygon bot, Polygon top)
		{
			List<Point3> pts3 = new List<Point3>();
			pts3.AddRange(bot.points3);
			pts3.AddRange(top.points3);

			this.points3 = pts3;

			int index = 0;
			foreach (Point3 pt in this.points3)
				pt.index = index++;

			this.AddPolygon(bot);
			this.AddPolygon(top);

			int k = bot.points3.Count;
			for (int i = 0; i < k; ++i)
			{
				int j = i, l = (i + 1) % k;
				if (i > 1) { j = (i + 1) % k; l = i; }

				Point3[] points3 = new Point3[4] {
			        bot.points3[j],bot.points3[l], top.points3[l],top.points3[j],
			    };

				Polygon plg = new Polygon(points3);
				this.AddPolygon(plg);
			}

			this.type = ProxyType.POLY;

			this.EstablishLocalFrame();
			this.Compute3DInfo();

		}
		public PolyProxy(Vector3d c, Vector3d s, Vector3d o1, Vector3d o2, Vector3d o3)
		{
			// create a box based on center, scale, and 3 axes
			this.center = new Point3(c);
			this.scale = s;

			double[,] isgn = new double[8, 3] {
				{-1,-1,-1},
				{-1,1,-1},
				{1,1,-1},
				{1,-1,-1},
				{-1,-1,1},
				{-1,1,1},
				{1,1,1},
				{1,-1,1},
			};

			Matrix3d A = new Matrix3d(o1, o2, o3);
			Matrix3d T = Matrix3d.IdentityMatrix();
			List<Point3> pts3 = new List<Point3>();
			for (int i = 0; i < 8; ++i)
			{
				T[0, 0] = isgn[i, 0];
				T[1, 1] = isgn[i, 1];
				T[2, 2] = isgn[i, 2];
				Vector3d pt3 = A * T * s + c;
				pts3.Add(new Point3(pt3));
			}

			this.Create(null, pts3);

			this.center.pos = c;
			this.scale = s;

		}
		protected void Create(Point2[] pts, Point3[] pts3)
		{
			this.points.AddRange(pts);
			this.points3.AddRange(pts3);
			this.Make();
			this.AssignIndex();
		}
		protected void Create(List<Point2> pts, List<Point3> pts3)
		{
			this.points = pts;
			this.points3 = pts3;
			this.Make();
			this.AssignIndex();
		}
		public bool MakeBox()
		{
			int n = this.points.Count;
			if (n != 8) return false;
			this.Make();
			this.type = ProxyType.CUBOID;
			this.EstablishOpposite();
			this.EstablishHexagonIndices();
			return true;
		}
		public void Make()
		{
			// pts2 can be empty, pts3 must not be empty!
			if (this.points3 == null) throw new Exception();

			int n = this.points3.Count;
			int k = n / 2;

			this.polygons.Clear();

			Point3[] pts3 = new Point3[k];
			for (int i = 0; i < k; ++i)
			{
				pts3[i] = this.points3[i];
			}

			Point2[] pts2 = null;
			if (this.points != null)
			{
				pts2 = new Point2[k];
				for (int i = 0; i < k; ++i)
				{
					pts2[i] = this.points[i];
				}
			}

			Polygon bot = new Polygon(pts2);
			bot.points3.AddRange(pts3);

			if (this.points != null)
			{
				pts2 = new Point2[k];
				for (int i = 0; i < k; ++i)
				{
					pts2[i] = this.points[k + i];
				}
			}
			for (int i = 0; i < k; ++i)
			{
				pts3[i] = this.points3[k + i];
			}
			Polygon top = new Polygon(pts2);
			top.points3.AddRange(pts3);
			this.AddPolygon(bot);
			this.AddPolygon(top);

			for (int i = 0; i < k; ++i)
			{
				int j = i, l = (i + 1) % k;
				if (i > 1) { j = (i + 1) % k; l = i; }

				Point2[] points2 = null;
				if (this.points != null)
				{
					points2 = new Point2[4] {
						bot.points[j],bot.points[l], top.points[l],top.points[j],
					};
				}
				Point3[] points3 = new Point3[4] {
			        bot.points3[j],bot.points3[l], top.points3[l],top.points3[j],
			    };


				Polygon plg = new Polygon(points2);
				plg.points3.AddRange(points3);
				this.AddPolygon(plg);
			}

			this.type = ProxyType.POLY;
			this.Compute3DInfo();
		}
		//public void Build()
		//{
		//    int n = this.points.Count;
		//    int k = n / 2;
		//    this.polygons.Clear();
		//    Point2[] pts2 = new Point2[k];
		//    for (int i = 0; i < k; ++i) {
		//        pts2[i] = this.points[i];
		//    }
		//    Point3[] pts3 = new Point3[k];
		//    for (int i = 0; i < k; ++i){
		//        pts3[i] = this.points3[i];
		//    }
		//    Polygon bot = new Polygon(pts2);
		//    bot.points3.AddRange(pts3);

		//    for (int i = 0; i < k; ++i) {
		//        pts2[i] = this.points[k+i];
		//    }
		//    for (int i = 0; i < k; ++i) {
		//        pts3[i] = this.points3[k+i];
		//    }
		//    Polygon top = new Polygon(pts2);
		//    top.points3.AddRange(pts3);
		//    this.AddPolygon(bot);
		//    this.AddPolygon(top);

		//    for (int i = 0; i < k; ++i)
		//    {
		//        int j = i, l = (i + 1) % k;
		//        if (i > 1) { j = (i + 1) % k; l = i; }
		//        Point2[] points2 = new Point2[4] {
		//            bot.points[j],bot.points[l], top.points[l],top.points[j],
		//        };
		//        Point3[] points3 = new Point3[4] {
		//            bot.points3[j],bot.points3[l], top.points3[l],top.points3[j],
		//        };
		//        List<Point2> tmplist = new List<Point2>();
		//        tmplist.AddRange(points2);
		//        Polygon plg = new Polygon(tmplist);
		//        plg.points3.AddRange(points3);
		//        this.AddPolygon(plg);
		//    }

		//    this.type = ProxyType.POLY;
		//    this.Compute3DInfo();
		//}
		public void AssignIndex()
		{
			if (this.points == null) return;

			int index = 0;
			foreach (Point2 p in this.points)
			{
				p.index = index++;
			}
		}
		public void ComputeVolume()
		{
			if (this.polygons.Count < 2)  
				this.volume = 0;
			Polygon bot = this.polygons[0];
			Polygon top = this.polygons[1];
			double h = (top.points3[0].pos - bot.points3[0].pos).Length();
			int n = bot.points3.Count;
			Vector3d o = bot.points3[0].pos;
			double sum = 0;
			for (int i = 1; i < n - 1; ++i) {
				Vector3d u = bot.points3[i].pos, v = bot.points3[i+1].pos;
				sum += ((u-o).Cross(v-o)).Length() / 2.0;
			}
			this.volume = sum*h;
		}
		public void ComputeCenter()
		{
			Vector3d c = new Vector3d();
			foreach (Point3 pt in this.points3)
			{
				c += pt.pos;
			}
			if (this.center == null)
				this.center = new Point3(c / this.points3.Count);
			else
				this.center.pos = c / this.points3.Count;
		}
		public void AddPolygon(Polygon plg)
		{
			this.polygons.Add(plg);
			plg.hostproxy = this;
		}
		public void Compute3DInfo()
		{
			this.ComputeCenter();
			Vector2d c2 = new Vector2d();

			if (this.points != null)
			{
				foreach (Point2 pt in this.points)
				{
					c2 += pt.pos;
				}
				if (this.center2 != null)
					this.center2.pos = c2 / this.points.Count;
				else
					this.center2 = new Point2(c2 / this.points.Count);
			}

			foreach (Polygon poly in this.polygons)
			{
				poly.ComputeCenter3();
				poly.ComputeNormal();
				if (poly.normal.Dot(poly.center.pos - this.center.pos) < 0)
					poly.normal = new Vector3d() - poly.normal;
				poly.ComputeLocalFrame();
				poly.ComputeLinesegmentsP3();
			}
			if (this.polygons.Count == 6) // a box
			{
				Vector3d x = (this.polygons[2].center.pos - this.polygons[4].center.pos);
				Vector3d y = (this.polygons[3].center.pos - this.polygons[5].center.pos);
				Vector3d z = (this.polygons[1].center.pos - this.polygons[0].center.pos);
				this.xlen = x.Length();
				this.ylen = y.Length();
				this.zlen = z.Length();
				x = x.Normalize();
				z = z.Normalize();
				y = y.Normalize();// z.Cross(x).Normalize(); // make sure it is right-hand convention

				this.scale = new Vector3d(xlen / 2, ylen / 2, zlen / 2);

				this.frame = new CoordinateFrame(this.center.pos, x, y, z);
			}

			this.ComputePointNormals();
		}
		
		private void ComputePointNormals()
		{
			foreach (Point3 p3 in this.points3)
			{
				Vector3d normal = new Vector3d();
				foreach (Polygon plg in this.polygons)
				{
					if (plg.points3.Contains(p3))
					{
						normal += plg.normal;
					}
				}
				p3.normal = normal.Normalize();
			}
		}
		private void EstablishOpposite()
		{
			if (this.type != ProxyType.CUBOID)
			{
				return;
			}
			this.polygons[0].opposite = this.polygons[1];
			this.polygons[1].opposite = this.polygons[0];
			this.polygons[2].opposite = this.polygons[3];
			this.polygons[4].opposite = this.polygons[2];
			this.polygons[3].opposite = this.polygons[5];
			this.polygons[5].opposite = this.polygons[3];
		}
		private void EstablishHexagonIndices()
		{
			this.hexagonindice = new List<int>();
			this.hexagonindice.Add(4);
			this.hexagonindice.Add(0);
			this.hexagonindice.Add(1);
			this.hexagonindice.Add(2);
			this.hexagonindice.Add(6);
			this.hexagonindice.Add(7);
		}
		private void EstablishLocalFrame()
		{
			if (this.polygons == null || this.polygons.Count < 5)
			{
				return;
			}

			this.ComputeCenter();

			foreach (Polygon plg in this.polygons)
				plg.ComputeCenter3();

			Vector3d zaxis = this.polygons[1].center.pos - this.polygons[0].center.pos;
			Vector3d xaxis = this.polygons[4].center.pos - this.polygons[2].center.pos;
			zaxis = zaxis.Normalize();
			xaxis = xaxis.Normalize();
			Vector3d yaxis = zaxis.Cross(xaxis).Normalize();

			this.frame = new CoordinateFrame(this.center.pos, xaxis, yaxis, zaxis);

		}

		public object Clone()
		{
			PolyProxy another = new PolyProxy();
			List<Point2> pts2 = new List<Point2>();
			List<Point3> pts3 = new List<Point3>();
			foreach (Point2 pt in this.points)
			{
				pts2.Add(pt.Clone() as Point2);
			}
			foreach (Point3 pt in this.points3)
			{
				pts3.Add(pt.Clone() as Point3);
			}
			another.points = pts2;
			another.points3 = pts3;

			List<Polygon> plgs = new List<Polygon>();
			foreach (Polygon plg in this.polygons)
			{
				plgs.Add(plg.Clone() as Polygon);
			}

			// polygons share points
			int j = 0;
			foreach (Polygon plg in this.polygons)
			{
				Polygon ply = plgs[j];
				for (int i = 0; i < plg.points3.Count; ++i)
				{
					int index = plg.points3[i].index;
					ply.points3[i] = pts3[index];
				}
				++j;
			}
			another.polygons = plgs;
			foreach (Polygon ply in plgs)
				ply.hostproxy = another;

			// 0--
			another.prevattachindex = another.attachindex
					= this.attachindex;
			another.center = new Point3(this.center.pos);
			another.index = this.index;
			another.type = this.type;
			another.txtquads = new List<Polygon>();
			if (this.txtquads != null) // copy texture quads
				foreach (Polygon plg in this.txtquads)
				{
					int index = this.polygons.IndexOf(plg);
					Polygon plg2 = another.polygons[index];
					plg2.textureindex = plg.textureindex;
					another.txtquads.Add(plg2);
				}
			another.frame = new CoordinateFrame(this.frame.o, this.frame.x, this.frame.y, this.frame.z);
			another.Compute3DInfo();
			return another;
		}
		public static void OptimizeBox(PolyProxy box, Vector3d c, double xlen, double ylen, double zlen)
		{
			Vector3d xdir = box.polygons[2].center.pos - box.polygons[4].center.pos;
			Vector3d ydir = box.polygons[3].center.pos - box.polygons[5].center.pos;
			Vector3d zdir = box.polygons[1].center.pos - box.polygons[0].center.pos;
			int sign = box.polygons[2].normal.Dot(box.polygons[4].normal) < 0 ? -1 : 1;
			Vector3d xaxis = (box.polygons[2].normal + box.polygons[4].normal * sign).Normalize();
			if (xaxis.Dot(xdir) < 0) xaxis = new Vector3d() - xaxis;
			sign = box.polygons[3].normal.Dot(box.polygons[5].normal) < 0 ? -1 : 1;
			Vector3d yaxis = (box.polygons[3].normal + box.polygons[5].normal * sign).Normalize();
			if (yaxis.Dot(ydir) < 0) yaxis = new Vector3d() - yaxis;
			sign = box.polygons[1].normal.Dot(box.polygons[0].normal) < 0 ? -1 : 1;
			Vector3d zaxis = (box.polygons[1].normal + box.polygons[0].normal * sign).Normalize();
			if (zaxis.Dot(zdir) < 0) zaxis = new Vector3d() - zaxis;
			xlen /= 2;
			ylen /= 2;
			zlen /= 2;
			box.points3[0].pos = c + new Vector3d(xlen, -ylen, -zlen);
			box.points3[1].pos = c + new Vector3d(xlen, ylen, -zlen);
			box.points3[2].pos = c + new Vector3d(-xlen, ylen, -zlen);
			box.points3[3].pos = c + new Vector3d(-xlen, -ylen, -zlen);
			box.points3[4].pos = c + new Vector3d(xlen, -ylen, zlen);
			box.points3[5].pos = c + new Vector3d(xlen, ylen, zlen);
			box.points3[6].pos = c + new Vector3d(-xlen, ylen, zlen);
			box.points3[7].pos = c + new Vector3d(-xlen, -ylen, zlen);
		}
		public static PolyProxy CreateCuboid(Vector3d center, Vector3d scale)
		{
			Vector3d off = scale;
			Point3[] bot = new Point3[4] {
				new Point3(new Vector3d(center - off)),
				new Point3(new Vector3d(center + new Vector3d(-off.x,off.y,-off.z))),
				new Point3(new Vector3d(center + new Vector3d(off.x,off.y,-off.z))),
				new Point3(new Vector3d(center + new Vector3d(off.x,-off.y,-off.z)))
			};
			Point3[] top = new Point3[4] {
				new Point3(new Vector3d(center + new Vector3d(-off.x,-off.y,off.z))),
				new Point3(new Vector3d(center + new Vector3d(-off.x,off.y,off.z))),
				new Point3(new Vector3d(center + new Vector3d(off.x,off.y,off.z))),
				new Point3(new Vector3d(center + new Vector3d(off.x,-off.y,off.z)))
			};
			Polygon plg1 = new Polygon(bot);
			Polygon plg2 = new Polygon(top);
			PolyProxy box = new PolyProxy(plg1, plg2);
			box.MakeBox();

			return box;
		}
		public static PolyProxy CreateCylinder(int nslices)
		{
			// create top & bot polygon
			Polygon plg_bot = Polygon.CreateCircle(nslices, -1);
			Polygon plg_top = Polygon.CreateCircle(nslices, 1);
			
			PolyProxy cylinder = new PolyProxy(plg_bot, plg_top);

			return cylinder;
		}

		

		// split the box into two boxes
		public List<PolyProxy> Split(Vector3d dir)
		{
			List<Polygon> plgs = new List<Polygon>();
			foreach (Polygon plg in this.polygons)
			{
				double cos = Math.Abs(plg.normal.Dot(dir));
				if (cos > 0.5)
				{
					plgs.Add(plg);
				}
			}
			
			if (plgs.Count != 2) throw new Exception();

			Polygon plg1 = plgs[0], plg2 = plgs[1];
			Vector3d center = (plg1.center.pos + plg2.center.pos)/2;
			Vector3d off = center - plg1.center.pos;
			Polygon iplg = plg1.Clone() as Polygon;
			for (int i = 0; i < iplg.points3.Count; ++i)
				iplg.points3[i].pos += off;

			PolyProxy box1 = new PolyProxy(plg1, iplg);
			PolyProxy box2 = new PolyProxy(iplg, plg2);
			box1.MakeBox();
			box2.MakeBox();

			List<PolyProxy> boxes = new List<PolyProxy>();
			boxes.Add(box1);
			boxes.Add(box2);

			return boxes;
		}


		public Vector3d GetLongestDirection()
		{
			if (this.scale.x > this.scale.y && this.scale.x > this.scale.z)
				return new Vector3d(1, 0, 0);
			else if (this.scale.y > this.scale.z && this.scale.y > this.scale.x)
				return new Vector3d(0, 1, 0);
			else return new Vector3d(0, 0, 1);
		}

		#region IComparable<PolyProxy> Members
		public int CompareTo(PolyProxy other)
		{
			return Math.Sign(other.depth - this.depth);
		}
		#endregion

	}
	
	public class GroundGrid : Polygon
	{
		public double width, height;
		public int rows, cols;
		public double zlevel = -0.7;
		public GroundGrid(double w, double h, int rows, int cols)
		{
			double x = w/2, z = h/2;
			double y = zlevel;
			Point3[] v = new Point3[4] {
				new Point3(new Vector3d(-x,y,-z)),
				new Point3(new Vector3d(-x,y,z)),
				new Point3(new Vector3d(x,y,z)),
				new Point3(new Vector3d(x,y,-z))
			};
			this.rows = rows;
			this.cols = cols;
			this.points3.AddRange(v);

			this.Compute();
		}
		public void Move2Level(double zlevel)
		{
			foreach (Point3 p in this.points3)
			{
				p.pos.y = zlevel;
			}
			this.ComputeLocalFrame();
			this.ComputeCenter3();
		}
	}

	public class MeshComponent
	{
		public int componentIndex = -1;
		public int colorIdx = 10;
		public bool isgroundcontact = false;
		public bool handled = false;
		public string name = "Unknown";
		public int vertexCount;
		public List<int> componentsGroupIndices = null;
		public double[] vertexPos = null;
		public int[] faceIndex = null;
		public int[] vertexIndex = null;
		public int[] updatedfaceIndex = null;
		public Mesh mesh = null;
		public PolyProxy boundingBox = null;
		public PolyProxy proxy = null;
        public MeshComponent parent = null;
		public List<MeshComponent> adjComponents = null;
        public List<MeshComponent> kids = null;
		public List<MeshComponent> symmetryGroup = null;
		public List<Contact> contacts = null;

		public ModelGraphNode graphNode = null;
		public Vector3d[] axes = new Vector3d[3]; 
		public int mainAxis = -1;
		public int originalMCidx = -1;

		public MeshComponent(List<int> vindex, List<double> vpos, List<int> findex)
		{
			this.vertexIndex = vindex.ToArray();
			this.vertexPos = vpos.ToArray();
			this.faceIndex = findex.ToArray();
		}
		public MeshComponent(int[] vindex, double[] vpos, int[] findex)
		{
			this.vertexIndex = vindex;
			this.vertexPos = vpos;
			this.faceIndex = findex;
		}
		public MeshComponent(Mesh m)
		{
			this.mesh = m;
			this.vertexPos = m.VertexPos;
			this.faceIndex = m.FaceIndex;
			this.FindBoundingBox();
		}

		public Vector3d MaxCoord()
		{
			return this.boundingBox.points3[6].pos;
		}
		public Vector3d MinCoord()
		{
			return this.boundingBox.points3[0].pos;
		}
		public void AddContact(Contact c)
		{
			if (this.contacts == null)
			{
				this.contacts = new List<Contact>();
			}
			if (!this.contacts.Contains(c))
				this.contacts.Add(c);
		}
	
		public void FindBoundingBox()
		{
			Vector3d maxCoord = new Vector3d(double.MinValue, double.MinValue, double.MinValue);
			Vector3d minCoord = new Vector3d(double.MaxValue, double.MaxValue, double.MaxValue);

			if (this.mesh != null)
			{
				maxCoord = this.mesh.MaxCoord();
				minCoord = this.mesh.MinCoord();
			}
			else
			{
				if (this.vertexPos == null) return;

				this.vertexCount = this.vertexPos.Length / 3;
				for (int i = 0, j = 0; i < vertexCount; i++, j += 3)
				{
					Vector3d v = new Vector3d(vertexPos, j);
					maxCoord = Vector3d.Max(maxCoord, v);
					minCoord = Vector3d.Min(minCoord, v);
				}
			}

			// the other six points
			Vector3d u1 = new Vector3d(minCoord.x, maxCoord.y, minCoord.z);
			Vector3d u2 = new Vector3d(maxCoord.x, minCoord.y, minCoord.z);
			Vector3d u3 = new Vector3d(maxCoord.x, maxCoord.y, minCoord.z);
			Vector3d v1 = new Vector3d(minCoord.x, maxCoord.y, maxCoord.z);
			Vector3d v2 = new Vector3d(maxCoord.x, minCoord.y, maxCoord.z);
			Vector3d v3 = new Vector3d(minCoord.x, minCoord.y, maxCoord.z);

			Point3[] botpts = new Point3[4] {
				new Point3(minCoord),
				new Point3(u1),
				new Point3(u3),
				new Point3(u2),
			};

			Point3[] toppts = new Point3[4] {
				new Point3(v3),
				new Point3(v1),
				new Point3(maxCoord),
				new Point3(v2),
			};

			Polygon top = new Polygon(toppts);
			Polygon bot = new Polygon(botpts);

			this.boundingBox = new PolyProxy(bot, top);

			//if (this.mainAxis == -1)
			//{
			//    CalculatePCAxes();
			//}
			//SamplePointsOnBbox();
		}
		// added--Han--for computing main axes
		public void CalculatePCAxes()
		{
			if (this.vertexPos == null) return;

			this.vertexCount = this.vertexPos.Length / 3;

			int n = this.vertexCount;
			double[,] mat = new double[n, 3];
			int index = 0;
			double x = 0, y = 0, z = 0;
			for (int i = 0, j = 0; i < n; ++i, j += 3)
			{
				mat[index, 0] = this.vertexPos[j];
				mat[index, 1] = this.vertexPos[j + 1];
				mat[index, 2] = this.vertexPos[j + 2];

				x += mat[index, 0];
				y += mat[index, 1];
				z += mat[index, 2];

				index++;
			}
			// the center
			x /= n;
			y /= n;
			z /= n;
			Vector3d center = new Vector3d(x, y, z);

			// PCA
			PrincipalComponentAnalysis pca = new PrincipalComponentAnalysis(mat,
				AnalysisMethod.Center);

			pca.Compute();

			if (pca.Components.Count < 3)
			{
				Program.OutputText(" < 3 principle axies found!", true);
				throw new Exception();
			}

			// axes
			axes[0] = new Vector3d(pca.Components[0].Eigenvector, 0).Normalize();
			axes[1] = new Vector3d(pca.Components[1].Eigenvector, 0).Normalize();
			axes[2] = new Vector3d(pca.Components[2].Eigenvector, 0).Normalize();

			double maxval = 0;
			for (int i = 0; i < 3; i++)
			{
				if (Math.Abs(axes[0][i]) > maxval)
				{
					maxval = Math.Abs(axes[0][i]);
					this.mainAxis = i;
				}
			}
		}

		public void SamplePointsOnBbox()
		{
			// added--Han
			CalculatePCAxes();
			this.boundingBox.sampledPointsOnBox = new List<Point3>();
			this.boundingBox.sampledPointsOnBox_visual = new List<Vector3d>();
			for (int i = 0; i < 8; i++)
			{
				this.boundingBox.sampledPointsOnBox.Add(new Point3(this.boundingBox.points3[i].pos));
				this.boundingBox.sampledPointsOnBox_visual.Add(new Vector3d(this.boundingBox.points3[i].pos));
			}
			// sample points
			this.mainAxis = -1;
			double maxval = 0;
			for (int i = 0; i < 3; i++)
			{
				if (Math.Abs(axes[0][i]) > maxval)
				{
					maxval = Math.Abs(axes[0][i]);
					this.mainAxis = i;
				}
			}
			int nsample = 100;
			Point3[] startpnts = new Point3[4];
			int[] pindex = new int[4];
			switch (this.mainAxis)
			{
				case 0:
					pindex = new int[4] { 0, 1, 5, 4 };
					break;
				case 1:
					pindex = new int[4] { 0, 4, 7, 3 };
					break;
				case 2:
					pindex = new int[4] { 0, 1, 2, 3 };
					break;
				default:
					break;
			}
			double interval = (this.boundingBox.points3[6].pos[this.mainAxis]
							- this.boundingBox.points3[0].pos[this.mainAxis]) / nsample;
			for (int i = 0; i < 4; i++)
				startpnts[i] = this.boundingBox.points3[pindex[i]];
			if (axes[0][this.mainAxis] < 0)
				axes[0] = -1.0 * axes[0];
			for (int i = 0; i < nsample; i++)
			{
				for (int j = 0; j < 4; j++)
				{
					Point3 newpnt = new Point3(startpnts[j].pos);
					newpnt.pos[this.mainAxis] += interval * i;
					this.boundingBox.sampledPointsOnBox.Add(newpnt);
					this.boundingBox.sampledPointsOnBox_visual.Add(new Vector3d(newpnt.pos));
				}
			}
			// sample on bbox faces

			// #1
			Vector3d start_dir1 = this.boundingBox.points3[0].pos,
				end_dir1 = this.boundingBox.points3[4].pos,
				end_dir2 = this.boundingBox.points3[1].pos;
			Vector3d dir1 = new Vector3d(0, 0, 1), dir2 = new Vector3d(0, 1, 0);
			double interval_1 = (end_dir1.z - start_dir1.z) / nsample,
				interval_2 = (end_dir2.y - start_dir1.y) / nsample;
			SampleOnFace(start_dir1, dir1, dir2, nsample, interval_1, interval_2);
			// #2
			start_dir1 = this.boundingBox.points3[3].pos;
			end_dir1 = this.boundingBox.points3[7].pos;
			end_dir2 = this.boundingBox.points3[2].pos;

			interval_1 = (end_dir1.z - start_dir1.z) / nsample;
			interval_2 = (end_dir2.y - start_dir1.y) / nsample;
			SampleOnFace(start_dir1, dir1, dir2, nsample, interval_1, interval_2);

			// #3
			start_dir1 = this.boundingBox.points3[0].pos;
			end_dir1 = this.boundingBox.points3[3].pos;
			end_dir2 = this.boundingBox.points3[1].pos;
			dir1 = new Vector3d(1, 0, 0);
			dir2 = new Vector3d(0, 1, 0);
			interval_1 = (end_dir1.x - start_dir1.x) / nsample;
			interval_2 = (end_dir2.y - start_dir1.y) / nsample;
			SampleOnFace(start_dir1, dir1, dir2, nsample, interval_1, interval_2);

			// #4
			start_dir1 = this.boundingBox.points3[4].pos;
			end_dir1 = this.boundingBox.points3[7].pos;
			end_dir2 = this.boundingBox.points3[5].pos;

			interval_1 = (end_dir1.x - start_dir1.x) / nsample;
			interval_2 = (end_dir2.y - start_dir1.y) / nsample;
			SampleOnFace(start_dir1, dir1, dir2, nsample, interval_1, interval_2);

			// #5
			start_dir1 = this.boundingBox.points3[0].pos;
			end_dir1 = this.boundingBox.points3[3].pos;
			end_dir2 = this.boundingBox.points3[4].pos;
			dir1 = new Vector3d(1, 0, 0);
			dir2 = new Vector3d(0, 0, 1);
			interval_1 = (end_dir1.x - start_dir1.x) / nsample;
			interval_2 = (end_dir2.z - start_dir1.z) / nsample;
			SampleOnFace(start_dir1, dir1, dir2, nsample, interval_1, interval_2);

			// #6
			start_dir1 = this.boundingBox.points3[1].pos;
			end_dir1 = this.boundingBox.points3[2].pos;
			end_dir2 = this.boundingBox.points3[5].pos;

			interval_1 = (end_dir1.x - start_dir1.x) / nsample;
			interval_2 = (end_dir2.z - start_dir1.z) / nsample;
			SampleOnFace(start_dir1, dir1, dir2, nsample, interval_1, interval_2);
		}

		public void SampleOnFace(Vector3d start_dir1, Vector3d dir1, Vector3d dir2, int nsample,
			double interval_1, double interval_2 )
		{
			for (int rp = 0; rp < nsample; ++rp)
			{
				Vector3d start_c = start_dir1 + rp * interval_2 * dir2;
				for (int cp = 0; cp < nsample; ++cp)
				{
					Vector3d newVec = start_c + cp * interval_1 * dir1;
					this.boundingBox.sampledPointsOnBox_visual.Add(newVec);
					this.boundingBox.sampledPointsOnBox.Add(new Point3(newVec));
				}
			}
		}

		public Object Clone()
		{
			// create a new component by copying from one in one graph in the group 
			int[] vertexIndex = this.vertexIndex.Clone() as int[];
			double[] vertexPos = this.vertexPos.Clone() as double[]; 
			int[] faceIndex = this.faceIndex.Clone() as int[]; 
			MeshComponent mc = new MeshComponent(vertexIndex, vertexPos, faceIndex);
			mc.updatedfaceIndex = this.updatedfaceIndex.Clone() as int[];
			mc.mesh = new Mesh(vertexPos, mc.updatedfaceIndex); //create a new mesh
			mc.FindBoundingBox();

			mc.boundingBox.center.pos = this.boundingBox.center.pos; //in case of update

			mc.componentIndex = this.componentIndex;
			mc.mainAxis = this.mainAxis;
			mc.originalMCidx = this.originalMCidx;
			mc.colorIdx = this.colorIdx;
			return mc;
		}

		public void FindPCABoundingBox()
		{
			if (this.vertexPos == null) return;

			this.vertexCount = this.vertexPos.Length / 3;

			int n = this.vertexCount;
			double[,] mat = new double[n, 3];
			int index = 0;
			double x = 0, y = 0, z = 0;
			for (int i = 0, j = 0; i < n; ++i, j+=3)
			{
				mat[index, 0] = this.vertexPos[j];
				mat[index, 1] = this.vertexPos[j + 1];
				mat[index, 2] = this.vertexPos[j + 2];

				x += mat[index, 0];
				y += mat[index, 1];
				z += mat[index, 2];

				index++;
			}
			// the center
			x /= n;
			y /= n;
			z /= n;
			Vector3d center = new Vector3d(x, y, z);

			// PCA
			PrincipalComponentAnalysis pca = new PrincipalComponentAnalysis(mat,
				AnalysisMethod.Center);

			pca.Compute();

			if (pca.Components.Count < 3)
			{
				Program.OutputText(" < 3 principle axies found!", true);
				throw new Exception();
			}

			// axies
			Vector3d axis_x = new Vector3d(pca.Components[0].Eigenvector, 0).Normalize();
            Vector3d axis_y = new Vector3d(pca.Components[1].Eigenvector, 0).Normalize();
            Vector3d axis_z = new Vector3d(pca.Components[2].Eigenvector, 0).Normalize();

			// scale
			double[,] trans = pca.Transform(mat, 3);

			double max_x = double.MinValue;
			double max_y = double.MinValue;
			double max_z = double.MinValue;
			double min_x = double.MaxValue;
			double min_y = double.MaxValue;
			double min_z = double.MaxValue;
			for (int i = 0, j = 0; i < n; ++i, j += 3)
			{	
				// compute the min max x,y,z
				if (max_x < trans[i, 0]) max_x = trans[i, 0];
				if (min_x > trans[i, 0]) min_x = trans[i, 0]; 
				if (max_y < trans[i, 1]) max_y = trans[i, 1];
				if (min_y > trans[i, 1]) min_y = trans[i, 1]; // compute the min x
				if (max_z < trans[i, 2]) max_z = trans[i, 2];
				if (min_z > trans[i, 2]) min_z = trans[i, 2]; // compute the min x
			}

			// shift the centric point
			double shiftX = (max_x + min_x) / 2;
			double scaleX = (max_x - min_x) / 2;
			double shiftY = (max_y + min_y) / 2;
			double scaleY = (max_y - min_y) / 2;
			double shiftZ = (max_z + min_z) / 2;
			double scaleZ = (max_z - min_z) / 2;
			
			center += (axis_x * shiftX + axis_y*shiftY + axis_z*shiftZ);
			Vector3d scale = new Vector3d(scaleX, scaleY, scaleZ);

			// the 8 corner points
			Vector3d u0 = center - scale.x * axis_x - scale.y * axis_y - scale.z * axis_z;
			Vector3d u1 = center - scale.x * axis_x + scale.y * axis_y - scale.z * axis_z;
			Vector3d u2 = center + scale.x * axis_x + scale.y * axis_y - scale.z * axis_z;
			Vector3d u3 = center + scale.x * axis_x - scale.y * axis_y - scale.z * axis_z;
			Vector3d v0 = center - scale.x * axis_x - scale.y * axis_y + scale.z * axis_z;
			Vector3d v1 = center - scale.x * axis_x + scale.y * axis_y + scale.z * axis_z;
			Vector3d v2 = center + scale.x * axis_x + scale.y * axis_y + scale.z * axis_z;
			Vector3d v3 = center + scale.x * axis_x - scale.y * axis_y + scale.z * axis_z;

			Point3[] botpts = new Point3[4] {
				new Point3(u0),
				new Point3(u1),
				new Point3(u2),
				new Point3(u3),
			};

			Point3[] toppts = new Point3[4] {
				new Point3(v0),
				new Point3(v1),
				new Point3(v2),
				new Point3(v3),
			};

			Polygon top = new Polygon(toppts);
			Polygon bot = new Polygon(botpts);

			this.boundingBox = new PolyProxy(bot, top);

		}
		// create mesh from existing components
		public void CreateMesh(Mesh originalMesh)
		{
			Dictionary<int, int> dict = new Dictionary<int, int>();
			int vindex = 0;
			foreach (int v in this.vertexIndex)
			{
				dict.Add(v, vindex++);
			}

			int[] faceindex = new int[this.faceIndex.Length * 3];
			int findex = 0;
			foreach (int f in this.faceIndex)
			{
				int b = f * 3;
				faceindex[findex] = dict[originalMesh.FaceIndex[b]];
				faceindex[findex + 1] = dict[originalMesh.FaceIndex[b + 1]];
				faceindex[findex + 2] = dict[originalMesh.FaceIndex[b + 2]];
				findex += 3;
			}
			this.mesh = new Mesh(vertexPos, faceindex);
			this.updatedfaceIndex = faceindex;
		}
		// create mesh from kids' existing meshes
		public void CreateMesh()
		{
			if (this.mesh != null || this.kids == null) return;

			List<Mesh> meshes = new List<Mesh>();
			foreach (MeshComponent c in this.kids)
			{
				if (c.mesh != null)
					meshes.Add(c.mesh);
			}
			this.mesh = Mesh.CreateMesh(meshes);
		}
		public void AddAdjComponent(MeshComponent mc)
		{
			if (this.adjComponents == null) this.adjComponents = new List<MeshComponent>();
			if (!this.adjComponents.Contains(mc))
				this.adjComponents.Add(mc);
		}
		public void InitComponentGroupIndices()
		{
			// the index for each component --order+color index
			this.componentsGroupIndices = new List<int>();
			this.componentsGroupIndices.Add(this.componentIndex);
		}

		// transform
		public void TransFromOld(Matrix4d T)
		{
			if (this.mesh != null) this.mesh.TransformFromOld(T);
			if (this.proxy != null) this.proxy.TransFromOld(T);
			if (this.boundingBox != null) this.boundingBox.TransFromOld(T);
		}
		public void Transform(Matrix4d T)
		{
			if (this.mesh != null) this.mesh.Transform(T);
			if (this.proxy != null) this.proxy.Transform(T);
			if (this.boundingBox != null) this.boundingBox.Transform(T);
		}
		public void Update()
		{
			if (this.mesh != null) this.mesh.UpdateVertexPos();
			if (this.proxy != null) this.proxy.Update();
			if (this.boundingBox != null) this.boundingBox.Update();
		}
	}

	////////////////////////////// Graph Operation --- Han (2013-12-24)//////////////////////////////
	/// <summary>
	/// The concept of dualedge is similar to halfedge, it is used to depict docking site
	/// it is constructed once a graph (for a model) is built
	/// </summary>
	public class DualEdge
	{
		public ModelGraphNode toNode = null;
		public ModelGraphNode fromNode = null;
		public DualEdge inverseEdge = null;
		public DualEdge prevEdge = null;
		public DualEdge nextEdge = null;
		public ModelGraphEdge edge = null;
		public int edgeID = -1;
		public List<DualEdge> oneRingNeigs = null;
		// regarding the out node, the direction of this edge
		public int dir = -1;

		public DualEdge(ModelGraphNode inND, ModelGraphNode outND,
			DualEdge prevND, DualEdge nextND, DualEdge invEdge, ModelGraphEdge e)
		{
			this.toNode = inND;
			this.fromNode = outND;
			this.prevEdge = prevND;
			this.nextEdge = nextND;
			this.inverseEdge = invEdge;
			this.edge = e;
		}

		public bool isEqual(DualEdge e)
		{
			bool iseq = false;
			if (e.fromNode.nodeType == this.fromNode.nodeType && e.toNode.nodeType == this.toNode.nodeType)
				iseq = true;
			return iseq;
		}
	}

	public static class MaxNumberNodeType
	{
		public static int MAXNODETYPE = 20;
	}
	public class ModelGraphNode
	{
		public int graphIdx = -1; // belong to which model/graph
		public int nodeIdx = -1; // in the parent graph, the index of this node
		public int nodeType = -1; // node type by color label
		public string name = "";
		public MeshComponent meshComponent = null; // corresponding meshcomponent
		public bool handled = false;
		public bool isgroundcontact = false;
		public List<bool> isDumyConnection = null; // empty connection in corresponding direction
		public Vector3d[] axes = new Vector3d[3];
		public int mainAxis = -1;
		public List<DualEdge> dualEdges = null;
		public List<ModelGraphNode> adjNodes = null;
		public List<ModelGraphEdge> edges = null;
		public List<int> edgeOrders = null;
		public int originNodeIdx = -1; //for cloned use
		public Vector3d pos3D = new Vector3d();
		public Vector2d pos2D = new Vector2d();
		public bool canStretch = false;
		// semantic tag
		// if a node is both entry and exit, then it must be a middle node
		// like the chain, accept force from back wheel and send to front wheel
		public bool isEntry = false, isExit = false;
		public int level = -1; // ladder to? tracking ball?

		public ModelGraphNode(MeshComponent mc, int gIDx, int nIdx, int nType)
		{
			this.meshComponent = mc;
			this.graphIdx = gIDx;
			this.nodeIdx = nIdx;
			this.originNodeIdx = nIdx;
			this.nodeType = nType;
			this.edges = new List<ModelGraphEdge>();
			this.isDumyConnection = new List<bool>();
			this.edgeOrders = new List<int>();
			if(mc != null)
				this.pos3D = mc.boundingBox.center.pos;
		}

		public void AddEdge(ModelGraphEdge edge)
		{
			if (this.edges == null)
			{
				this.edges = new List<ModelGraphEdge>();
			}
			if (!this.edges.Contains(edge))
				this.edges.Add(edge);
		}

		public void TransForm(Matrix4d tran)
		{
			Vector4d _pos = new Vector4d(this.pos3D, 1);
			this.pos3D = (tran * _pos).XYZ();
		}

		public Object Clone()
		{
			MeshComponent mc = this.meshComponent.Clone() as MeshComponent;
			ModelGraphNode node = new ModelGraphNode(mc, this.graphIdx, this.nodeIdx, this.nodeType);
			mc.graphNode = node;
			mc.colorIdx = this.nodeType;
			node.isDumyConnection = new List<bool>(this.isDumyConnection);
			for (int i = 0; i < 3; ++i)
				node.axes[i] = this.axes[i];
			node.isEntry = this.isEntry;
			node.isExit = this.isExit;
			node.pos3D = this.pos3D;
			node.mainAxis = this.mainAxis;
			node.canStretch = this.canStretch;
			node.edgeOrders = new List<int>(this.edgeOrders);
			node.name = this.name.Clone() as string;
			node.level = this.level;
            node.isgroundcontact = this.isgroundcontact;
			return node;
		}

		public List<ModelGraphEdge> OneRingTraversal(bool adddummyEdges)
		{
			if(this.edges == null || this.edges.Count < 2) return null;
			// traverse all edges of current node, make sure one-ring neighborhood
			// can be formed, that is, each vertex at least has valence 3
			List<ModelGraphEdge> dummyEdges = new List<ModelGraphEdge>();
			// 1. calculate pca for neighboring nodes
			List<ModelGraphEdge> realEdges = new List<ModelGraphEdge>(),
				dumEdges = new List<ModelGraphEdge>();
			List<ModelGraphNode> realNodes = new List<ModelGraphNode>(),
				dumNodes = new List<ModelGraphNode>();
			// separate to two parts: up/down
			List<ModelGraphNode> upperNodes = new List<ModelGraphNode>(),
				downNodes = new List<ModelGraphNode>();
			List<ModelGraphEdge> upperEdges = new List<ModelGraphEdge>(),
				downEdges = new List<ModelGraphEdge>();
			double maxheight = double.MinValue, minheight = double.MaxValue;
			double maxboxheight = double.MinValue, minboxheight = double.MaxValue;
			for (int i = 0; i < this.edges.Count; ++i)
			{
				if (this.edges[i].isDummyEdge)
				{
					dumEdges.Add(this.edges[i]);
					dumNodes.Add(this.adjNodes[i]);
				}
				else
				{
					if (this.adjNodes[i].pos3D.y > this.pos3D.y)
					{
						upperNodes.Add(this.adjNodes[i]);
						upperEdges.Add(this.edges[i]);
					}
					else
					{
						downNodes.Add(this.adjNodes[i]);
						downEdges.Add(this.edges[i]);
					}
					maxheight = maxheight > this.adjNodes[i].pos3D.y?
					maxheight: this.adjNodes[i].pos3D.y;
					minheight = minheight < this.adjNodes[i].pos3D.y ?
					minheight : this.adjNodes[i].pos3D.y;
					maxboxheight = maxboxheight > this.adjNodes[i].meshComponent.MaxCoord().y?
					maxboxheight: this.adjNodes[i].meshComponent.MaxCoord().y;
					minboxheight = minboxheight < this.adjNodes[i].meshComponent.MinCoord().y ?
					minboxheight : this.adjNodes[i].meshComponent.MinCoord().y;
				}
			}//end-for
			if (downNodes.Count + upperNodes.Count <= 1) return dummyEdges;			
			//re-separate up/down
			List<ModelGraphNode> collectedNodes = new List<ModelGraphNode>();
			List<ModelGraphEdge> collectedEdges = new List<ModelGraphEdge>();
			collectedNodes.AddRange(upperNodes);
			collectedNodes.AddRange(downNodes);
			collectedEdges.AddRange(upperEdges);
			collectedEdges.AddRange(downEdges);
			upperNodes = new List<ModelGraphNode>();
			downNodes = new List<ModelGraphNode>();
			downEdges = new List<ModelGraphEdge>();
			upperEdges = new List<ModelGraphEdge>();
			double middle = minboxheight + (maxboxheight - minboxheight) / 2;
			for(int i = 0; i < collectedNodes.Count; ++i)
			{
				ModelGraphNode mn = collectedNodes[i];
				if (mn.meshComponent.boundingBox.center.pos.y > middle)
				{
					upperNodes.Add(mn);
					upperEdges.Add(collectedEdges[i]);
				}
				else
				{
					downNodes.Add(mn);
					downEdges.Add(collectedEdges[i]);
				}
			}

			//order separately
			//re-order edges&adjnodes			
			// if up and down side is not obvious, consider one set
			if (maxheight - minheight < (maxboxheight - minboxheight) * 0.3 ||
				(upperNodes.Count == 0 || downNodes.Count == 0))
			{
				upperNodes.AddRange(downNodes);
				upperEdges.AddRange(downEdges);
				downNodes = new List<ModelGraphNode>();
				downEdges = new List<ModelGraphEdge>();
			}
			//upperNodes.AddRange(downNodes);
			//upperEdges.AddRange(downEdges);
			//downNodes = new List<ModelGraphNode>();
			//downEdges = new List<ModelGraphEdge>();
			for (int level = 0; level < 2; ++level)
			{
				if (level == 0)
				{
					realNodes = upperNodes;
					realEdges = upperEdges;
				}
				else
				{
					realNodes = downNodes;
					realEdges = downEdges;
				}
				int nNodes = realNodes.Count;
				if (nNodes <= 1)
					continue;
				List<ModelGraphEdge> orderedEdges = new List<ModelGraphEdge>();
				List<ModelGraphNode> orderedNodes = new List<ModelGraphNode>();
				double[,] mat = new double[realNodes.Count, 3];
				Vector3d center = new Vector3d();
				for (int i = 0; i < realNodes.Count; ++i)
				{
					for (int j = 0; j < 3; ++j)
					{
						mat[i, j] = realNodes[i].pos3D[j];
						center[j] += mat[i, j];
					}
				}
				center /= nNodes;
				if (nNodes == 0)
					nNodes = 0;				
				
				Vector3d[] axes = new Vector3d[2];
				if (realNodes.Count < 3)
				{
					axes[0] = new Vector3d(1, 0, 0);
					axes[1] = new Vector3d(0, 0, 1);
				}
				else 
				{
					PrincipalComponentAnalysis pca = new PrincipalComponentAnalysis(mat, AnalysisMethod.Center);
					pca.Compute();
					axes[0] = new Vector3d(pca.Components[0].Eigenvector, 0).Normalize();
					axes[1] = new Vector3d(pca.Components[1].Eigenvector, 0).Normalize();
				}
				//axes[0] = new Vector3d(1, 0, 0);
				//axes[1] = new Vector3d(0, 0, 1);
				// 2. order edges in counter-clockwise
				Vector2d[] locPos = new Vector2d[nNodes];
				Vector2d center2D = new Vector2d();
				for (int i = 0; i < nNodes; ++i)
				{
					double x = (realNodes[i].pos3D - center).Dot(axes[0]);
					double y = (realNodes[i].pos3D - center).Dot(axes[1]);
					locPos[i] = new Vector2d(x, y);
					center2D += locPos[i];
				}
				center2D /= nNodes;
				for (int i = 0; i < nNodes; ++i)
					locPos[i] -= center2D;
				bool[] visited = new bool[nNodes];
				visited[0] = true;
				List<int> edgeOrder = new List<int>();
				edgeOrder.Add(0);
				double thresh = 1e-3;
				// order
				while (true)
				{
					Vector2d currvec = locPos[edgeOrder[edgeOrder.Count - 1]];
					Vector3d currvec3D = new Vector3d(currvec);
					double angle = 0, minAngle = double.MaxValue, maxAngle = double.MinValue;
					int nextEid = -1, nearestEid = -1;
					for (int i = 0; i < nNodes; ++i)
					{
						if (visited[i]) continue;
						Vector3d neigvec3D = new Vector3d(locPos[i]);

						if (currvec.Length() < thresh || locPos[i].Length() < thresh)
							angle = 0;
						else
						{
							double dotv = locPos[i].Dot(currvec) / (locPos[i].Length() * currvec.Length());
							if (Math.Abs(Math.Abs(dotv) - 1) < thresh)
								angle = Math.PI;
							else
								angle = Math.Acos(locPos[i].Dot(currvec) / (locPos[i].Length() * currvec.Length()));
						}
						Vector3d dirz = neigvec3D.Cross(currvec3D);
						if (dirz.z > 0 && angle < minAngle)
						{
							minAngle = angle;
							nextEid = i;
						}
						else if (dirz.z <= 0 && angle > maxAngle)
						{
							maxAngle = angle;
							nearestEid = i;
						}
					}//end-for
					if (nextEid != -1)
					{
						edgeOrder.Add(nextEid);
						visited[nextEid] = true;
					}
					else if (nearestEid != -1)
					{
						edgeOrder.Add(nearestEid);
						visited[nearestEid] = true;
					}
					if (edgeOrder.Count == nNodes)
						break;
				}//end-while
				for (int i = 0; i < nNodes; ++i)
				{
					orderedEdges.Add(realEdges[edgeOrder[i]]);
					orderedNodes.Add(realNodes[edgeOrder[i]]);
				}
				
				// 3. add dummy edges if needed
				if (adddummyEdges)
				{
					for (int i = 0; i < nNodes; ++i)
					{
						ModelGraphNode n1 = orderedNodes[i];
						ModelGraphNode n2 = orderedNodes[(i + 1) % nNodes];
						ModelGraphEdge[] es = new ModelGraphEdge[3];
						es[0] = orderedEdges[i];
						es[1] = orderedEdges[(i + 1) % nNodes];
						es[2] = null;
						if (!n1.adjNodes.Contains(n2))
						{
							Vector3d pos = (n1.pos3D + n2.pos3D) / 2;
							ModelGraphEdge me = new ModelGraphEdge(pos, this.graphIdx, -1);
							me.isDummyEdge = true;
							if (adddummyEdges)
							{
								me.AddConnectingNode(n1);
								me.AddConnectingNode(n2);
								n1.adjNodes.Add(n2);
								n2.adjNodes.Add(n1);
							}
							dummyEdges.Add(me);
							me.adjEdges = new List<ModelGraphEdge>();
							es[2] = me;
						}
						else
						{
							foreach (ModelGraphEdge mge in n1.edges)
							{
								if (mge.conntectingNodes.Contains(n2))
								{
									es[2] = mge;
									break;
								}
							}
						}
						if (es[2] == null)
						{
							Program.OutputText("Not found the thrid edge!", true);
							break;
						}
						//add adj edges ----with test
						for (int ei = 0; ei < 3; ++ei)
						{
							if (!adddummyEdges && es[ei].isDummyEdge) continue;
							for (int ej = 0; ej < 3; ++ej)
							{
								if (ei == ej) continue;
								if (!adddummyEdges && es[ej].isDummyEdge) continue;
								if (!es[ei].adjEdges.Contains(es[ej]))
									es[ei].adjEdges.Add(es[ej]);
							}
						}
					}//end-add-dummy-edges
				}
				if (level == 0)
				{
					upperNodes = orderedNodes;
					upperEdges = orderedEdges;
				}
				else
				{
					downNodes = orderedNodes;
					downEdges = orderedEdges;
				}
			}//end-separate-order
			// dummy edges between up and down nodes
			if(downNodes.Count > 0)
			{
				// only add one connection from upper to down
				ModelGraphNode n1 = upperNodes[0];
				int nidx = -1;
				double mindist = double.MaxValue;
				ModelGraphNode n2 = null;
				for (int j = 0; j < downNodes.Count; ++j)
				{
					n2 = downNodes[j];
					double dist = (n1.pos3D - n2.pos3D).Length();
					if ( dist < mindist)
					{
						mindist = dist;
						nidx = j;
					}
				}
				// add dumy edges
				ModelGraphEdge[] es = new ModelGraphEdge[3];
				es[0] = upperEdges[0];
				es[1] = downEdges[nidx];
				es[2] = null;
				if (!n1.adjNodes.Contains(n2))
				{
					Vector3d pos = (n1.pos3D + n2.pos3D) / 2;
					ModelGraphEdge me = new ModelGraphEdge(pos, this.graphIdx, -1);
					me.isDummyEdge = true;
					if (adddummyEdges)
					{
						me.AddConnectingNode(n1);
						me.AddConnectingNode(n2);
						n1.adjNodes.Add(n2);
						n2.adjNodes.Add(n1);
					}
					dummyEdges.Add(me);
					me.adjEdges = new List<ModelGraphEdge>();
					es[2] = me;
				}
				else
				{
					foreach (ModelGraphEdge mge in n1.edges)
					{
						if (mge.conntectingNodes.Contains(n2))
						{
							es[2] = mge;
							break;
						}
					}
				}
				//add adj edges
				for (int ei = 0; ei < 3; ++ei)
				{
					if (!adddummyEdges && es[ei].isDummyEdge) continue;
					for (int ej = 0; ej < 3; ++ej)
					{
						if (ei == ej) continue;
						if (!adddummyEdges && es[ej].isDummyEdge) continue;
						if (!es[ei].adjEdges.Contains(es[ej]))
							es[ei].adjEdges.Add(es[ej]);
					}
				}
			}//end-for

			// pre-excluded nodes/edges
			if (adddummyEdges)
			{
				this.edges.AddRange(dumEdges);
				this.adjNodes.AddRange(dumNodes);
			}

			return dummyEdges;
		}//End-traverse-one-ring-neighborhood

		public void AddAdjEdges()
		{
			if (this.edges == null || this.edges.Count < 3)
			{
				if (this.edges.Count == 2)
				{
					if (!this.edges[0].adjEdges.Contains(this.edges[1]))
					{
						this.edges[0].adjEdges.Add(this.edges[1]);
						this.edges[1].adjEdges.Add(this.edges[0]);
					}
				}
				return;
			}
			List<ModelGraphEdge> orderedEdges = new List<ModelGraphEdge>();
			List<ModelGraphNode> orderedNodes = new List<ModelGraphNode>();
			int nNodes = this.adjNodes.Count;
			double[,] mat = new double[nNodes, 3];
			Vector3d center = new Vector3d();
			for (int i = 0; i < this.adjNodes.Count; ++i)
			{
				for (int j = 0; j < 3; ++j)
				{
					mat[i, j] = this.adjNodes[i].pos3D[j];
					center[j] += mat[i, j];
				}
			}
			center /= nNodes;

			Vector3d[] axes = new Vector3d[2];

			PrincipalComponentAnalysis pca = new PrincipalComponentAnalysis(mat, AnalysisMethod.Center);
			pca.Compute();
			axes[0] = new Vector3d(pca.Components[0].Eigenvector, 0).Normalize();
			axes[1] = new Vector3d(pca.Components[1].Eigenvector, 0).Normalize();

			//axes[0] = new Vector3d(1, 0, 0);
			//axes[1] = new Vector3d(0, 1, 0);
			// 2. order edges in counter-clockwise
			Vector2d[] locPos = new Vector2d[nNodes];
			Vector2d center2D = new Vector2d();
			for (int i = 0; i < nNodes; ++i)
			{
				double x = (this.adjNodes[i].pos3D - center).Dot(axes[0]);
				double y = (this.adjNodes[i].pos3D - center).Dot(axes[1]);
				locPos[i] = new Vector2d(x, y);
				center2D += locPos[i];
			}
			center2D /= nNodes;
			for (int i = 0; i < nNodes; ++i)
				locPos[i] -= center2D;
			bool[] visited = new bool[nNodes];
			visited[0] = true;
			List<int> edgeOrder = new List<int>();
			edgeOrder.Add(0);
			double thresh = 1e-3;
			// order
			while (true)
			{
				Vector2d currvec = locPos[edgeOrder[edgeOrder.Count - 1]];
				Vector3d currvec3D = new Vector3d(currvec);
				double angle = 0, minAngle = double.MaxValue, maxAngle = double.MinValue;
				int nextEid = -1, nearestEid = -1;
				for (int i = 0; i < nNodes; ++i)
				{
					if (visited[i]) continue;
					Vector3d neigvec3D = new Vector3d(locPos[i]);

					if (currvec.Length() < thresh || locPos[i].Length() < thresh)
						angle = 0;
					else
					{
						double dotv = locPos[i].Dot(currvec) / (locPos[i].Length() * currvec.Length());
						if (Math.Abs(Math.Abs(dotv) - 1) < thresh)
							angle = Math.PI;
						else
							angle = Math.Acos(locPos[i].Dot(currvec) / (locPos[i].Length() * currvec.Length()));
					}
					Vector3d dirz = neigvec3D.Cross(currvec3D);
					if (dirz.z > 0 && angle < minAngle)
					{
						minAngle = angle;
						nextEid = i;
					}
					else if (dirz.z <= 0 && angle > maxAngle)
					{
						maxAngle = angle;
						nearestEid = i;
					}
				}//end-for
				if (nextEid != -1)
				{
					edgeOrder.Add(nextEid);
					visited[nextEid] = true;
				}
				else if (nearestEid != -1)
				{
					edgeOrder.Add(nearestEid);
					visited[nearestEid] = true;
				}
				if (edgeOrder.Count == nNodes)
					break;
			}//end-while
			for (int i = 0; i < nNodes; ++i)
			{
				orderedEdges.Add(this.edges[edgeOrder[i]]);
				orderedNodes.Add(this.adjNodes[edgeOrder[i]]);
			}
			this.adjNodes = orderedNodes;
			this.edges = orderedEdges;
			for (int i = 0; i < nNodes; ++i)
			{
				ModelGraphEdge[] e3 = new ModelGraphEdge[3];
				e3[0] = this.edges[(i - 1 + nNodes) % nNodes];
				e3[1] = this.edges[i];
				e3[2] = this.edges[(i +1) % nNodes];
				if(!e3[1].adjEdges.Contains(e3[0]))
				{
					e3[1].adjEdges.Add(e3[0]);
					e3[0].adjEdges.Add(e3[1]);
				}
				if(!e3[1].adjEdges.Contains(e3[2]))
				{
					e3[1].adjEdges.Add(e3[2]);
					e3[2].adjEdges.Add(e3[1]);
				}
			}
		}
		public void OrderEdges(ModelGraphNode startNeig, int MaxNumOfEdge)
		{
			// startIdx: start the orientation
			// MaxNumOfEdge: the maximum numeber of edges in this node group---for dummy node
			// project to plane XY (screen) --editable
			int nEdges = this.edges.Count;
			List<Vector2d> projContactPnts = new List<Vector2d>();
			bool[] visited = new bool[nEdges];

			Vector2d mincoord = new Vector2d(double.MaxValue, double.MaxValue);
			int[] axes = new int[2] { 0, 1 };

			double avgAngle = 360 / MaxNumOfEdge;
			this.isDumyConnection = new List<bool>();
			for (int i = 0; i < MaxNumOfEdge; ++i)
				this.isDumyConnection.Add(true);
			// tangent plane
			// normal could get from mesh normal, default z-axis
			Vector3d normal = new Vector3d(0, 0, 1);
			// project all neighboring ndoes to its tangent plane
			Vector3d nodePos = this.pos3D;
			normal = normal.Normalize();
			double d = normal.Dot(nodePos);
			double sqr = normal.SquareLength();
			List<Vector3d> projPoints = new List<Vector3d>();
			int startIdx = -1;
			double maxy = double.MinValue;
			foreach (ModelGraphNode mn in this.adjNodes)
			{
				Vector3d pos = mn.pos3D;
				Vector3d projPnt = pos;// +(d - normal.Dot(pos)) * normal;
				projPnt -= nodePos;
				projPoints.Add(projPnt);
				if (pos.y > maxy)
				{
					maxy = pos.y;
					startIdx = projPoints.Count - 1;
				}
			}
			//local axes
			Vector3d ox, oy;
			if (startNeig != null)
			{
				int index = this.adjNodes.IndexOf(startNeig);
				ox = projPoints[index];
			}
			else
			{
				// upper --xy plane
				ox = projPoints[startIdx];
			}
			ox.Normalize();
			oy = normal.Cross(ox);
			oy.Normalize();
			//
			ox = new Vector3d(1.0, 0, 0);
			oy = new Vector3d(0, 1.0, 0);
			// angles
			Vector2d xp = new Vector2d(0, 1.0);
			double epsilon = 0.00001;
			List<double> angles = new List<double>();
			for (int i = 0; i < projPoints.Count; ++i)
			{
				Vector2d v = new Vector2d(projPoints[i].Dot(ox),
					projPoints[i].Dot(oy));
				v = v.Normalize();
				// dot product
				double dotval = xp.Dot(v);
				double angle;
				if (Math.Abs(dotval - 1.0) <= epsilon)
					angle = 0.0;
				else if (Math.Abs(dotval + 1.0) <= epsilon)
					angle = Math.PI;
				else
				{
					double cross;
					angle = Math.Acos(dotval);
					//cross product
					cross = xp[0] * v[1] - xp[1] * v[0];
					// vector p2 is clockwise from vector p1 
					// with respect to the origin (0.0)
					if (cross < 0)
					{
						angle = 2 * Math.PI - angle;
					}
				}
				double degree = angle * 180.0 / Math.PI;
				angles.Add(degree);
			}
			List<int> eorder = new List<int>();
			for (int i = 0; i < angles.Count; ++i)
				eorder.Add(i);
			for (int i = 0; i < angles.Count; ++i)
			{
				for (int j = i + 1; j < angles.Count; ++j)
				{
					if (angles[j] < angles[i])
					{
						double tmp = angles[j];
						angles[j] = angles[i];
						angles[i] = tmp;
						int i_tmp = eorder[j];
						eorder[j] = eorder[i];
						eorder[i] = i_tmp;
					}
				}
			}

			// put into each edge dir
			double thr = 45;
			this.edgeOrders = new List<int>();
			for (int i = 0; i < angles.Count; ++i)
			{
				for (int ag = 0; ag < MaxNumOfEdge; ++ag)
				{
					double min = ag * avgAngle - thr;
					double max = ag * avgAngle + thr;
					if (ag == 0)
					{
						if ((angles[i] >= 0 && angles[i] <= max) || (angles[i] >= 360 - thr && angles[i] <= 360))
						{
							this.isDumyConnection[ag] = false;
							this.edgeOrders.Add(ag);
							break;
						}
					}
					else
					{
						if (angles[i] > min && angles[i] < max)
						{
							this.isDumyConnection[ag] = false;
							this.edgeOrders.Add(ag);
							break;
						}
					}
				}
			}

			// irregular node
			//if (this.edgeOrders.Count != angles.Count)
			//{
			//    this.edgeOrders = new List<int>();
			//    for (int i = 0; i < angles.Count; ++i)
			//    {
			//        if ((angles[i] > 0 && angles[i] < thr) ||
			//            (angles[i] > 360 - thr && angles[i] < 360))
			//            this.edgeOrders.Add(0);
			//    }
			//}

			if (eorder.Count != this.edges.Count || this.edgeOrders.Count != angles.Count)
			{
				Program.OutputText("Wrong order!", true);
				return;
			}
			List<ModelGraphEdge> orderEdges = new List<ModelGraphEdge>();
			
			for (int i = 0; i < eorder.Count; ++i)
			{
				ModelGraphEdge e = this.edges[eorder[i]];
				orderEdges.Add(e);
				if (e.conntectingNodes[0].nodeIdx == this.nodeIdx)
				{
					e.edgeType[0] = new Tuple<int, int>(this.nodeType, i);
				}
				else
				{
					e.edgeType[1] = new Tuple<int, int>(this.nodeType, i);
				}
			}
			
			this.edges = orderEdges;
		}//end-order edge

		public void OrderEdges(int tag)
		{
			int nEdges = this.edges.Count;			
			List<Vector3d> visContactPoints = new List<Vector3d>();
			this.adjNodes = new List<ModelGraphNode>();
			for (int i = 0; i < nEdges; i++)
			{
				ModelGraphEdge ec = this.edges[i];
				Vector3d contactpnt = ec.pos;
				visContactPoints.Add(contactpnt);
				if (ec.conntectingNodes[0] == this)
					this.adjNodes.Add(ec.conntectingNodes[1]);
				else
					this.adjNodes.Add(ec.conntectingNodes[0]);
			}
			if (nEdges <= 1) 
				return;
			if (tag == 0) return;
			// 2. project to plane and align or to one direction
			// find the two main axis (possibly one direction)
			int[] mainAxes = new int[2] { 0, 1 };
			List<int> orderedEdgeIdx = new List<int>();			
			// project to a plane
			List<Vector2d> projContactPnts = new List<Vector2d>();
			bool[] visited = new bool[nEdges];
			Vector2d center = new Vector2d();
			Vector2d mincoord = new Vector2d(double.MaxValue, double.MaxValue);
			// find the left lower corner pnt
			for (int i = 0; i < nEdges; i++)
			{
				ModelGraphEdge ec = this.edges[i];
				Vector3d contactpnt = ec.pos;
				Vector2d vec = new Vector2d(contactpnt[mainAxes[0]], contactpnt[mainAxes[1]]);
				if (tag == 1)
					vec = this.adjNodes[i].pos2D;
				projContactPnts.Add(vec);
				visited[i] = false;
				// leftmost coordinate for reference
				mincoord = Vector2d.Min(mincoord, vec);
			}
			center = this.pos2D;
			// counter-clock wise order
			orderedEdgeIdx.Add(0);
			visited[0] = true;
			int track = 0;
			List<Vector2d> locPos = new List<Vector2d>();
			for (int i = 0; i < nEdges; ++i)
				locPos.Add(projContactPnts[i] - center);
			bool nosol = false; // if the project points are messy, no more angle in the order can be found
			while (true)
			{
				Vector2d currVec = locPos[orderedEdgeIdx[track++]];
				Vector3d v1 = new Vector3d(currVec[0], currVec[1], 0);
				double minangle = double.MaxValue, maxangle = double.MinValue;
				int idx = -1;
				for (int i = 0; i < nEdges; i++)
				{
					if (visited[i]) continue;
					double angle = Math.Acos(currVec.Dot(locPos[i])/(currVec.Length()*locPos[i].Length()));
					if (locPos[i].Length() == 0 || currVec.Length() == 0)
						angle = 0;
					// for consistent order - counter-clock wise
					Vector3d v2 = new Vector3d(locPos[i][0], locPos[i][1], 0);
					Vector3d crossVec = v1.Cross(v2);
					if (!nosol)
					{
						// z > 0 --> counter-clock wise
						if (crossVec[2] > 0 && angle < minangle)
						{
							minangle = angle;
							idx = i;
						}
					}
					else
					{
						//if (angle < minangle)
						//{
						//    minangle = angle;
						//    idx = i;
						//}
						if (angle > maxangle)
						{
							maxangle = angle;
							idx = i;
						}
					}
				}
				if (idx == -1) // couldn't find the neighbor, that is, all z < 0, use large angle
				{
					nosol = true;
					track--;
					continue;
				}
				orderedEdgeIdx.Add(idx);
				visited[idx] = true;
				if (orderedEdgeIdx.Count == nEdges)
					break;
			}//end-order

			List<ModelGraphEdge> orderEdges = new List<ModelGraphEdge>();
			int idx_start = -1, minType = MaxNumberNodeType.MAXNODETYPE;
			List<int> connNodeTypes = new List<int>();
			for (int i = 0; i < nEdges; ++i)
			{
				ModelGraphEdge e = this.edges[orderedEdgeIdx[i]];
				orderEdges.Add(e);
				if (e.conntectingNodes[0].nodeIdx == this.nodeIdx)
				{
					connNodeTypes.Add(e.conntectingNodes[1].nodeType);
					if (e.conntectingNodes[1].nodeType < minType)
					{
						minType = e.conntectingNodes[1].nodeType;
						idx_start = i;
					}
				}
				else
				{
					connNodeTypes.Add(e.conntectingNodes[0].nodeType);
					if (e.conntectingNodes[0].nodeType < minType)
					{
						minType = e.conntectingNodes[0].nodeType;
						idx_start = i;
					}
				}
			}
			// always put the smallest node label at first for future comparison
			// if in such cases: 0011364700, move all 0 to the front
			if (idx_start == 0 && connNodeTypes[connNodeTypes.Count - 1] == minType)
			{
				int lastidx = connNodeTypes.Count - 1;
				bool allsame = true;
				for (int i = 0; i < connNodeTypes.Count; ++i)
				{
					if (connNodeTypes[i] != minType)
					{
						allsame = false;
						break;
					}
				}
				if (!allsame)
				{
					while (connNodeTypes[lastidx] == minType)
					{
						connNodeTypes.RemoveAt(lastidx);
						connNodeTypes.Insert(0, minType);
						orderEdges.Insert(0, orderEdges[lastidx]);
						orderEdges.RemoveAt(lastidx + 1);
					}
				}
			}
			this.edges = orderEdges;
			this.adjNodes = new List<ModelGraphNode>();
			for (int i = 0; i < nEdges; i++)
			{
				ModelGraphEdge ec = this.edges[i];
				Vector3d contactpnt = ec.pos;
				visContactPoints.Add(contactpnt);
				if (ec.conntectingNodes[0] == this)
					this.adjNodes.Add(ec.conntectingNodes[1]);
				else
					this.adjNodes.Add(ec.conntectingNodes[0]);
			}
		}


	}//End -- ModelGraphNode

	public class ModelGraphEdge
	{
		public int graphIdx = -1; // belong to which model/graph
		public int edgeIdx = -1; // the index of this edge
		public bool updated = false;
		// for axis-aligned bounding box, record its face index
		// y-upright orientation
		// 2: y+, -2: y-, 1: x+, -1: x-, 3: z+, -3: z-
		public int[] faceIndex = new int[2]{0,0}; 
		public Vector3d oldpos, pos; // joint position of the two connected nodes
		// 2 components, for operation (insert/remove) facility
		public List<ModelGraphNode> conntectingNodes = null;
		// connected node type + edge Index (orientation order)
		// each edge only has two connecting components/type
		public Tuple<int, int>[] edgeType = new Tuple<int,int>[2];
		public DualEdge[] dualEdges = new DualEdge[2];
		public List<ModelGraphEdge> adjEdges = new List<ModelGraphEdge>();
		public bool visitedEdgeInSearch = false;
		// for constructing manifold
		public bool isDummyEdge = false;
		// in any case, if there is a direction of this edge
		// i) support, one box beneath the other
		// ii) slider always points to the beneath boxes, otherwise a box points to it for exit
 		//		ladder should be both directions
		public ModelGraphNode start = null, end = null;
		public List<bool> trueAdjEdges = null;
		public List<int> cycleIndices = new List<int>();
        public bool singleEdge = false;
		// functionality
		public bool support = false; // node1 support node2

		public int level = -1;

		public ModelGraphEdge(Vector3d _pos, int gIdx, int eIdx)
		{
			this.oldpos = this.pos = _pos;
			this.graphIdx = gIdx;
			this.edgeIdx = eIdx;
		}		

		public void AddConnectingNode(ModelGraphNode node)
		{
			if (this.conntectingNodes == null)
			{
				this.conntectingNodes = new List<ModelGraphNode>();
			}
			if (!this.conntectingNodes.Contains(node))
			{
				this.conntectingNodes.Add(node);
				node.AddEdge(this);
			}
		}
		public void UpdatePos()
		{
			this.oldpos = this.pos;
		}
		public void RestorePos()
		{
			this.pos = this.oldpos;
		}
		public void TransformFromOld(Matrix4d Q)
		{
			Vector4d _pos = new Vector4d(this.oldpos, 1);
			this.pos = (Q * _pos).XYZ();
			this.updated = true;
		}
		public void TransformFromCurrent(Matrix4d Q)
		{
			Vector4d _pos = new Vector4d(this.pos, 1);
			this.pos = (Q * _pos).XYZ();
			this.updated = true;
		}
		public void TransformFromOld(Matrix4d Q, Vector3d center)
		{
			Vector4d _pos = new Vector4d(this.oldpos - center, 1);
			this.pos = (Q * _pos).XYZ() + center;
		}
		public void TransformFromCurrent(Matrix4d Q, Vector3d center)
		{
			Vector4d _pos = new Vector4d(this.pos - center, 1);
			this.pos = (Q * _pos).XYZ() + center;
		}

	}// End -- ModelGraphEdge

	public class Subgraph
	{
		public int graphIdx = -1; // belong to which graph
		public int subgraphIdx = -1; // the subgraph group in GraphGroup..
		public List<ModelGraphNode> nodeSets = null;
		// outgoing edge + linked node type + linked node
		public List<ModelGraphEdge> outgoingEdges = new List<ModelGraphEdge>();
		public List<ModelGraphNode> edgeNodes = null; 
		// nodes that are connected by outgoing edges, but not inluced in the subgraph
		public List<ModelGraphNode> connNodes = null; 
		// inside edges
		public List<ModelGraphEdge> ingoingEdges = new List<ModelGraphEdge>();
		public int[] numConnNodeTypes; // outgoing edges connecting node types
		public double width = 0, height = 0; // bounding box
		public List<List<MeshComponent>> historyMCs = null;
		// for facades like structure, e.g., left and right connectors
		public List<ModelGraphEdge> connEdges1 = null, connEdges2 = null;
		public List<ModelGraphNode> connNodes1 = null, connNodes2 = null;

		public bool isvertical = false;
		public double[] moveLimit = new double[2];
		public double[] widheig = new double[2];
		public double[] distToNeigs = new double[2];
		public Subgraph symSubgraph = null; // for across model subgraph replacement
		public Subgraph reflectSubgraph = null; // reflection subgraph replacement

        public bool flip = false;

		public Subgraph()
		{
			// for record, only use node information
			this.nodeSets = new List<ModelGraphNode>();
		}

		public Subgraph(Subgraph sg)
		{
			this.graphIdx = sg.graphIdx;
			this.nodeSets = new List<ModelGraphNode>();
			foreach (ModelGraphNode mn in sg.nodeSets)
				this.nodeSets.Add(mn);
			this.numConnNodeTypes = new int[MaxNumberNodeType.MAXNODETYPE];
			this.outgoingEdges = new List<ModelGraphEdge>(sg.outgoingEdges);
			this.edgeNodes = new List<ModelGraphNode>(sg.edgeNodes);
			this.connNodes = new List<ModelGraphNode>(sg.connNodes);
			this.ingoingEdges = new List<ModelGraphEdge>(sg.ingoingEdges);
            this.flip = sg.flip;
		}

		public Subgraph(int gIdx, List<ModelGraphNode> nodes, List<ModelGraphEdge> outEdges, List<ModelGraphEdge> inEdges)
		{
			this.graphIdx = gIdx;
			this.nodeSets = new List<ModelGraphNode>();
			foreach (ModelGraphNode mn in nodes)
				this.nodeSets.Add(mn);
			this.numConnNodeTypes = new int[MaxNumberNodeType.MAXNODETYPE];
			this.outgoingEdges = new List<ModelGraphEdge>();
			this.edgeNodes = new List<ModelGraphNode>();
			this.connNodes = new List<ModelGraphNode>();
			if (outEdges != null)
			{
				for (int i = 0; i < outEdges.Count; i++)
				{
					this.outgoingEdges.Add(outEdges[i]);
					// connected node types
					ModelGraphNode node1 = outEdges[i].conntectingNodes[0];
					ModelGraphNode node2 = outEdges[i].conntectingNodes[1];
					if (nodes.Contains(node1))
					{
						//if( !this.edgeNodes.Contains(node1)) 
						//may have repeated nodes, but for each edge, it is necessary
							this.edgeNodes.Add(node1);
						this.connNodes.Add(node2);
						this.numConnNodeTypes[node2.nodeType]++;
					}
					else if (nodes.Contains(node2))
					{
						//if (!this.edgeNodes.Contains(node2)) 
						this.edgeNodes.Add(node2);
						this.connNodes.Add(node1);
						this.numConnNodeTypes[node1.nodeType]++;
					}
					else
					{
						break;
					}
				}
			}
			ingoingEdges = new List<ModelGraphEdge>();
			if (inEdges != null)
			{
				ingoingEdges.AddRange(inEdges);
			}
		}

		public bool IsRoughlyMatched(Subgraph another)
		{
			bool isMatched = true;
			// --number of degree match
			if (this.outgoingEdges.Count != another.outgoingEdges.Count)
				return false;
			// --correponding type match
			for (int i = 0; i < MaxNumberNodeType.MAXNODETYPE; ++i)
			{
				if (this.numConnNodeTypes[i] != another.numConnNodeTypes[i])
				{
					isMatched = false;
					break;
				}
			}
			return isMatched;
		}

		public void OrderOutgoingEdges()
		{
			if (this.outgoingEdges.Count == 0) return;
			List<ModelGraphEdge> orderEdges = new List<ModelGraphEdge>();
			ModelGraphEdge edge = outgoingEdges[0];
			orderEdges.Add(edge);
			int eidx = 0;
			// for each node, its edges are ordered in counter clock-wise
			// locally, the adjacent edgs are ordered
			while (eidx < orderEdges.Count)
			{
				ModelGraphEdge e = orderEdges[eidx];
				foreach (ModelGraphEdge me in e.adjEdges)
				{
					if (outgoingEdges.Contains(me))
					{
						orderEdges.Add(edge);
						break;
					}					
				}
				
				eidx++;
			}
			//List<int> edgeDirs = new List<int>();
			//edgeDirs.Add(edge.faceIndex[0]);
			//if(this.nodeSets.Contains(edge.conntectingNodes[1]))
			//    edgeDirs[0] = edge.faceIndex[1];
			//counter clock-wise order (edge idx: 123456, the num should be increasing)
			//
			//while (eidx < orderEdges.Count)
			//{
			//    ModelGraphEdge e = orderEdges[eidx];				
			//    int mindiff = int.MaxValue;
			//    foreach (ModelGraphEdge me in e.adjEdges)
			//    {
			//        if (!outgoingEdges.Contains(me)) continue;
			//        int edgeDir = me.faceIndex[0];
			//        if (this.nodeSets.Contains(me.conntectingNodes[1]))
			//            edgeDir = me.faceIndex[1];
			//        int diff = edgeDir - edgeDirs[eidx];
			//        if (diff < mindiff)
			//        {
			//            mindiff = diff;
			//            edge = me;
			//        }
			//    }
			//    orderEdges.Add(edge);
			//    eidx++;
			//}
		}//OrderOutgoingEdges

		public void OrderEdgeNodes()
		{ 
			// use pca to order edge nodes
			// distinct edge nodes
			IEnumerable<ModelGraphNode> distinctEdgeNodes = this.edgeNodes.Distinct();
			int nNodes = distinctEdgeNodes.Count();
			if (nNodes == 1) return;
			double[,] mat = new double[nNodes,3];
			Vector3d[] axis = new Vector3d[2];
			Vector3d center = new Vector3d();
			List<Vector2d> projPostions = new List<Vector2d>();
			for (int i = 0; i < nNodes; ++i)
			{
				for (int j = 0; j < 3; ++j)
					mat[i, j] = distinctEdgeNodes.ElementAt(i).meshComponent.boundingBox.center.pos[j];
				center += distinctEdgeNodes.ElementAt(i).meshComponent.boundingBox.center.pos;
			}
			center /= nNodes;

			PrincipalComponentAnalysis pca = new PrincipalComponentAnalysis(mat, AnalysisMethod.Center);
			pca.Compute();
			axis[0] = new Vector3d(pca.Components[0].Eigenvector, 0);
			axis[1] = new Vector3d(pca.Components[1].Eigenvector, 0); 
			
			foreach (ModelGraphNode mn in distinctEdgeNodes)
			{
				Vector3d pos = mn.meshComponent.boundingBox.center.pos;
				Vector2d projpos = new Vector2d((pos - center).Dot(axis[0]), (pos - center).Dot(axis[1]));
				projPostions.Add(projpos);
			}

			List<int> nodeOrder = new List<int>();
			nodeOrder.Add(0);
			bool[] visited = new bool[nNodes];
			for (int i = 0; i < nNodes; ++i)
				visited[i] = false;
			visited[0] = true;

			while (true)
			{
				Vector2d currvec = projPostions[nodeOrder[nodeOrder.Count - 1]];
				Vector3d currvec3D = new Vector3d(currvec);
				double angle = 0, minAngle = double.MaxValue, maxAngle = double.MinValue;
				int nextEid = -1, nearestEid = -1;
				double thresh = 1e-2;
				for (int i = 1; i < nNodes; ++i)
				{
					if (visited[i]) continue;
					Vector3d neigvec3D = new Vector3d(projPostions[i]);
					double acosvalue = projPostions[i].Dot(currvec) / (projPostions[i].Length() * currvec.Length());
					if (currvec.Length() < thresh || projPostions[i].Length() < thresh)
						angle = 0;
					else if (Math.Abs(Math.Abs(acosvalue) - 1) < thresh)
					{
						if (acosvalue > 0)
							angle = 0;
						else
							angle = Math.PI;
					}
					else
						angle = Math.Acos(acosvalue);
					Vector3d dirz = neigvec3D.Cross(currvec3D);
					if (dirz.z > 0 && angle < minAngle)
					{
						minAngle = angle;
						nextEid = i;
					}
					else if (dirz.z <= 0 && angle > maxAngle)
					{
						maxAngle = angle;
						nearestEid = i;
					}
				}//end-for
				if (nextEid != -1)
				{
					nodeOrder.Add(nextEid);
					visited[nextEid] = true;
				}
				else if (nearestEid != -1)
				{
					nodeOrder.Add(nearestEid);
					visited[nearestEid] = true;
				}
				if (nodeOrder.Count == nNodes)
					break;				
			}//end-while
			// re-order
			ReOrder(nodeOrder, distinctEdgeNodes);
		}//OrderEdgeNodes

		private void ReOrder(List<int> nodeOrder, IEnumerable<ModelGraphNode> distinctEdgeNodes)
		{
			int nNodes = nodeOrder.Count;
			List<ModelGraphNode> orderedNodes = new List<ModelGraphNode>();
			List<ModelGraphEdge> orderedEdges = new List<ModelGraphEdge>();
			for (int i = 0; i < nNodes; ++i)
			{
				ModelGraphNode node = distinctEdgeNodes.ElementAt(nodeOrder[i]);
				int originalLoc = this.edgeNodes.IndexOf(node);
				while (originalLoc<this.edgeNodes.Count &&
					this.edgeNodes[originalLoc] == node)
				{
					orderedNodes.Add(node);
					orderedEdges.Add(this.outgoingEdges[originalLoc]);
					++originalLoc;
				}
			}
			this.edgeNodes = orderedNodes;
			this.outgoingEdges = orderedEdges;
		}//ReOrder

		public bool IsMatchedSubgraph(Subgraph other)
		{
			bool matched = false;
			IEnumerable<ModelGraphNode> eNodes1 = this.edgeNodes.Distinct();
			IEnumerable<ModelGraphNode> eNodes2 = other.edgeNodes.Distinct();
			if ( eNodes1.Count() != eNodes2.Count()) return false;

			List<int> str1 = new List<int>(), str2 = new List<int>();
			int nNodes = eNodes1.Count();
			for (int i = 0; i < nNodes; ++i)
			{
 				str1.Add(eNodes1.ElementAt(i).nodeType);
				str2.Add(eNodes2.ElementAt(i).nodeType);
			}
				
			for (int j = 0; j < nNodes; ++j)
			{
				if (str2[j] != str1[0]) continue;
				// compare
				int idx1 = 0;
				int idx2 = j;					
				while (idx1 < nNodes)
				{
					if (str2[idx2 % nNodes] != str1[idx1])
						break;
					++idx1;
					++idx2;
				}
				if (idx1 == nNodes)
				{
 					//find match
					List<int> order = new List<int>();
					for (int i = 0; i < nNodes; ++i)
						order.Add((i - j + nNodes) % nNodes);
					other.ReOrder(order, eNodes2);
					matched = true;
					break;
				}
				if (matched) break;
			}
			return matched;
		}//IsMatchedSubgraph

		public object Clone()
		{
			// copy a new subgraph with NEW-ASSIGNED MCs
			List<ModelGraphNode> sg_nodeSets = new List<ModelGraphNode>();
			List<ModelGraphEdge> sg_outgoingEdges = new List<ModelGraphEdge>();
			List<ModelGraphEdge> sg_ingoingEdges = new List<ModelGraphEdge>();

			int nedges = this.outgoingEdges.Count;
			int[] sg_connNodeTypes = new int[MaxNumberNodeType.MAXNODETYPE];

			foreach (ModelGraphNode mn in this.nodeSets)
			{
				ModelGraphNode newmn = mn.Clone() as ModelGraphNode;
				newmn.originNodeIdx = mn.nodeIdx;
				sg_nodeSets.Add(newmn);
			}

			for (int i = 0; i < this.ingoingEdges.Count; ++i)
			{
				ModelGraphEdge e = this.ingoingEdges[i];
				ModelGraphNode node1 = e.conntectingNodes[0];
				ModelGraphNode node2 = e.conntectingNodes[1];
                int index1 = this.nodeSets.IndexOf(node1);
                int index2 = this.nodeSets.IndexOf(node2);

				ModelGraphEdge edge = new ModelGraphEdge(e.pos, e.graphIdx, e.edgeIdx);
				edge.AddConnectingNode(sg_nodeSets[index1]);
				edge.AddConnectingNode(sg_nodeSets[index2]);
				sg_ingoingEdges.Add(edge);
			}

            //for (int i = 0; i < this.outgoingEdges.Count; ++i)
            //{
            //    ModelGraphEdge e = this.outgoingEdges[i];
            //    ModelGraphNode node1 = e.conntectingNodes[0];
            //    ModelGraphNode node2 = e.conntectingNodes[1];
            //    int index1 = -1;
            //    int index2 = -1;
            //    for (int j = 0; j < sg_nodeSets.Count; ++j)
            //    {
            //        if (node1.nodeIdx == sg_nodeSets[j].nodeIdx)
            //        {
            //            index1 = j;
            //            break;
            //        }
            //    }
            //    for (int j = 0; j < sg_nodeSets.Count; ++j)
            //    {
            //        if (node2.nodeIdx == sg_nodeSets[j].nodeIdx)
            //        {
            //            index2 = j;
            //            break;
            //        }
            //    }
            //    ModelGraphEdge edge = new ModelGraphEdge(e.pos, e.graphIdx, e.edgeIdx);
            //    edge.AddConnectingNode(sg_nodeSets[index1]);
            //    edge.AddConnectingNode(sg_nodeSets[index2]);
            //    sg_outgoingEdges.Add(edge);
            //}

			Subgraph sg = new Subgraph(this.graphIdx, sg_nodeSets, sg_outgoingEdges, sg_ingoingEdges);
            sg.flip = this.flip;
			return sg;
		}
	}//End -- Subgraph

	/// <summary>
	/// A graph is built on a model once it is decomposed into connected components
	/// </summary>
	public class ModelGraph : ICloneable
	{
		public string classname = "";
		public string meshfile = "";
		public string objfilename = "";
		public int graphIndex = -1; // in a graph group
		public int modelType = -1; // e.g. chair, bicycle
		public double ground_touch_ycoord = 0; // reference for model editing
		public Mesh mesh = null;
		// for edit reference
		public ModelGraph originalGraph = null;
		public List<MeshComponent> meshComponents = null;
		// coarse-level, e.g., meta (backrest) with sub-components (bars)
		public List<MeshComponent> metaComponents = null;
		//public List<Contact> contacts = null;
		public double contactThreshold = 0;
		public double flipLimit = double.MinValue;
		public int nodeCount = 0;
		public List<ModelGraphNode> graphNodes = new List<ModelGraphNode>();
		public List<ModelGraphEdge> graphEdges = new List<ModelGraphEdge>();
		// hash map of edge types
		int minPrime = -1;
		private Dictionary<ModelGraphEdge, int> edgeMap = null;
		private List<ModelGraphEdge>[] edgeClass = null;
		private List<int> edgeMapRealIndices = null;

		// display on graph view
		public List<Vector2d> nodePos = new List<Vector2d>();

		// neighboring component id + direction
		// 0 1 2 3 --- top, bottom, left, right
		public List<List<Tuple<int, int>>> neighborDirs = new List<List<Tuple<int, int>>>();
		// equaivalent node group -- subgraph
		public List<List<Subgraph>> inModelSubgraphs = new List<List<Subgraph>>();
		
		// default, nodes with same type are viewed as symmetry
		public List<List<ModelGraphNode>> nodesByType = null;
		public List<List<int>> edgePairs = null; // formed by edge pairs among all node types

		// replaceable subgraphs are fixed once found (at original graph) ---- for dynamic reference
		public List<List<Subgraph>> replaceableSubgraphs = null;
		public List<int> preGroupTypes = new List<int>();
		// node indices for searching replaceable subgraphs, those in fixed pre-group are excluded
		public List<ModelGraphNode> activeNodes = null;
		// dual/halfedge
		public List<DualEdge> dualEdges = null;
		public List<List<ModelGraphEdge>> equCutpairs = null;
		public List<Subgraph> equSubgraphs = null;
		public List<Subgraph> inModelReplaceableSubgraphs = null;
		public double[] widheig = new double[2];
		public double[] oldwidheig = new double[2];
		public List<Subgraph> reflectiveSubgraphs = null;

		public List<Face> faceCycles = null;
		public int maxLevel = -1;

		public ModelGraph()
		{ }

		public ModelGraph(List<MeshComponent> mcs)
		{
			// Initialize nodes
			this.InitModelGraph();
			int nodeIdx = 0;
			// Initialize nodes
			this.meshComponents = new List<MeshComponent>();	
			this.graphNodes = new List<ModelGraphNode>();
			foreach (MeshComponent mc in mcs)
			{
				this.meshComponents.Add(mc);
				// node type will be input after this
				ModelGraphNode mn = new ModelGraphNode(mc, this.graphIndex, nodeIdx++, -1);
				mc.graphNode = mn;
				mn.name = mc.name.Clone() as string;
				this.graphNodes.Add(mn);
			}
			this.nodeCount = mcs.Count;
		}

		public void InitModelGraph()
		{					
			// Initialize variables
			this.nodePos = new List<Vector2d>();
			this.neighborDirs = new List<List<Tuple<int, int>>>();
			this.nodesByType = new List<List<ModelGraphNode>>();
			for (int i = 0; i < MaxNumberNodeType.MAXNODETYPE; i++)
			{
				List<ModelGraphNode> types = new List<ModelGraphNode>();
				nodesByType.Add(types);
			}
		}

		public void FormGraph()
		{
			// form the graph by calculating bounding box intersection
			this.nodeCount = this.graphNodes.Count;
			//****//
			FindEdges(); // find contacts
			ConstructEdgeArray(); // edge map
			// order edge
			foreach (ModelGraphNode mgn in this.graphNodes)
				mgn.OrderEdges(0);
		}// end-Form graph

		public void FindEdges()
		{
			//--- Collect contact / edge info ---//
			// the mesh could be non-uniform vertices, using boundingbox points
			this.graphEdges = new List<ModelGraphEdge>();
			int edgeIdx = 0;
			for (int iNode = 0; iNode < nodeCount - 1; ++iNode)
			{
				for (int jNode = iNode + 1; jNode < nodeCount; ++jNode)
				{
					MeshComponent m1 = this.meshComponents[iNode];
					MeshComponent m2 = this.meshComponents[jNode];
					int isIntersect = m1.boundingBox.IntersectPoly(m2.boundingBox);
					if (isIntersect == 0)
						continue;
					Vector3d contactPoint = (m1.boundingBox.center.pos +
						m2.boundingBox.center.pos) / 2;
					ModelGraphEdge edge = new ModelGraphEdge(contactPoint, this.graphIndex, edgeIdx++);
					// the edge will be added to the node in "AddConnectingNode"
					edge.AddConnectingNode(this.graphNodes[iNode]);
					edge.AddConnectingNode(this.graphNodes[jNode]);
					this.graphEdges.Add(edge);
					// add edge to nodes
					//this.graphNodes[iNode].AddEdge(edge);
					//this.graphNodes[jNode].AddEdge(edge);
				}//end-for jNode
			}//end-for iNode
		}

		public void ConstructEdgeArray()
		{
			//--- Collect the edge for each node type pair ---///
			// #edge types = MaxNumberNodeType.MAXNODETYPE * (MaxNumberNodeType.MAXNODETYPE-1)/2 ... C(n,2)
			int edgeArray2DCount = MaxNumberNodeType.MAXNODETYPE * (MaxNumberNodeType.MAXNODETYPE + 1) / 2;
			// (0,0) (0,2)....(1,1)....(n-2,n-1) --- (p, q) totally, n(n+1)/2
			// map to  n * p - p * (p - 1)/2 + q - p
			int n = MaxNumberNodeType.MAXNODETYPE;
			edgePairs = new List<List<int>>();
			for (int i = 0; i < edgeArray2DCount; ++i)
			{
				List<int> empty = new List<int>();
				edgePairs.Add(empty);
			}
			for (int eidx = 0; eidx < this.graphEdges.Count; ++eidx)
			{
				int epair_index = EdgeMap(this.graphEdges[eidx]);
				edgePairs[epair_index].Add(eidx);
			}
		}//end-construct edeg array

		public int EdgeMap(ModelGraphEdge e)
		{
			int edgeArrayIndex = -1;
			int n = MaxNumberNodeType.MAXNODETYPE;
			int n1 = e.conntectingNodes[0].nodeIdx;
			int n2 = e.conntectingNodes[1].nodeIdx;
			int p = e.conntectingNodes[0].nodeType;
			int q = e.conntectingNodes[1].nodeType;
			if (q < p)
			{
				int tmp = p;
				p = q;
				q = tmp;
			}
			edgeArrayIndex = n * p - p * (p - 1) / 2 + q - p;
			return (edgeArrayIndex);
		}//end-edge map

		public void CollectNeighborEdges()
		{
			foreach (ModelGraphEdge mge in this.graphEdges)
				mge.adjEdges = new List<ModelGraphEdge>();
			foreach (ModelGraphNode mgn in this.graphNodes)
			{
				int nedge = mgn.edges.Count;
				// all edges with distance 1 are viewed as adj edges
				for (int i = 0; i < nedge - 1; ++i)
				{
					ModelGraphEdge iedge = mgn.edges[i];
					for (int j = i + 1; j < nedge; ++j)
					{
						ModelGraphEdge jedge = mgn.edges[j];
						if (!iedge.adjEdges.Contains(jedge))
						{
							iedge.adjEdges.Add(jedge);
							jedge.adjEdges.Add(iedge);
						}
					}
				}
			}//end-each-node
		}//end-adj-edges

		public void ExtractEquNodes()
		{
			// equivalent nodes mean:
			// --equal node type
			// --degree of edges is the same
			// --each type of edges (connected node type) matches
			// the set covers all nodes, each group includes at least 1 node
			bool[] visited = new bool[nodeCount];
			for (int i = 0; i < nodeCount; i++)
				visited[i] = false;
			for (int iNode = 0; iNode < nodeCount; iNode++)
			{
				if (visited[iNode]) continue;
				List<ModelGraphNode> inodeSets = new List<ModelGraphNode>();
				inodeSets.Add(this.graphNodes[iNode]);
				List<ModelGraphEdge> ioutEdges = GetOutgoingEdges(inodeSets);
				List<ModelGraphEdge> iinEdges = GetIngoingEdges(inodeSets);
				Subgraph isg = new Subgraph(this.graphIndex, inodeSets, ioutEdges, iinEdges); 
				List<Subgraph> subgraphs = new List<Subgraph>();
				subgraphs.Add(isg);
				// begin search
				for (int jNode = iNode + 1; jNode < nodeCount; jNode++)
				{
					if (visited[jNode]) continue;
					List<ModelGraphNode> jnodeSets = new List<ModelGraphNode>();
					jnodeSets.Add(this.graphNodes[jNode]);
					List<ModelGraphEdge> joutEdges = GetOutgoingEdges(jnodeSets);
					List<ModelGraphEdge> jinEdges = GetIngoingEdges(jnodeSets);
					List<MeshComponent> jMCs = new List<MeshComponent>();
					jMCs.Add(this.meshComponents[jNode]);
					Subgraph jsg = new Subgraph(this.graphIndex, jnodeSets, joutEdges, jinEdges); 
					if (isg.IsRoughlyMatched(jsg))
					{
						subgraphs.Add(jsg);
						visited[jNode] = true;
					}
				}//end - for jnode
				visited[iNode] = true;
				if (subgraphs.Count > 1)
				{
					this.inModelSubgraphs.Add(subgraphs);
				}
			}//end - for inode
		}//end-search-qual-nodes

		public Subgraph GetSubgraph(List<ModelGraphNode> nodeSets)
		{
			Subgraph sg;
			List<ModelGraphEdge> outEdges = GetOutgoingEdges(nodeSets);
			List<ModelGraphEdge> inEdges = GetIngoingEdges(nodeSets);
			sg = new Subgraph(this.graphIndex, nodeSets, outEdges, inEdges);
			return sg;
		}

		public List<ModelGraphEdge> GetOutgoingEdges(List<ModelGraphNode> nodeSets)
		{
			// given a set of nodes, list all outgoing edges (inside connections are excluded)
			// boundary node + connected node + edge
			List<ModelGraphEdge> edgeIdxs = new List<ModelGraphEdge>();
			for (int eidx = 0; eidx < this.graphEdges.Count; ++eidx)
			{
				if (this.graphEdges[eidx].isDummyEdge) continue;
				ModelGraphNode node1 = this.graphEdges[eidx].conntectingNodes[0];
				ModelGraphNode node2 = this.graphEdges[eidx].conntectingNodes[1];
				int nodeType1 = node1.nodeType;
				int nodeType2 = node2.nodeType;
				if ( (nodeSets.Contains(node1) && !nodeSets.Contains(node2)) ||
					(nodeSets.Contains(node2) && !nodeSets.Contains(node1)))
					edgeIdxs.Add(this.graphEdges[eidx]);
			}
			return edgeIdxs;
		} //end-get outgoing edges

		public List<ModelGraphEdge> GetIngoingEdges(List<ModelGraphNode> nodeSets)
		{
			// given a set of nodes, list all ingoing edges 
			// edge index
			List<ModelGraphEdge> edgeIdxs = new List<ModelGraphEdge>();
			for (int eidx = 0; eidx < this.graphEdges.Count; ++eidx)
			{
				if (this.graphEdges[eidx].isDummyEdge) continue;
				ModelGraphNode node1 = this.graphEdges[eidx].conntectingNodes[0];
				ModelGraphNode node2 = this.graphEdges[eidx].conntectingNodes[1];
				if (nodeSets.Contains(node1) && nodeSets.Contains(node2))
					edgeIdxs.Add(this.graphEdges[eidx]);
			}
			return edgeIdxs;
		} //end-get ingoing edges

		public List<ModelGraphNode> GetCompleteSubgraphNodeSets(List<ModelGraphNode> nodeSets)
		{
			List<ModelGraphNode> comNodes = new List<ModelGraphNode>();
			comNodes.AddRange(nodeSets);
			foreach (ModelGraphNode mn in this.graphNodes)
			{
				if (comNodes.Contains(mn)) continue;
				bool toInclude = true;
				// check whether this node only connects to the node in "nodeSets"
				// and have no other connections, then it should be included to "nodeSets"
				foreach (ModelGraphEdge e in mn.edges)
				{
					if ((e.conntectingNodes[0] == mn &&
						!nodeSets.Contains(e.conntectingNodes[1])) ||
						(e.conntectingNodes[1] == mn &&
						!nodeSets.Contains(e.conntectingNodes[0])))
					{
						toInclude = false;
						break;
					}
				}
				if (toInclude)
				{
					comNodes.Add(mn);
				}
			}
			return comNodes;
		}
		public void SetNodePositions(int width, int height)
		{
			// width/height: size of meshview panel 
			Vector3d mincoord = this.mesh.MinCoord();
			Vector3d maxcoord = this.mesh.MaxCoord();
			double xratio = (double)width / (maxcoord.x - mincoord.x);
			double yratio = (double)height / (maxcoord.y - mincoord.y);
			this.nodePos.Clear();
			foreach (MeshComponent mc in this.meshComponents)
			{
				Point3 center3d = mc.boundingBox.center;
				Vector2d center2d = new Vector2d(center3d.pos.x, center3d.pos.y);
				center2d = new Vector2d(xratio * (center2d.x - mincoord.x), yratio * (center2d.y - mincoord.y));
				center2d = new Vector2d(width - center2d.x, height - center2d.y);
				this.nodePos.Add(center2d);
			}
		}

		public object Clone()
		{
			ModelGraph g = new ModelGraph();
			g.InitModelGraph();
			g.graphNodes = new List<ModelGraphNode>();
			g.meshComponents = new List<MeshComponent>();
			g.objfilename = this.objfilename.Clone() as string;
			foreach (ModelGraphNode mn in this.graphNodes)
			{
				MeshComponent newmc = mn.meshComponent.Clone() as MeshComponent;
				g.meshComponents.Add(newmc);
				newmc.colorIdx = mn.nodeType;
				ModelGraphNode newmn = new ModelGraphNode(newmc, this.graphIndex, mn.nodeIdx, mn.nodeType);
				newmn.pos3D = mn.pos3D;
				newmn.name = mn.name.Clone() as string;
				newmc.graphNode = newmn;
				newmn.level = mn.level;
                newmn.isgroundcontact = mn.isgroundcontact;
				g.graphNodes.Add(newmn);
			}
			g.modelType = this.modelType;
			g.classname = this.classname.Substring(0);
			g.meshfile = this.meshfile.Clone() as string;
			g.graphIndex = this.graphIndex;
			g.mesh = new Mesh(this.mesh);
			g.objfilename = this.objfilename.Clone() as string;
           

			g.graphEdges = new List<ModelGraphEdge>();
			for (int i = 0; i < this.graphEdges.Count; i++)
			{
				int mcIdx1 = this.graphEdges[i].conntectingNodes[0].nodeIdx;
				int mcIdx2 = this.graphEdges[i].conntectingNodes[1].nodeIdx;
				ModelGraphEdge edge = new ModelGraphEdge(this.graphEdges[i].pos, this.graphIndex, this.graphEdges[i].edgeIdx);
				edge.AddConnectingNode(g.graphNodes[mcIdx1]);
				edge.AddConnectingNode(g.graphNodes[mcIdx2]);
				edge.faceIndex[0] = this.graphEdges[i].faceIndex[0];
				edge.faceIndex[1] = this.graphEdges[i].faceIndex[1];
				edge.support = this.graphEdges[i].support;
				edge.level = this.graphEdges[i].level;
				edge.cycleIndices = new List<int>();
				g.graphEdges.Add(edge);
			}

            g.faceCycles = new List<Face>();
			if (this.faceCycles != null)
			{
				int fcIdx = 0;
				foreach (Face face in this.faceCycles)
				{
					Face gFace = new Face();
					foreach (ModelGraphNode mn in face.vertices)
					{
						int mcIdx = mn.nodeIdx;
						gFace.vertices.Add(g.graphNodes[mcIdx]);
					}
					foreach (ModelGraphEdge me in face.edges)
					{
						int meIdx = me.edgeIdx;
						gFace.edges.Add(g.graphEdges[meIdx]);
						g.graphEdges[meIdx].cycleIndices.Add(fcIdx);
					}
					g.faceCycles.Add(gFace);
				}
			}
			// adj nodes/edges
			foreach (ModelGraphNode mn in g.graphNodes)
				mn.OrderEdges(0);
			g.CollectNeighborEdges();
			
			g.nodeCount = this.nodeCount;
			g.graphIndex = this.graphIndex;
			g.preGroupTypes = new List<int>();
			g.preGroupTypes.AddRange(this.preGroupTypes);
			g.nodePos = new List<Vector2d>();
			g.nodePos.AddRange(this.nodePos);
			// replaceable subgraphs will not be copied, unless additional requirments
			// copied in other places
			return g;
		}

		public void MakeSelfCopy()
		{
			// keep original graph for edit reference
			this.originalGraph = this.Clone() as ModelGraph;
			// re-construct replaceable subgraphs
			// re-direct nodes to original graph
			// only keep the node index is enough
			if (this.replaceableSubgraphs != null)
			{
				this.originalGraph.replaceableSubgraphs = new List<List<Subgraph>>();
				foreach (List<Subgraph> sgs in this.replaceableSubgraphs)
				{
					List<Subgraph> newSGs = new List<Subgraph>();
					foreach (Subgraph sg in sgs)
					{
						Subgraph newsg = new Subgraph();
						newsg.graphIdx = sg.graphIdx;
						newsg.nodeSets = new List<ModelGraphNode>();
						foreach (ModelGraphNode mn in sg.nodeSets)
						{
							ModelGraphNode newmn = new ModelGraphNode(null, mn.graphIdx, mn.nodeIdx, mn.nodeType);
							newmn.name = mn.name;
							newsg.nodeSets.Add(newmn);
						}
						newSGs.Add(newsg);
					}
					this.originalGraph.replaceableSubgraphs.Add(newSGs);
				}//end-copy-replaceable-subgraph-chair-like-structure
			}
			if (this.inModelReplaceableSubgraphs != null)
			{
				this.originalGraph.inModelReplaceableSubgraphs = new List<Subgraph>();
				foreach (Subgraph sg in this.inModelReplaceableSubgraphs)
				{
					Subgraph newsg = new Subgraph();
					newsg.graphIdx = sg.graphIdx;
					newsg.nodeSets = new List<ModelGraphNode>();
					foreach (ModelGraphNode mn in sg.nodeSets)
					{
						ModelGraphNode newmn = new ModelGraphNode(null, mn.graphIdx, mn.nodeIdx, mn.nodeType);
						newsg.nodeSets.Add(newmn);
					}
					this.originalGraph.inModelReplaceableSubgraphs.Add(newsg);
				}//end-copy-inModelReplaceable-subgraph-facade-like-structure
			}
		}

		public void ResetGraph()
		{ 
			// after editing, restore from saved original graph
			this.InitModelGraph();
			int nodeIdx = 0;
			this.graphNodes = new List<ModelGraphNode>();
			this.meshComponents = new List<MeshComponent>();
			foreach (ModelGraphNode mn in this.originalGraph.graphNodes)
			{
				MeshComponent newmc = mn.meshComponent.Clone() as MeshComponent;
				this.meshComponents.Add(newmc);
				newmc.colorIdx = mn.nodeType;
				ModelGraphNode newmn = new ModelGraphNode(newmc, this.graphIndex, nodeIdx++, mn.nodeType);
				newmc.graphNode = newmn;
				newmn.name = mn.name.Clone() as string;
				this.graphNodes.Add(newmn);
			}
			this.mesh = new Mesh(this.originalGraph.mesh);
			this.graphEdges = new List<ModelGraphEdge>();
			for (int i = 0; i < this.originalGraph.graphEdges.Count; i++)
			{
				int mcIdx1 = this.originalGraph.graphEdges[i].conntectingNodes[0].nodeIdx;
				int mcIdx2 = this.originalGraph.graphEdges[i].conntectingNodes[1].nodeIdx;
				ModelGraphEdge edges = new ModelGraphEdge(this.originalGraph.graphEdges[i].pos, this.originalGraph.graphIndex, i);
				edges.AddConnectingNode(this.graphNodes[mcIdx1]);
				edges.AddConnectingNode(this.graphNodes[mcIdx2]);
				this.graphEdges.Add(edges);
			}
			this.nodeCount = this.originalGraph.nodeCount;
			this.preGroupTypes = new List<int>();
			this.preGroupTypes.AddRange(this.originalGraph.preGroupTypes);
			this.nodePos = new List<Vector2d>();
			this.nodePos.AddRange(this.originalGraph.nodePos);
		}

		public void AddEdge(ModelGraphEdge e)
		{
			if (this.graphEdges == null) this.graphEdges = new List<ModelGraphEdge>();
			if (!this.graphEdges.Contains(e))
			{
				this.graphEdges.Add(e);
			}
		}

		public void AddDummyEdges()
		{
			if (this.modelType == 0)
			{
				// add dummy edges for each node 
				foreach (ModelGraphEdge me in this.graphEdges)
					me.adjEdges = new List<ModelGraphEdge>();
				foreach (ModelGraphNode mn in this.graphNodes)
				{
					List<ModelGraphEdge> dummyEdges = mn.OneRingTraversal(false);
					//if (dummyEdges == null || dummyEdges.Count == 0) continue;
					//foreach (ModelGraphEdge mge in dummyEdges)
					//    mge.edgeIdx = this.graphEdges.Count;
					//this.graphEdges.AddRange(dummyEdges);
				}
				return;
			}
			this.BuildDualGraph(1);
			//return;
			// for facades, only add bouncary connection
			// boundary nodes
			List<ModelGraphNode> ringNodes = new List<ModelGraphNode>();
			List<DualEdge> onering = new List<DualEdge>();
			for (int i = 0; i < this.dualEdges.Count; ++i)
			{
				DualEdge de = this.dualEdges[i];
				ModelGraphNode origin = de.fromNode;
				DualEdge next = de.nextEdge;
				onering = new List<DualEdge>();
				onering.Add(de);
				ringNodes = new List<ModelGraphNode>();
				ringNodes.Add(de.fromNode);
				ringNodes.Add(de.toNode);
				while (true)
				{
					onering.Add(next);
					if (!ringNodes.Contains(next.toNode))
						ringNodes.Add(next.toNode);
					if (next.toNode == origin)
					{
						break;
					}
					next = next.nextEdge;
				}
				if (onering.Count > 6)
					break;
			}
			Vector3d maxbound = new Vector3d(double.MinValue, double.MinValue, double.MinValue),
				minbound = new Vector3d(double.MaxValue, double.MaxValue,double.MaxValue);
			foreach (ModelGraphNode mn in ringNodes)
			{
				maxbound = Vector3d.Max(maxbound, mn.meshComponent.MaxCoord());
				minbound = Vector3d.Min(minbound, mn.meshComponent.MinCoord());
			}
			List<ModelGraphNode> northBounds = new List<ModelGraphNode>(),
				southBounds = new List<ModelGraphNode>(),
				westBounds = new List<ModelGraphNode>(),
				eastBounds = new List<ModelGraphNode>();
			double thresh_x = (maxbound.x - minbound.x) / 6,
				thresh_y = (maxbound.y - minbound.y) / 6;
			foreach (ModelGraphNode mn in ringNodes)
			{
				if (Math.Abs(mn.meshComponent.MaxCoord().x - maxbound.x) < thresh_x)
				{
					eastBounds.Add(mn);
				}
				if (Math.Abs(mn.meshComponent.MaxCoord().y - maxbound.y) < thresh_y)
				{
					northBounds.Add(mn);
				}
				if (Math.Abs(mn.meshComponent.MinCoord().x - minbound.x) < thresh_x)
				{
					westBounds.Add(mn);
				}
				if (Math.Abs(mn.meshComponent.MinCoord().y - minbound.y) < thresh_y)
				{
					southBounds.Add(mn);
				}
			}
			// higheset and lowest are fixed, in case left and right bound not satisfied
			List<List<ModelGraphNode>> vbounds = new List<List<ModelGraphNode>>();
			vbounds.Add(northBounds);
			vbounds.Add(southBounds);
			for (int i = 0; i < 2; ++i)
			{
				for (int j = 0; j < vbounds[i].Count; ++j)
				{
					int k = 0;
					for (; k < vbounds[i].Count; ++k)
					{
						if(j == k) continue;
						if (vbounds[i][k].meshComponent.MinCoord().x <
							vbounds[i][j].meshComponent.MinCoord().x)
							break;
					}
					if (k >= vbounds[i].Count)
					{
						if (!westBounds.Contains(vbounds[i][j]))
							westBounds.Add(vbounds[i][j]);
					}
					k = 0;
					for (; k < vbounds[i].Count; ++k)
					{
						if (j == k) continue;
						if (vbounds[i][k].meshComponent.MaxCoord().x >
							vbounds[i][j].meshComponent.MaxCoord().x)
							break;
					}
					if (k >= vbounds[i].Count)
					{
						if (!eastBounds.Contains(vbounds[i][j]))
							eastBounds.Add(vbounds[i][j]);
					}
				}
			}
			// in each side, all boundary nodes can access to each other for cut facility
			List<List<ModelGraphNode>> boundaryNodes = new List<List<ModelGraphNode>>();
			// the order of newly inserted edges following counter-clockwise order
			boundaryNodes.Add(northBounds);
			boundaryNodes.Add(westBounds);
			boundaryNodes.Add(southBounds);
			boundaryNodes.Add(eastBounds);
			List<ModelGraphEdge> handled = new List<ModelGraphEdge>();
			for (int i = 0; i < 4; ++i)
			{
				for (int j = 0; j < boundaryNodes[i].Count; ++j)
				{
					ModelGraphNode jnode = boundaryNodes[i][j];
					// index the boundary edge
					int index = -1;
					List<ModelGraphEdge> bedges = new List<ModelGraphEdge>();
					for(int n = 0; n < jnode.edges.Count; ++n)
					{
						if (jnode.edges[n].isDummyEdge) continue;
						if (boundaryNodes[i].Contains(jnode.adjNodes[n]))
						{
							index = n;
							if (!handled.Contains(jnode.edges[index]))
							{
								bedges.Add(jnode.edges[index]);
							}
						}
					}
					if (bedges.Count == 0)
					{
						continue;
					}
					for (int bi = 0; bi < bedges.Count; ++bi)
					{
						ModelGraphEdge jedge = bedges[bi];
						handled.Add(jedge);
						for (int k = 0; k < boundaryNodes[i].Count; ++k)
						{
							if (j == k) continue;
							ModelGraphNode knode = boundaryNodes[i][k];
							if (!jnode.adjNodes.Contains(knode))
							{
								Vector3d pos = (jnode.pos3D + knode.pos3D) / 2;
								ModelGraphEdge me = new ModelGraphEdge(pos, this.graphIndex, this.graphEdges.Count);
								me.isDummyEdge = true;
								me.AddConnectingNode(jnode);
								me.AddConnectingNode(knode);
								jnode.adjNodes.Add(knode);
								knode.adjNodes.Add(jnode);
								this.graphEdges.Add(me);
								jedge.adjEdges.Add(me);
								me.adjEdges = new List<ModelGraphEdge>();
								me.adjEdges.Add(jedge);
							}
							else
							{
								int eidx = jnode.adjNodes.IndexOf(knode);
								//if (jnode.edges[eidx].isDummyEdge)
								//{
									jnode.edges[eidx].adjEdges.Add(jedge);
									jedge.adjEdges.Add(jnode.edges[eidx]);
								//}
							}
						}
					}
				}
			}//end-for
			
		}//End-add-dummyEdges

		public bool BuildDualGraph(int tag)
		{
			// buld dual graph 			
			// 1. project points to tangent plane 
			// the normals can be changed using mesh normal 
			// default z-axis
			bool issucess = true;
			Vector3d normal = new Vector3d(0, 0, 1);
			List<Tuple<ModelGraphNode, ModelGraphNode>> edgemap = new List<Tuple<ModelGraphNode, ModelGraphNode>>();
			List<DualEdge> halfedges = new List<DualEdge>();
			this.dualEdges = new List<DualEdge>();
			int numDualEdges = 0;

			//clear 
			foreach (ModelGraphNode mn in this.graphNodes)
				mn.dualEdges = null;
			foreach (ModelGraphEdge e in this.graphEdges)
			{
				e.dualEdges = new DualEdge[2];
				e.adjEdges = new List<ModelGraphEdge>();
			}

			// add adj components
			foreach (ModelGraphNode mn in this.graphNodes)
			{
				if (mn.edges == null) continue;
				mn.adjNodes = new List<ModelGraphNode>();
				foreach (ModelGraphEdge e in mn.edges)
				{
					if (e.conntectingNodes[0] == mn)
						mn.adjNodes.Add(e.conntectingNodes[1]);
					else
						mn.adjNodes.Add(e.conntectingNodes[0]);
				}
			}

			foreach (ModelGraphNode mn in this.graphNodes)
			{
				// input start neighboring ModelGraphNode and n#dirs
				if (mn.edges == null || mn.edges.Count == 0)
				{
					continue;
				}
				//**Order**//
				mn.OrderEdges(null, 4);
				//mn.OrderEdges(tag);
				mn.dualEdges = new List<DualEdge>();
				// add edge
				for (int eidx = 0; eidx < mn.edges.Count; ++eidx)
				{
					ModelGraphEdge e = mn.edges[eidx];
					ModelGraphNode neigMN = null;
					if (e.conntectingNodes[0] == mn)
						neigMN = e.conntectingNodes[1];
					else
						neigMN = e.conntectingNodes[0];
					DualEdge dualedge = new DualEdge(neigMN, mn, null, null, null, e);
					int index = edgemap.IndexOf(new Tuple<ModelGraphNode, ModelGraphNode>(mn, neigMN));
					if (index == -1)
						index = edgemap.IndexOf(new Tuple<ModelGraphNode, ModelGraphNode>(neigMN, mn));
					if (index != -1)
					{
						// already record
						DualEdge de = halfedges[index];
						de.inverseEdge = dualedge;
						dualedge.inverseEdge = de;
					}
					else
					{
						edgemap.Add(new Tuple<ModelGraphNode, ModelGraphNode>(mn, neigMN));
						halfedges.Add(dualedge);
					}
					dualedge.edgeID = numDualEdges++;
					//dualedge.dir = mn.edgeOrders[eidx];
					if (e.dualEdges[0] == null)
						e.dualEdges[0] = dualedge;
					else if (e.dualEdges[1] == null)
						e.dualEdges[1] = dualedge;
					else
					{
						Program.OutputText("wrong dual-edge table!", true);
						break;
					}
					dualedge.oneRingNeigs = new List<DualEdge>();
					mn.dualEdges.Add(dualedge); // ordered
					this.dualEdges.Add(dualedge);
				}//end-one node
			}//end-for each MC
			//return true;
			// 2. build next and previous pointer
			foreach (DualEdge de in this.dualEdges)
			{
				ModelGraphNode dest = de.toNode;
				int index = dest.dualEdges.IndexOf(de.inverseEdge);
				if (index == -1)
				{
					Program.OutputText("wrong dual-edge table!", true);
					break;
				}
				//counter-clock-wise
				int nneig = dest.dualEdges.Count;
				int next = (index - 1 + nneig) % nneig;
				de.nextEdge = dest.dualEdges[next];
				dest.dualEdges[next].prevEdge = de;
			}
			//return true;
			// 3. traverse all nodes find one-ring
			bool[] visited = new bool[numDualEdges];
			bool[,] edgePairs = new bool[numDualEdges, numDualEdges];
			for (int i = 0; i < numDualEdges; ++i)
				visited[i] = false;
			for (int i = 0; i < numDualEdges; ++i)
			{
				if (visited[i]) continue;
				DualEdge de = this.dualEdges[i];
				ModelGraphNode origin = de.fromNode;
				DualEdge next = de.nextEdge;
				visited[i] = true;
				List<DualEdge> onering = new List<DualEdge>();
				bool isFace = false;
				onering.Add(de);
				List<ModelGraphNode> ringNodes = new List<ModelGraphNode>();
				ringNodes.Add(de.fromNode);
				ringNodes.Add(de.toNode);
				while (true)
				{
					onering.Add(next);
					if(!ringNodes.Contains(next.toNode))
						ringNodes.Add(next.toNode);
					visited[next.edgeID] = true;
					// at boundary, the next of a half-edge will go through the whole boundary
					// since counter-clock-wise is used, the previous connection has to be also on the boundary, see note
					if (next.toNode == origin)
					{
						isFace = true;
						break;
					}
					next = next.nextEdge;
				}
				if (!isFace) 
					continue;
			}//end-for
			return issucess;
		}//End-build-dual-graph

		public void AssignIndex()
		{
			int idx = 0;
			foreach (ModelGraphNode mn in this.graphNodes)
				mn.nodeIdx = idx++;
			idx = 0;
			foreach (ModelGraphEdge me in this.graphEdges)
				me.edgeIdx = idx++;
		}

		public List<Face> GetAllFaces()
		{
			// collect all circles from the graph
			List<Face> faces = new List<Face>();
			// @t, 0,..t-1: visited, to avoid repetition
			int edgecount = this.graphEdges.Count;
			bool[] visited = new bool[edgecount*2];
			for(int eidx = 0; eidx < edgecount; eidx+=2)
			{ 
				ModelGraphEdge me = this.graphEdges[eidx];
				ModelGraphNode fromNode = me.conntectingNodes[0],
					toNode = me.conntectingNodes[1];
				if(eidx%2 ==1)
				{
					fromNode = me.conntectingNodes[1];
					toNode = me.conntectingNodes[0];
				}
				visited[eidx * 2] = true;
				//visited[eidx * 2 + 1] = true;
				bool[] labeled = new bool[edgecount * 2];
				TreeNode tnode = new TreeNode(me.edgeIdx, eidx, null);
				List<TreeNode> tree = new List<TreeNode>();
				tree.Add(tnode);
				TreeNode root = tnode;
				ModelGraphNode rootNode = fromNode;
				labeled[eidx] = true;
				while (tree.Count > 0)
				{
					TreeNode currTNode = tree[0];
					ModelGraphEdge currEdge = this.graphEdges[tree[0].index];
					int edgeMapId = tree[0].eindex;
					fromNode = currEdge.conntectingNodes[1]; // end toNode
					if (currTNode.eindex % 2 == 1)
					{
						fromNode = currEdge.conntectingNodes[0];
					}
					if (currTNode.index == 15)
						labeled[edgeMapId] = true;
					labeled[edgeMapId] = true;
					tree.RemoveAt(0);
					foreach (ModelGraphEdge mge in fromNode.edges)
					{
						if (mge == currEdge) continue;
						if (mge.edgeIdx == 3)
							edgeMapId = 0;
						toNode = mge.conntectingNodes[0] == fromNode ?
							mge.conntectingNodes[1] : mge.conntectingNodes[0];
						edgeMapId = mge.edgeIdx * 2;
						if (mge.conntectingNodes[1] == fromNode)
							++edgeMapId;
						if (visited[edgeMapId] || labeled[edgeMapId]) continue;
						labeled[edgeMapId] = true;
						// previous nodes must have traversed all circles
						if (toNode == rootNode && currTNode.parent != root && mge.edgeIdx != root.index )
						{
							// find a circle
							Face face = new Face();
							//ModelGraphEdge childEdge = this.graphEdges[tnode.index];
							ModelGraphNode childNode = fromNode;
							face.AddElement(childNode, mge);
							tnode = currTNode;
							while (tnode != null)
							{
								ModelGraphEdge parentEdge = this.graphEdges[tnode.index];
								ModelGraphNode parentNode = parentEdge.conntectingNodes[0];
								if (parentNode == childNode)
									parentNode = parentEdge.conntectingNodes[1];
								face.AddElement(parentNode, parentEdge);
								tnode = tnode.parent;
								childNode = parentNode;
							}
							
							if (face.vertices.Count < 3) continue;
							// check repetition
							bool exist = false;
							foreach (Face f in faces)
							{
								if (face.Included(f))
								{
									exist = true;
									break;
								}
							}
							if(!exist)
								faces.Add(face);
							break; // this branch has to be terminated, otherwise redundancy
						}
						//if (visited[edgeMapId]) continue;
						tnode = new TreeNode(mge.edgeIdx, edgeMapId, currTNode);
						currTNode.children.Add(tnode);
						tree.Add(tnode);
					}
				}//end-while
			}//end-for-all-circles			
			return faces;
		}//End-get-faces

		bool[] nodeVisited = null;
		List<TreeNode> nodePropogationTree = null;
		List<Face> cycles = null;
		public bool DFSCycle(TreeNode treenode, ModelGraphNode root)
		{
			TreeNode node = null;
			if (treenode.parent != null && treenode.parent.index != root.nodeIdx &&
				this.graphNodes[treenode.index].adjNodes.Contains(root))
			{
				// find a circle
				Face face = new Face();
				TreeNode tnode = treenode;
				ModelGraphNode currentNode = root;
				while (tnode != null)
				{
					ModelGraphNode parentNode = this.graphNodes[tnode.index];
					ModelGraphEdge theEdge = null;
					foreach (ModelGraphEdge me in parentNode.edges)
					{
						if (me.conntectingNodes.Contains(currentNode))
						{
							theEdge = me;
							break;
						}
					}
					face.AddElement(parentNode, theEdge);
					tnode = tnode.parent;
					currentNode = parentNode;
				}
				if (face.vertices.Count > 2)
					cycles.Add(face);
				return true;
			}
			foreach (ModelGraphNode mn in this.graphNodes[treenode.index].adjNodes)
			{
				if (nodeVisited[mn.nodeIdx]) continue;
				node = new TreeNode(mn.nodeIdx, mn.nodeIdx, treenode);
				nodePropogationTree.Add(node);
				treenode.children.Add(node);
				nodeVisited[mn.nodeIdx] = true;
				DFSCycle(node, root);
				nodeVisited[mn.nodeIdx] = false;
			}
			return false;
		}//End-DFS

		public void GetCycles()
		{
			// collect all circles from the graph
			int nnodes = this.graphNodes.Count;
			cycles = new List<Face>();
			foreach (ModelGraphNode mn in this.graphNodes)
			{
				nodeVisited = new bool[nnodes];
				nodePropogationTree = new List<TreeNode>();
				TreeNode node = new TreeNode(mn.nodeIdx, mn.nodeIdx, null);
				nodePropogationTree.Add(node);
				nodeVisited[mn.nodeIdx] = true;
				DFSCycle(node, mn);
			}
			//filter
			for(int i = 0; i < cycles.Count-1; ++i)
			{
				Face i_face = cycles[i];
				for (int j = i + 1; j < cycles.Count; ++j)
				{
					Face j_face = cycles[j];
					//int included = Included(i_face, j_face);
					double thresh = 0.8;
					if (this.modelType == 9) thresh = 0.2;
					//if (this.modelType == 8) thresh = 0.6;
					int included = Included(i_face, j_face, thresh);
					if (included == 1)
					{
						cycles.RemoveAt(i);
						--i;
						break;
					}
					else if (included == 2)
					{
						cycles.RemoveAt(j);
						--j;
					}
				}
			}
			
		}//End-cycles

		public int Included(Face f1, Face f2, double thresh)
		{
			if (f1.vertices.Count >= f2.vertices.Count)
			{
				int nthresh = (int)Math.Ceiling(f2.vertices.Count * thresh);
				if (f2.vertices.Count == 3 )// || f2.vertices.Count == 4)
					nthresh = 3;
				int ncontain = 0;
				foreach (ModelGraphNode mn in f2.vertices)
				{
					if (f1.vertices.Contains(mn))
						ncontain++;
				}
				if (ncontain >= nthresh)
					return 1;
				else
					return 0;
			}
			else
			{
				int nthresh = (int)Math.Ceiling(f1.vertices.Count * thresh);
				if (f1.vertices.Count == 3)// || f1.vertices.Count == 4)
					nthresh = 3;
				int ncontain = 0;
				foreach (ModelGraphNode mn in f1.vertices)
				{
					if (f2.vertices.Contains(mn))
						ncontain++;
				}
				if (ncontain >= nthresh)
					return 2;
				else
					return 0;
			}
		}//end-included

		public int Included(Face f1, Face f2)
		{
			if (f1.vertices.Count >= f2.vertices.Count)
			{
				foreach (ModelGraphNode mn in f2.vertices)
				{
					if (!f1.vertices.Contains(mn))
						return 0;
				}
				return 1;
			}
			else
			{
				foreach (ModelGraphNode mn in f1.vertices)
				{
					if (!f2.vertices.Contains(mn))
						return 0;
				}
				return 2;
			}
		}//end-included

		public void CompleteAdjEdgesByCycles(bool withcycle)
		{ 
			// add dummy edges without redundancy
			foreach (ModelGraphEdge mge in this.graphEdges)
			{
				mge.adjEdges = new List<ModelGraphEdge>();
				mge.cycleIndices = new List<int>();
			}

			AssignIndex();


			foreach (ModelGraphNode mn in this.graphNodes)
				mn.AddAdjEdges();

				//mn.OneRingTraversal(false);
			if (!withcycle) return;

				//***1. locate all circles
                ////List<Face> faces = GetAllFaces();	
                this.GetCycles();
                this.faceCycles = cycles;
				//***2. all edges in the cycle can navigate to the other
				int faceIdx = 0;
				foreach (Face face in this.faceCycles)
				{
					int nedges = face.edges.Count;
					for (int i = 0; i < nedges - 1; ++i)
					{
						ModelGraphEdge e1 = face.edges[i];
						for (int j = i + 1; j < nedges; ++j)
						{
							ModelGraphEdge e2 = face.edges[j];
							if (!e1.adjEdges.Contains(e2) && e1!= e2)
							{
								e1.adjEdges.Add(e2);
								e2.adjEdges.Add(e1);
							}
						}
					}
					foreach (ModelGraphEdge me in face.edges)
						me.cycleIndices.Add(faceIdx);
					++faceIdx;
				}//end-face-iteration

			foreach (ModelGraphEdge me in this.graphEdges)
			{
				me.trueAdjEdges = new List<bool>();
				foreach (ModelGraphEdge mge in me.adjEdges)
					me.trueAdjEdges.Add(true);
			}
		}//End-complete-graph

		public void AddDummyEdgesBydistance(int maxdistance)
		{
			// its initial adj edges have distance 0
			int[] nInitAdjEdgs = new int[this.graphEdges.Count];
			for (int i = 0; i < this.graphEdges.Count; ++i )
				nInitAdjEdgs[i] = this.graphEdges[i].adjEdges.Count;

			// ATTENTION!!!!!!
			// first save all neighbors within the given distance to an array
			// then assign to the edges, otherwise all new added neighbors will be
			// counted for the following edges
			List<List<ModelGraphEdge>> distEdges = new List<List<ModelGraphEdge>>();

			foreach (ModelGraphEdge me in this.graphEdges)
			{
				List<ModelGraphEdge> adjEdges = new List<ModelGraphEdge>();
				//me.trueAdjEdges = new List<bool>();
				int start = 0, startIdx = 0;
				adjEdges.AddRange(me.adjEdges);
				int ndist0 = me.adjEdges.Count;
				while (start < maxdistance)
				{
					int ncurrent = adjEdges.Count;
					for (int i = startIdx; i < ncurrent; ++i)
					{
						ModelGraphEdge i_adjEdge = adjEdges[i];
						foreach (ModelGraphEdge adjme in i_adjEdge.adjEdges)
						{
							if (!me.adjEdges.Contains(adjme) && !adjEdges.Contains(adjme)
								&& adjme != me)
							{
								adjEdges.Add(adjme);
							}
						}
					}
					++start;
					startIdx += ncurrent;
				}
				for (int i = 0; i < ndist0; ++i)
				{
					adjEdges.RemoveAt(0);
					//me.trueAdjEdges.Add(true);
				}
				distEdges.Add(adjEdges);				
			}
			
			for (int index = 0; index < this.graphEdges.Count; ++index )
			{
				ModelGraphEdge me = this.graphEdges[index];
				List<ModelGraphEdge> adjEdges = distEdges[index];
				foreach (ModelGraphEdge adjme in adjEdges)
				{
					if (!me.adjEdges.Contains(adjme))
					{
						me.adjEdges.Add(adjme);
						adjme.adjEdges.Add(me);
						me.trueAdjEdges.Add(false);
						adjme.trueAdjEdges.Add(false);
					}
				}
			}

			
		}//End-add-dummy-edges-by-distance

		public void AddIsoAdjEdges()
		{
			List<ModelGraphEdge> isoEdges = new List<ModelGraphEdge>(), outEdges = null;
			List<ModelGraphNode> isoNodes = new List<ModelGraphNode>(), outNodes = null;
			foreach (ModelGraphNode mn in this.graphNodes)
			{
				if (mn.edges.Count == 1)
				{
					isoEdges.Add(mn.edges[0]);
					isoNodes.Add(mn);
				}
			}
			for(int i = 0; i < isoEdges.Count; ++i)
			{
				ModelGraphNode mgn = isoNodes[i];
				ModelGraphEdge mge = isoEdges[i];
				ModelGraphNode adjmgn = (mgn.edges[0].conntectingNodes[0] == mgn ?
					mgn.edges[0].conntectingNodes[1] : mgn.edges[0].conntectingNodes[0]);
				while (true)
				{
					if (adjmgn.adjNodes.Count > 2 && mge.cycleIndices != null
						&& mge.cycleIndices.Count == 0)
						break;
					// trace the string
					mge = (adjmgn.edges[0] == mge ? adjmgn.edges[1] :
						adjmgn.edges[0]);
					mgn = adjmgn;
					adjmgn = (mge.conntectingNodes[0] == adjmgn ? mge.conntectingNodes[1] :
						mge.conntectingNodes[0]);
				}
				isoEdges.RemoveAt(i);
				isoNodes.RemoveAt(i);
				isoEdges.Insert(i, mge);
				isoNodes.Insert(i, mgn);
			}
			if (isoEdges.Count > 2)
			{
				OrderEdges(isoEdges, isoNodes, out outEdges, out outNodes);

				for (int i = 0; i < outEdges.Count; ++i)
				{
					ModelGraphEdge edge1 = outEdges[i],
						edge2 = outEdges[(i + 1) % outEdges.Count];

					if (!edge1.adjEdges.Contains(edge2))
					{
						edge1.adjEdges.Add(edge2);
						if (edge1.trueAdjEdges != null)
							edge1.trueAdjEdges.Add(false);
						edge2.adjEdges.Add(edge1);
						if (edge2.trueAdjEdges != null)
							edge2.trueAdjEdges.Add(false);
					}
				}
			}
			// share the same node
			//foreach (ModelGraphNode mn in this.graphNodes)
			//{
			//    for (int i = 0; i < mn.edges.Count-1; ++i)
			//    {
			//        ModelGraphEdge mei = mn.edges[i];
			//        for (int j = i + 1; j < mn.edges.Count; ++j)
			//        {
			//            ModelGraphEdge mej = mn.edges[j];
			//            if(!mei.adjEdges.Contains(mej))
			//            {
			//                mei.adjEdges.Add(mej);
			//                mei.trueAdjEdges.Add(true);
			//                mej.adjEdges.Add(mei);
			//                mej.trueAdjEdges.Add(true);
			//            }
			//        }
			//    }
			//}
		}//End-add-iso-edges

		//public void CalculateNodeLvelByDirections()
		//{
		//    // either user indicate the direction of the edges,
		//    // the default: up ladder / tracking ball
		//    //if (this.modelType != 9) return;
		//    double minHeight = double.MaxValue, maxHeight = double.MinValue;
		//    foreach (ModelGraphNode mn in this.graphNodes)
		//    {
		//        double height = mn.meshComponent.boundingBox.center.pos.y;
		//        if (height < minHeight)
		//            minHeight = height;
		//        if (height > maxHeight)
		//            maxHeight = height;
		//        mn.level = -1;
		//    }
		//    foreach (ModelGraphEdge me in this.graphEdges)
		//        me.level = -1;
		//    double thresh = (maxHeight - minHeight) * 0.05;
		//    List<ModelGraphNode> baseNodes = new List<ModelGraphNode>();
		//    foreach (ModelGraphNode mn in this.graphNodes)
		//    {
		//        double height = mn.meshComponent.boundingBox.center.pos.y;
		//        if (Math.Abs(height - minHeight) < thresh)
		//        {
		//            baseNodes.Add(mn);
		//            mn.level = 0;
		//        }
		//    }
		//    // start from base nodes
		//    while (true)
		//    {
		//        if (baseNodes.Count == 0) break;
		//        int nnodes = baseNodes.Count;
		//        for (int i = 0; i < nnodes; ++i)
		//        {
		//            ModelGraphNode mn = baseNodes[0];
		//            foreach (ModelGraphEdge me in mn.edges)
		//            {
		//                ModelGraphNode other = me.conntectingNodes[0] == mn ?
		//                    me.conntectingNodes[1] : me.conntectingNodes[0];
		//                if (other.level != -1) continue;
		//                double height = other.meshComponent.boundingBox.center.pos.y;

		//                if (mn.name.Contains("fork"))
		//                {
		//                    if (height - mn.meshComponent.boundingBox.center.pos.y > 2*thresh)
		//                        other.level = mn.level;
		//                    else other.level = mn.level - 2;
		//                }
		//                else if (other.name.Contains("fork"))
		//                {
		//                    if (height - mn.meshComponent.boundingBox.center.pos.y > 2*thresh)
		//                        other.level = mn.level+2;
		//                    else other.level = mn.level;
		//                }
		//                else if (Math.Abs(height - mn.meshComponent.boundingBox.center.pos.y) < thresh)
		//                {
		//                    other.level = mn.level;
							
		//                }
		//                else if (height - mn.meshComponent.boundingBox.center.pos.y > thresh)
		//                {
		//                    other.level = mn.level + 1;
		//                    if (me.conntectingNodes[0] == mn)
		//                    {
		//                        me.faceIndex[0] = 6;
		//                        me.faceIndex[1] = 5;
		//                    }
		//                    else
		//                    {
		//                        me.faceIndex[0] = 5;
		//                        me.faceIndex[1] = 6;
		//                    }
		//                }
		//                else
		//                {
		//                    other.level = mn.level - 1;
		//                    if (me.conntectingNodes[0] == mn)
		//                    {
		//                        me.faceIndex[0] = 5;
		//                        me.faceIndex[1] = 6;
		//                    }
		//                    else
		//                    {
		//                        me.faceIndex[0] = 6;
		//                        me.faceIndex[1] = 5;
		//                    }
		//                }
		//                baseNodes.Add(other);
		//            }
		//            baseNodes.RemoveAt(0);
		//        }
		//    }

		//    foreach (ModelGraphEdge e in this.graphEdges)
		//    {
		//        e.level = Math.Max(e.conntectingNodes[0].level, e.conntectingNodes[1].level);
		//    }
		//}//CalculateNodeLvelByDirections


		public void CalculateNodeLvelByDirections()
		{
			// either user indicate the direction of the edges,
			// the default: up ladder / tracking ball

			double minHeight = double.MaxValue, maxHeight = double.MinValue;
			foreach (ModelGraphNode mn in this.graphNodes)
			{
				double height = mn.meshComponent.boundingBox.center.pos.y;
				if (height < minHeight)
					minHeight = height;
				if (height > maxHeight)
					maxHeight = height;
				mn.level = -1;
			}
			foreach (ModelGraphEdge me in this.graphEdges)
				me.level = -1;
			double thresh = (maxHeight - minHeight) * 0.05;
			List<List<ModelGraphNode>> baseNodes = new List<List<ModelGraphNode>>();
			foreach (ModelGraphNode mn in this.graphNodes)
			{
				double height = mn.meshComponent.boundingBox.center.pos.y;
				if (Math.Abs(height - minHeight) < thresh)
				{
					List<ModelGraphNode> nodes = new List<ModelGraphNode>();
					nodes.Add(mn);
					baseNodes.Add(nodes);
					mn.level = 0;
				}
			}
			// start from base nodes
			for (int ib = 0; ib < baseNodes.Count; ++ib)
			{
				List<ModelGraphNode> queue = new List<ModelGraphNode>();
				List<int> parents = new List<int>();
				ModelGraphNode node = baseNodes[ib][0];
				queue.Add(node);
				parents.Add(-1);
				ModelGraphNode cur = node, prev = null;
				int cursor = 0;
				while (cursor < queue.Count)
				{
					cur = queue[cursor];
					if(parents[cursor] != -1)
						prev = queue[parents[cursor]];
					foreach (ModelGraphEdge me in cur.edges)
					{
						ModelGraphNode other = (me.conntectingNodes[0] == cur ?
							me.conntectingNodes[1] : me.conntectingNodes[0]);
						if (other.level != -1) continue;
						Vector3d dir = other.meshComponent.boundingBox.center.pos -
							cur.meshComponent.boundingBox.center.pos;
						dir = dir.Normalize();
						if (prev != null)
						{
							Vector3d prevDir = cur.meshComponent.boundingBox.center.pos
								- prev.meshComponent.boundingBox.center.pos;
							prevDir = prevDir.Normalize();
							double acosv = dir.Dot(prevDir);
							if (Math.Abs(acosv) < 0.5) continue;
						}
						if (other.name.Contains("bridge") ||
							other.name.Contains("connector") ||
							other.name.Contains("ladder")||
							other.name.Contains("slider")) 
							continue;
						other.level = cur.level + 1;
						queue.Add(other);
						parents.Add(cursor);
					}
					++cursor;
				}
			}//end-for
			bool allLabeled = true;
			while (true)
			{
				allLabeled = true;
				foreach (ModelGraphNode mn in this.graphNodes)
				{
					this.maxLevel = (this.maxLevel > mn.level ? this.maxLevel : mn.level);
					if (mn.level != -1) continue;
					int maxLevel = -1;
					foreach (ModelGraphEdge me in mn.edges)
					{
						ModelGraphNode other = (me.conntectingNodes[0] == mn ?
							me.conntectingNodes[1] : me.conntectingNodes[0]);
						maxLevel = (maxLevel > other.level ? maxLevel : other.level);
					}
						mn.level = maxLevel;
					this.maxLevel = (this.maxLevel > mn.level ? this.maxLevel : mn.level);
					if (maxLevel == -1) allLabeled = false;
				}
				if (allLabeled) break;
			}
			
			foreach (ModelGraphEdge e in this.graphEdges)
			{
				e.level = Math.Max(e.conntectingNodes[0].level, e.conntectingNodes[1].level);
			}
		}//CalculateNodeLvelByDirections

		public void OrderEdges(List<ModelGraphEdge> edges, List<ModelGraphNode> nodes,
			out List<ModelGraphEdge> orderedEdges, out List<ModelGraphNode> orderedNodes)
		{
			int nneigs = edges.Count;
			double[,] mat = new double[nneigs, 3];
			Vector3d center = new Vector3d();
			for (int i = 0; i < nodes.Count; ++i)
			{
				for (int j = 0; j < 3; ++j)
				{
					mat[i, j] = nodes[i].pos3D[j];
					center[j] += mat[i, j];
				}
			}
			center /= nneigs;
			PrincipalComponentAnalysis pca = new PrincipalComponentAnalysis(mat, AnalysisMethod.Center);
			pca.Compute();
			//if (pca.Components.Count < 3)
			//{
			//    Program.OutputText(" < 3 principle axies found!", true);
			//    throw new Exception();
			//}
			// usually the plane will be XZ 
			Vector3d[] axes = new Vector3d[2];
			axes[0] = new Vector3d(pca.Components[0].Eigenvector, 0).Normalize();
			axes[1] = new Vector3d(pca.Components[1].Eigenvector, 0).Normalize();
			if (this.modelType == 8)
			{
				axes[0] = new Vector3d(1, 0, 0);
				axes[1] = new Vector3d(0, 0, 1);
			}
			// 2. order edges in counter-clockwise
			Vector2d[] locPos = new Vector2d[nneigs];
			Vector2d center2D = new Vector2d();
			for (int i = 0; i < nneigs; ++i)
			{
				double x = (nodes[i].pos3D).Dot(axes[0]);
				double y = (nodes[i].pos3D).Dot(axes[1]);
				locPos[i] = new Vector2d(x, y);
				center2D += locPos[i];
			}
			center2D /= nneigs;
			for (int i = 0; i < nneigs; ++i)
				locPos[i] -= center2D;
			bool[] visited = new bool[nneigs];
			visited[0] = true;
			List<int> edgeOrder = new List<int>();
			edgeOrder.Add(0);
			// order
			while (true)
			{
				Vector2d currvec = locPos[edgeOrder[edgeOrder.Count - 1]];
				Vector3d currvec3D = new Vector3d(currvec);
				double angle = 0, minAngle = double.MaxValue, maxAngle = double.MinValue;
				int nextEid = -1, nearestEid = -1;
				double thresh = 1e-2;
				for (int i = 0; i < nneigs; ++i)
				{
					if (visited[i]) continue;
					Vector3d neigvec3D = new Vector3d(locPos[i]);
					double acosvalue = locPos[i].Dot(currvec) / (locPos[i].Length() * currvec.Length());
					if (currvec.Length() < thresh || locPos[i].Length() < thresh)
						angle = 0;
					else if (Math.Abs(Math.Abs(acosvalue) - 1) < thresh)
					{
						if (acosvalue > 0)
							angle = 0;
						else
							angle = Math.PI;
					}
					else
						angle = Math.Acos(acosvalue);
					Vector3d dirz = neigvec3D.Cross(currvec3D);
					if (dirz.z > 0 && angle < minAngle)
					{
						minAngle = angle;
						nextEid = i;
					}
					else if (dirz.z <= 0 && angle > maxAngle)
					{
						maxAngle = angle;
						nearestEid = i;
					}
				}//end-for
				if (nextEid != -1)
				{
					edgeOrder.Add(nextEid);
					visited[nextEid] = true;
				}
				else if (nearestEid != -1)
				{
					edgeOrder.Add(nearestEid);
					visited[nearestEid] = true;
				}
				if (edgeOrder.Count == nneigs)
					break;
			}//end-while
			//re-order edges&adjnodes
			orderedEdges = new List<ModelGraphEdge>();
			orderedNodes = new List<ModelGraphNode>();
			for (int i = 0; i < nneigs; ++i)
			{
				orderedEdges.Add(edges[edgeOrder[i]]);
				orderedNodes.Add(nodes[edgeOrder[i]]);
			}
		}//End-order-edge


		//////////////////// Add: 2014-09-03 ////////////////////
		public void SearchMatchingsubgraphs(int maxSearchNum)
		{
			LabelSingleEdge();
			//if(this.meshfile.Contains("playground"))
			//    this.CompleteAdjEdgesByCycles(true);
			if (this.meshfile.Contains("race")
				|| this.meshfile.Contains("pentagon")
				)
				SearchMatchingsubgraphs1D(maxSearchNum);
			else
				SearchMatchingsubgraphs3D(maxSearchNum);
		}

        public void SearchMatchingsubgraphs1D(int maxSearchNum)
        {
            CollectEdgeMap();
            
            //ReDefEdgeDir();
            EnumerateMatchingEdgePair1D(maxSearchNum);
			//this.Filtersoultions();
        }

		public void SearchMatchingsubgraphs3D(int maxSearchNum)
		{
			//this.CalculateNodeLvelByDirections();
            this.FindGroundTouchingNodes();
			//if (this.meshfile.Contains("playground"))
			//    this.SearchReplaceableSubgraphsWithHeight(false);
			//else
				this.SearchReplaceableSubgraphsWithHeight(true);
			this.Filtersoultions();
            RankSolutions();
		}

		private void CollectEdgeMap()
		{
			// use hashmap to store the edge types by the two node types
			// hash function: max * minPrime + min, minPrime is the minimum prime that is larger that max
			if (this.graphNodes == null || this.graphNodes.Count == 0) return;
			int maxNodeTypeNumber = 0;
			int nNodes = this.graphNodes.Count;
			
			foreach (ModelGraphNode mn in this.graphNodes)
			{
				maxNodeTypeNumber = maxNodeTypeNumber > mn.nodeType ? maxNodeTypeNumber : mn.nodeType;
			}
			this.minPrime = minPrimeThanNumber(maxNodeTypeNumber+1);
			this.edgeClass = new List<ModelGraphEdge>[this.minPrime * this.minPrime];
            this.edgeMap = new Dictionary<ModelGraphEdge, int>();
			this.edgeMapRealIndices = new List<int>();
			foreach (ModelGraphEdge me in this.graphEdges)
			{
				int n1 = me.conntectingNodes[0].nodeType,
					n2 = me.conntectingNodes[1].nodeType;
				this.edgeMap.Add( me, Math.Max(n1, n2) * this.minPrime + Math.Min(n1, n2) );
				int mapIdx = this.edgeMap[me];
				if (mapIdx > this.edgeClass.Length)
				{
					Program.OutputText("Wrong map index!", true);
					break;
				}
				if (this.edgeClass[mapIdx] == null)
					this.edgeClass[mapIdx] = new List<ModelGraphEdge>();
				
				this.edgeClass[mapIdx].Add(me);
                int idx = 0;
                bool exist = false;
				for (; idx < this.edgeMapRealIndices.Count; ++idx)
				{
                    if (mapIdx < this.edgeMapRealIndices[idx])
                    {
                        this.edgeMapRealIndices.Insert(idx, mapIdx);
                        break;
                    }
                    else if (mapIdx == this.edgeMapRealIndices[idx])
                    {
                        exist = true;
                        break;
                    }
				}
                if (exist) continue;
                if (idx >= this.edgeMapRealIndices.Count)
                    this.edgeMapRealIndices.Add(mapIdx);
			}			

		}//CollectEdgeMap

		private int minPrimeThanNumber(int number)
		{
			int prm = number+1;
			while (true)
			{
				int e1 = 2;
				while (e1 < prm)
				{
					int e2 = prm / e1;
					if (prm == e1 * e2)
						break;
					++e1;
				}
				if (e1 >= prm) break;
				++prm;
			}
			return prm;
		}//minPrimeThanNumber

        private void LabelSingleEdge()
        {
            foreach (ModelGraphNode mn in this.graphNodes)
            {
                if (mn.edges.Count == 1)
                {
                    mn.edges[0].singleEdge = true;
                }
            }
        }//LabelSingleEdge

        public void ReDefEdgeDir()
        {
            foreach (ModelGraphEdge me in this.graphEdges)
            {
                me.faceIndex[0] = MapEdgeDir(me.faceIndex[0]);
                me.faceIndex[1] = MapEdgeDir(me.faceIndex[1]);
            }
        }//ReDefEdgeDir

        private int MapEdgeDir(int direction)
        {
            int index = -1;
            switch (direction)
            {
                case 1:
                    index = 1;
                    break;
                case -1:
                    index = 3;
                    break;
                case 2:
                    index = 6;
                    break;
                case -2:
                    index = 5;
                    break;
                case 3:
                    index = 4;
                    break;
                case -3:
                    index = 2;
                    break;
            }
            return index;
        }//MapEdgeDir

        private void EnumerateMatchingEdgePair1D(int maxSearchNum)
		{
			// enumerate all matching edge pairs and corresponding subgraphs
			// c(n,2) n: #edges
			int nclass = this.edgeMapRealIndices.Count;
			this.equSubgraphs = new List<Subgraph>();
			this.equCutpairs = new List<List<ModelGraphEdge>>();
			for (int c1 = 0; c1 < nclass; ++c1)
			{
				int classIdx1 = this.edgeMapRealIndices[c1];
				ModelGraphEdge cut_1_1, cut_1_2, cut_2_1, cut_2_2;
				for (int c_1_e_1 = 0; c_1_e_1 < this.edgeClass[classIdx1].Count-1; ++c_1_e_1)
				{
					// select 2 matching edges from class 1
					cut_1_1 = this.edgeClass[classIdx1][c_1_e_1];
                    if (cut_1_1.singleEdge) 
                        continue; // don not consider single edges (degenerate)
					for (int c_2_e_1 = c_1_e_1 + 1; c_2_e_1 < this.edgeClass[classIdx1].Count; ++c_2_e_1)
					{
						cut_2_1 = this.edgeClass[classIdx1][c_2_e_1];
                        if (cut_2_1.singleEdge) continue;
						// select 2 matching edges from class 2 (all edges can come from the same class)
						for (int c2 = c1; c2 < nclass; ++c2)
						{
							int classIdx2 = this.edgeMapRealIndices[c2];
                            int startIdx = 0;
                            if (c1 == c2) startIdx = c_2_e_1 + 1;

                            for (int c_1_e_2 = startIdx; c_1_e_2 < this.edgeClass[classIdx2].Count - 1; ++c_1_e_2)
							{
								if (c1 == c2 && (c_1_e_2 == c_1_e_1 || c_1_e_2 == c_2_e_1)) continue;
								cut_1_2 = this.edgeClass[classIdx2][c_1_e_2];
                                if (cut_1_2.singleEdge) continue;
                                List<ModelGraphEdge> cut1 = new List<ModelGraphEdge>();
                                cut1.Add(cut_1_1); cut1.Add(cut_1_2);
                                for (int c_2_e_2 = c_1_e_2+1; c_2_e_2 < this.edgeClass[classIdx2].Count; ++c_2_e_2)
								{
                                    if (c1 == c2 && (c_2_e_2 == c_1_e_1 || c_2_e_2 == c_2_e_1 ||
                                        c_2_e_2 == c_1_e_2)) continue;
									cut_2_2 = this.edgeClass[classIdx2][c_2_e_2];
                                    if (cut_2_2.singleEdge) continue;
                                    List<ModelGraphEdge> cut2 = new List<ModelGraphEdge>();
									cut2.Add(cut_2_1); cut2.Add(cut_2_2);
									List<Subgraph> sg1, sg2;
									int ncut1 = GetCutSets(cut1, out sg1);
									int ncut2 = GetCutSets(cut2, out sg2);
									if (ncut1 == ncut2 && ncut1 == 2)
									{
 										//found one
										AddMatchSubgraphs(sg1, sg2);
										if (maxSearchNum != -1 && this.equCutpairs.Count / 2 > maxSearchNum)
											return;
									}
								}//end-cut2-e2
							}//end-cut1-e2
						}//end-class-2
					}//end-cut2-e1
				}//end-cut-1-e1
			}//end-class-1
		}//EnumerateMatchingEdgePair1D	

		public int GetCutSets(List<ModelGraphEdge> cut, out List<Subgraph> subgraphs)
		{
			// a cut split the graph into several ( at least 2 ) independent sets			
			int nEdges = this.graphEdges.Count;
			int nNodes = this.graphNodes.Count;
			//sg = null;
			List<ModelGraphNode> edgeNodes = new List<ModelGraphNode>();
			// label inactive cut
			bool[] isActEdges = new bool[nEdges];
			for (int i = 0; i < nEdges; ++i)
				isActEdges[i] = true;
			for (int i = 0; i < cut.Count; i++)
			{
				if (!edgeNodes.Contains(cut[i].conntectingNodes[0]))
					edgeNodes.Add(cut[i].conntectingNodes[0]);
				if (!edgeNodes.Contains(cut[i].conntectingNodes[1]))
					edgeNodes.Add(cut[i].conntectingNodes[1]);
			}
			int nEdgeNodes = edgeNodes.Count;
			// DFS to find out inaccessible nodes
			bool[,] isAccNodes = new bool[nEdgeNodes, nEdgeNodes];
			List<bool> accNodes = new List<bool>();
			
			// start from each node (according to the cut), 
			// find all nodes that can be accessed from it by removing all cut in the input
			List<List<ModelGraphNode>> independendNodeSets = new List<List<ModelGraphNode>>();
			bool[] visited = new bool[nEdgeNodes];
			for (int n = 0; n < nEdgeNodes; n++)
			{
				if (visited[n]) continue;
				List<ModelGraphNode> accessibleNodes = GetAllAccessibleNodes(cut, edgeNodes[n]);
				independendNodeSets.Add(accessibleNodes);
				foreach (ModelGraphNode mn in accessibleNodes)
				{
					int idx = edgeNodes.IndexOf(mn);
					if (idx != -1)
						visited[idx] = true;
				}
				if (accessibleNodes.Count == nNodes) break;
			}//end-while

			subgraphs = new List<Subgraph>();
			int ncut = independendNodeSets.Count;

			if (ncut < 2) return ncut;
			
			for (int i = 0; i < ncut; ++i)
			{
				List<ModelGraphEdge> outEdges = GetOutgoingEdges(independendNodeSets[i]);
				if (cut.Count != outEdges.Count) continue;
				IEnumerable<ModelGraphEdge> diff = cut.Except(outEdges);
				if (diff.Count() > 0) continue;
				// correct split
				List<ModelGraphEdge> inEdges = GetIngoingEdges(independendNodeSets[i]);
				// make sure the order of outgoing edges is the same
				Subgraph subgraph = new Subgraph(this.graphIndex,independendNodeSets[i], cut, inEdges);				
				subgraphs.Add(subgraph);
			}
			//ncut = subgraphs.Count;
			//sg = subgraphs[0].nodeSets.Count < subgraphs[1].nodeSets.Count ? subgraphs[0] : subgraphs[1];
			return ncut;
		}//GetCutSets

		public List<ModelGraphNode> GetAllAccessibleNodes(List<ModelGraphEdge> cut, ModelGraphNode node)
		{
			List<ModelGraphNode> accessibleNodes = new List<ModelGraphNode>();
			accessibleNodes.Add(node);
			bool[] added = new bool[this.graphNodes.Count];
			added[node.nodeIdx] = true;
			int cur = 0;
			while (cur < accessibleNodes.Count)
			{
				ModelGraphNode mn = accessibleNodes[cur];
				foreach (ModelGraphEdge e in mn.edges)
				{
					if (cut.Contains(e)) continue;
					ModelGraphNode otherNode = e.conntectingNodes[0] == mn?
						e.conntectingNodes[1]:e.conntectingNodes[0];
					if(!added[otherNode.nodeIdx])
					{
						accessibleNodes.Add(otherNode);
						added[otherNode.nodeIdx] = true;
					}
				}
				++cur;
			}
			return accessibleNodes;
		}//GetAllAccessibleNodes

		public void SetNodeIndex()
		{
			int nidx = 0;
			foreach (ModelGraphNode mgn in this.graphNodes)
				mgn.nodeIdx = nidx++;
			int eidx = 0;
			foreach (ModelGraphEdge mge in this.graphEdges)
				mge.edgeIdx = eidx++;
		}//end-set-node-index

		private void FindGroundTouchingNodes()
		{
			if (this.graphNodes == null) return;

			// ground contacts
			double miny = double.MaxValue; Polygon plg = null;
			foreach (ModelGraphNode mn in this.graphNodes)
			{
				foreach (Polygon p in mn.meshComponent.boundingBox.polygons)
				{
					double yy = p.center.pos.y;
					if (yy < miny)
					{
						miny = yy;
						plg = p;
					}
				}
			}

			double thres2 = 1.0 / 64;
			foreach (ModelGraphNode mn in this.graphNodes)
			{
				foreach (Polygon p in mn.meshComponent.boundingBox.polygons)
				{
					double yy = p.center.pos.y;
					if (Math.Abs(yy - miny) < thres2)
					{
						mn.isgroundcontact = true;
						break;
					}
				}
			}
		}//FindGroundTouching

		List<ModelGraphEdge> a_route, b_route;
		List<List<ModelGraphEdge>> aEquCutPairs = null, bEquCutPairs = null;
        bool[] a_labeled = null, b_labeled = null;
		public void SearchReplaceableSubgraphsWithHeight(bool useHeight)
		{
			// initialization
			aEquCutPairs = new List<List<ModelGraphEdge>>();
			bEquCutPairs = new List<List<ModelGraphEdge>>();
			this.equCutpairs = new List<List<ModelGraphEdge>>();
			this.equSubgraphs = new List<Subgraph>();
			this.AssignIndex();

			List<bool> singleEdges = Enumerable.Repeat<bool>(true, this.graphEdges.Count).ToList();			

			foreach (ModelGraphEdge e in this.graphEdges)
				e.adjEdges = new List<ModelGraphEdge>();
			// In 3D graph, adjacent edges 
			foreach (ModelGraphNode mn in this.graphNodes)
			{
				for (int i = 0; i < mn.edges.Count - 1; ++i)
				{
					for (int j = i + 1; j < mn.edges.Count; ++j)
					{
 						if(!mn.edges[i].adjEdges.Contains(mn.edges[j]))
							mn.edges[i].adjEdges.Add(mn.edges[j]);
						
 						if(!mn.edges[j].adjEdges.Contains(mn.edges[i]))
							mn.edges[j].adjEdges.Add(mn.edges[i]);
					}
				}
			}
			if (this.faceCycles != null)
			{
				foreach (Face face in this.faceCycles)
				{					
					for (int i = 0; i < face.edges.Count - 1; ++i)
					{
						for (int j = i + 1; j < face.edges.Count; ++j)
						{
                            if (!face.edges[i].adjEdges.Contains(face.edges[j]))
                                face.edges[i].adjEdges.Add(face.edges[j]);
                            if (!face.edges[j].adjEdges.Contains(face.edges[i]))
                                face.edges[j].adjEdges.Add(face.edges[i]);
						}
					}
                    bool grounCycle = false;
                    foreach(ModelGraphNode v in face.vertices)
                        if (v.isgroundcontact)
                        {
                            grounCycle = true;
                            break;
                        }
                    if (grounCycle) continue;
					foreach (ModelGraphEdge e in face.edges)
					{
						singleEdges[e.edgeIdx] = false;
					}//label single edge cut
				}
			}//label adj edges
            a_labeled = new bool[this.graphEdges.Count];
            b_labeled = new bool[this.graphEdges.Count];
			// search
			for (int ai = 0; ai < this.graphEdges.Count-1; ++ai)
			{
				ModelGraphEdge a_e = this.graphEdges[ai];
				if (a_e.singleEdge) continue;
				List<ModelGraphEdge> aes = new List<ModelGraphEdge>();
				aes.Add(a_e);
				for (int bi = ai+1; bi < this.graphEdges.Count; ++bi)
				{
                    if (bi == ai) continue;
					// check whether two edges can connect					
					ModelGraphEdge b_e = this.graphEdges[bi];
					if (b_e.singleEdge) continue;
					List<ModelGraphEdge> bes = new List<ModelGraphEdge>();
					bes.Add(b_e);
					if (IsEqualEdgeWithHeight(a_e, b_e, null, null, useHeight))
					{
						a_route = new List<ModelGraphEdge>();
						b_route = new List<ModelGraphEdge>();
						a_route.Add(a_e);
						b_route.Add(b_e);
                        for (int i = 0; i < this.graphEdges.Count; ++i)
                        {
                            a_labeled[i] = false;
                            b_labeled[i] = false;
                        }
                        a_labeled[a_e.edgeIdx] = true;
                        b_labeled[b_e.edgeIdx] = true;
						this.DFSMatchedCutWithLevel(a_e, b_e, useHeight);
						//if(singleEdges[a_e.edgeIdx] && singleEdges[b_e.edgeIdx])
						//{
						//    aEquCutPairs.Add(aes);
						//    bEquCutPairs.Add(bes);
						//}
					}
				}
			}//search

            for (int i = 0; i < aEquCutPairs.Count - 1; ++i)
            {
                for (int j = i + 1; j < aEquCutPairs.Count; ++j)
                {
                    bool aequal = this.IsEqualCut(aEquCutPairs[i], aEquCutPairs[j]);
                    bool bequal = this.IsEqualCut(bEquCutPairs[i], bEquCutPairs[j]);
                    if (aequal && bequal)
                    {
                        aEquCutPairs.RemoveAt(j);
                        bEquCutPairs.RemoveAt(j);
                        --j;
                        continue;
                    }
                }
            }
			for (int cidx = 0; cidx < aEquCutPairs.Count; ++cidx)
			{
				List<Subgraph> a_outsgs = null, b_outsgs = null;				
				int acuts = GetCutSets(aEquCutPairs[cidx], out a_outsgs);
				int bcuts = GetCutSets(bEquCutPairs[cidx], out b_outsgs);
                if (acuts < 2 || bcuts < 2) continue;
				AddMatchSubgraphs(a_outsgs, b_outsgs);
			}// collect

            
		}//end-search-replaceable-with-level

        private void RankSolutions()
        {
            //order solutions
            int npairs = this.equSubgraphs.Count / 2;
            int[] rank = new int[npairs];
			int nlimit = this.graphNodes.Count/4;
            for (int i = 0; i < npairs; ++i)
            {
                rank[i] = 0;
                if (this.equCutpairs[2 * i].Count > 1)
                    rank[i] += 1;
				int ndiff = Math.Abs(this.equSubgraphs[2 * i].nodeSets.Count -
					this.equSubgraphs[2 * i + 1].nodeSets.Count);
				if(ndiff < nlimit)
					rank[i] += ndiff / 5;
            }
            int[] rank_index = new int[npairs];
            for (int i = 0; i < npairs; ++i)
                rank_index[i] = i;
            for (int i = 0; i < npairs - 1; ++i)
            {
                for (int j = i + 1; j < npairs; ++j)
                {
                    if (rank[j] > rank[i])
                    {
                        int tmp = rank[j];
                        rank[j] = rank[i];
                        rank[i] = tmp;
                        tmp = rank_index[j];
                        rank_index[j] = rank_index[i];
                        rank_index[i] = tmp;
                    }
                }
            }
            List<Subgraph> sgs = new List<Subgraph>();
            for (int i = 0; i < npairs; ++i)
            {
                sgs.Add(this.equSubgraphs[rank_index[i] * 2]);
                sgs.Add(this.equSubgraphs[rank_index[i] * 2 + 1]);
            }
            this.equSubgraphs = new List<Subgraph>(sgs);
            this.equCutpairs = new List<List<ModelGraphEdge>>();
            foreach (Subgraph sg in this.equSubgraphs)
                this.equCutpairs.Add(sg.outgoingEdges);
        }//RankSolutions

		private void AddMatchSubgraphs(List<Subgraph> a_outsgs, List<Subgraph> b_outsgs)
		{
			for (int i = 0; i < a_outsgs.Count; ++i)
			{
				for (int j = 0; j < b_outsgs.Count; ++j)
				{
					int k = 0;
                    
					for (; k < a_outsgs[i].edgeNodes.Count; ++k)
					{
						if (a_outsgs[i].edgeNodes[k].nodeType != b_outsgs[j].edgeNodes[k].nodeType)
							break;
                        int a_fidx = a_outsgs[i].outgoingEdges[k].conntectingNodes[0] == a_outsgs[i].edgeNodes[k] ?
                            a_outsgs[i].outgoingEdges[k].faceIndex[0] : a_outsgs[i].outgoingEdges[k].faceIndex[1];
                        int b_fidx = b_outsgs[j].outgoingEdges[k].conntectingNodes[0] == b_outsgs[j].edgeNodes[k] ?
                            b_outsgs[j].outgoingEdges[k].faceIndex[0] : b_outsgs[j].outgoingEdges[k].faceIndex[1];
                        if (a_fidx != b_fidx) break;
					}
                    if (a_outsgs[i].nodeSets.Count == b_outsgs[j].nodeSets.Count
                        && a_outsgs[i].nodeSets.Count == 1)
                        continue;
					if (k >= a_outsgs[i].edgeNodes.Count) // found one
					{
						if (a_outsgs[i].nodeSets.Count > b_outsgs[j].nodeSets.Count)
						{
							this.equSubgraphs.Add(b_outsgs[j]);
							this.equSubgraphs.Add(a_outsgs[i]);
							this.equCutpairs.Add(b_outsgs[j].outgoingEdges);
							this.equCutpairs.Add(a_outsgs[i].outgoingEdges);
						}
						else
						{
							this.equSubgraphs.Add(a_outsgs[i]);
							this.equSubgraphs.Add(b_outsgs[j]);
							this.equCutpairs.Add(a_outsgs[i].outgoingEdges);
							this.equCutpairs.Add(b_outsgs[j].outgoingEdges);
						}
						FlipBoundary(this.equSubgraphs.Count - 2);
					}
				}
			}
		}//AddMatchSubgraphs

		private void FlipBoundary(int i)
		{
			if (this.equSubgraphs[i].outgoingEdges.Count != 2) return;
			if (!IsEqualEdgeType(this.equSubgraphs[i].outgoingEdges[0],
				this.equSubgraphs[i + 1].outgoingEdges[1])) return;
			Vector3d sg1_e1 = this.equSubgraphs[i].connNodes[0].meshComponent.boundingBox.center.pos
				- this.equSubgraphs[i].edgeNodes[0].meshComponent.boundingBox.center.pos,
				sg1_e2 = this.equSubgraphs[i].connNodes[1].meshComponent.boundingBox.center.pos
				- this.equSubgraphs[i].edgeNodes[1].meshComponent.boundingBox.center.pos,
				sg2_e1 = this.equSubgraphs[i + 1].connNodes[0].meshComponent.boundingBox.center.pos
				- this.equSubgraphs[i + 1].edgeNodes[0].meshComponent.boundingBox.center.pos,
				sg2_e2 = this.equSubgraphs[i + 1].connNodes[1].meshComponent.boundingBox.center.pos
				- this.equSubgraphs[i + 1].edgeNodes[1].meshComponent.boundingBox.center.pos;
			double cosv1 = sg1_e1.Dot(sg2_e1),
				cosv2 = sg1_e2.Dot(sg2_e2);
			
			Vector3d n1 = sg1_e1.Cross(sg2_e1),
				n2 = sg1_e2.Cross(sg2_e2);
			double nv = n1.Dot(n2);
			if (cosv1 * cosv2 > 0 || nv > 0) return; 
			// flip
			ModelGraphEdge me = this.equSubgraphs[i].outgoingEdges[0];
			this.equSubgraphs[i].outgoingEdges.RemoveAt(0);
			this.equSubgraphs[i].outgoingEdges.Add(me);
			ModelGraphNode mn = this.equSubgraphs[i].connNodes[0];
			this.equSubgraphs[i].connNodes.RemoveAt(0);
			this.equSubgraphs[i].connNodes.Add(mn);
			mn = this.equSubgraphs[i].edgeNodes[0];
			this.equSubgraphs[i].edgeNodes.RemoveAt(0);
			this.equSubgraphs[i].edgeNodes.Add(mn);
		}

		private bool IsEqualEdgeType(ModelGraphEdge e1, ModelGraphEdge e2)
		{
			return ((e1.conntectingNodes[0].nodeType == e2.conntectingNodes[0].nodeType
				&& e1.conntectingNodes[1].nodeType == e2.conntectingNodes[1].nodeType)
				|| (e1.conntectingNodes[0].nodeType == e2.conntectingNodes[1].nodeType
				&& e1.conntectingNodes[1].nodeType == e2.conntectingNodes[0].nodeType));
		}

		private void Filtersoultions()
		{
			//remove repetition
			bool hasRep = false;
			for (int i = 0; i < this.equCutpairs.Count - 1; i += 2)
			{

				for (int j = i + 2; j < this.equCutpairs.Count; j += 2)
				{
					List<ModelGraphEdge> i_acut = this.equCutpairs[i],
						i_bcut = this.equCutpairs[i + 1],
						j_acut = this.equCutpairs[j],
						j_bcut = this.equCutpairs[j + 1];
					if ((this.equSubgraphs[i].nodeSets.Count == this.equSubgraphs[j].nodeSets.Count &&
						this.equSubgraphs[i+1].nodeSets.Count == this.equSubgraphs[j+1].nodeSets.Count
						&& this.IsEqualCut(i_acut, j_acut) && this.IsEqualCut(i_bcut, j_bcut)) ||
						(this.equSubgraphs[i].nodeSets.Count == this.equSubgraphs[j+1].nodeSets.Count
						&&this.equSubgraphs[i+1].nodeSets.Count == this.equSubgraphs[j].nodeSets.Count
						&& this.IsEqualCut(i_acut, j_bcut) && this.IsEqualCut(i_bcut, j_acut)))
					{
						if (this.equCutpairs[j].Count == 1)
							hasRep = false;
						Removesolution(j + 1);
						Removesolution(j);
						hasRep = true;
						j -= 2;
					}
				}
			}
			if (hasRep)
				hasRep = false;
			int minLimit = 1;
			int nlimit = (int)((double)this.graphNodes.Count * 0.6);
			if (this.meshfile.Contains("playground")
				|| this.meshfile.Contains("pentagon")) 
				nlimit = this.graphNodes.Count;
			for (int i = 0; i < this.equSubgraphs.Count - 1; i += 2)
			{
				if(Math.Abs(this.equSubgraphs[i].nodeSets.Count-
					this.equSubgraphs[i + 1].nodeSets.Count) < minLimit ||
					this.equSubgraphs[i].nodeSets.Count > nlimit ||
					this.equSubgraphs[i + 1].nodeSets.Count > nlimit)
				{
					Removesolution(i + 1);
					Removesolution(i);
					i -= 2;
					continue;
				}
				if(ReverseSupport(this.equSubgraphs[i],this.equSubgraphs[i+1]) )
				{
					Removesolution(i + 1);
					Removesolution(i);
					i -= 2;
					continue;
				}
				List<int> edgeDist1 = CalEdgeDist(this.equSubgraphs[i]),
					edgeDist2 = CalEdgeDist(this.equSubgraphs[i + 1]);
				for (int j = 0; j < edgeDist1.Count; ++j)
				{
					if (edgeDist1[j] != edgeDist2[j])
					{
						Removesolution(i + 1);
						Removesolution(i);
						i -= 2;
						break;
					}
				}
			}
		
		}//End-of-filter

		private bool ReverseSupport(Subgraph sg1, Subgraph sg2)
		{
			Vector3d upDir = new Vector3d(0,1,0);
			double thresh = 0.3;
			for (int i = 0; i < sg1.outgoingEdges.Count; ++i)
			{
				Vector3d edir1 = sg1.connNodes[i].meshComponent.boundingBox.center.pos
					- sg1.edgeNodes[i].meshComponent.boundingBox.center.pos;
				Vector3d edir2 = sg2.connNodes[i].meshComponent.boundingBox.center.pos
					- sg2.edgeNodes[i].meshComponent.boundingBox.center.pos;
				if (Math.Abs(edir1.Dot(upDir)) < thresh && Math.Abs(edir2.Dot(upDir)) < thresh)
				{
					if (edir1.Dot(edir2) < 0)
						return true;
				}
			}
			return false;
		}//ReverseSupport

		private List<int> CalEdgeDist(Subgraph sg)
		{
			// edge dist (without cycle, but from node to node)
			
			List<int> edgeDist = new List<int>();
			if (sg.outgoingEdges.Count == 1)
			{
				edgeDist.Add(0);
				return edgeDist;
			}
			for (int i = 0; i < sg.outgoingEdges.Count; ++i)
			{
				ModelGraphEdge e1 = sg.outgoingEdges[i];
				ModelGraphEdge e2  = sg.outgoingEdges[(i+1)%sg.outgoingEdges.Count];
				int dist = 0, start = 0, end = 1;
				List<ModelGraphEdge> queue = new List<ModelGraphEdge>();
				queue.Add(e1);
				bool found = false;
				while (start < end)
				{
					++dist;
					found = false;
					for (int j = start; j < end; ++j)
					{
						ModelGraphEdge e = queue[j];
						for (int n = 0; n < e.conntectingNodes.Count; ++n)
						{
							if (e.conntectingNodes[n].edges.Contains(e2))
							{
								edgeDist.Add(dist);
								found = true;
								break;
							}
							foreach (ModelGraphEdge me in e.conntectingNodes[n].edges)
							{
								if (!queue.Contains(me))
									queue.Add(me);
							}
						}
						if (found) break;
					}					
					start = end;
					end = queue.Count;
					if (found) break;
				}
				if (!found) edgeDist.Add(dist);
			}
			return edgeDist;
		}//EdgeDist

		private bool IsEqualCut(List<ModelGraphEdge> aEdges, List<ModelGraphEdge> bEdges)
		{
			if (aEdges.Count != bEdges.Count) return false;
			IEnumerable<ModelGraphEdge> diffset = aEdges.Except(bEdges);
			if (diffset.Count() == 0) return true;
			return false;
		}//IsEquaCut

		private void Removesolution(int index)
		{
			this.equCutpairs.RemoveAt(index);
			this.equSubgraphs.RemoveAt(index);
		}//End-reomove-solution

		public void DFSMatchedCutWithLevel(ModelGraphEdge a_edge, ModelGraphEdge b_edge, bool useHeight)
		{
			List<ModelGraphEdge> a_neig = new List<ModelGraphEdge>(),
				b_neig = new List<ModelGraphEdge>();

			
			foreach (ModelGraphEdge e in a_edge.adjEdges)
            {
				if (!a_labeled[e.edgeIdx])
				{
					a_neig.Add(e);
					a_labeled[e.edgeIdx] = true;
					//if(a_edge.faceIndex.Intersect(e.faceIndex).Count() ==0)
					//    a_labeled[e.edgeIdx] = true;

				}
            }
			foreach (ModelGraphEdge e in b_edge.adjEdges)
            {
                if (!b_labeled[e.edgeIdx])
				{
					b_neig.Add(e);
					b_labeled[e.edgeIdx] = true;
					//if (b_edge.faceIndex.Intersect(e.faceIndex).Count() == 0)
					//b_labeled[e.edgeIdx] = true;
				}
            }

			if (a_neig.Count == 0 || b_neig.Count == 0)
			{
				if (a_route.Count < minmumCutedges || b_route.Count < minmumCutedges) return;
				List<ModelGraphEdge> aedges = new List<ModelGraphEdge>(a_route);
				List<ModelGraphEdge> bedges = new List<ModelGraphEdge>(b_route);
				aEquCutPairs.Add(aedges);
				bEquCutPairs.Add(bedges);
			}

			foreach (ModelGraphEdge ae in a_neig)
			{
				//if (ae.singleEdge) continue;
				a_route.Add(ae);
				a_labeled[ae.edgeIdx] = true;
				//IEnumerable<int> sameCycles = ae.faceIndex.Intersect(a_edge.faceIndex);
				//for (int i = 0; i < sameCycles.Count(); ++i)
				//{
				//    foreach (ModelGraphEdge me in this.faceCycles[sameCycles.ElementAt(i)].edges)
				//        a_labeled[me.edgeIdx] = true;
				//}
				foreach (ModelGraphEdge be in b_neig)
				{
                    if (be == ae )
						//|| be.edgeIdx < ae.edgeIdx) 
						continue;
					//if (be.singleEdge) continue;
					if (IsEqualEdgeWithHeight(ae, be, a_edge, b_edge, useHeight))
					{
						b_route.Add(be);
						b_labeled[be.edgeIdx] = true;

						//sameCycles = be.faceIndex.Intersect(b_edge.faceIndex);
						//for (int i = 0; i < sameCycles.Count(); ++i)
						//{
						//    foreach (ModelGraphEdge me in this.faceCycles[sameCycles.ElementAt(i)].edges)
						//        b_labeled[me.edgeIdx] = true;
						//}

						this.DFSMatchedCutWithLevel(ae, be, useHeight);
						b_labeled[be.edgeIdx] = false;
						b_route.Remove(be);
					}
				}
				a_labeled[ae.edgeIdx] = false;
				a_route.Remove(ae);
			}

			if (a_neig.Count >= minmumCutedges || b_neig.Count >= minmumCutedges)
			{
				List<ModelGraphEdge> aedges = new List<ModelGraphEdge>(a_route);
				List<ModelGraphEdge> bedges = new List<ModelGraphEdge>(b_route);
				aEquCutPairs.Add(aedges);
				bEquCutPairs.Add(bedges);
			}

		}//DFS-match-cut-with-level


		public int minmumCutedges = 1;
	
		public bool IsEqualEdgeWithHeight(ModelGraphEdge e1, ModelGraphEdge e2, 
			ModelGraphEdge e10, ModelGraphEdge e20, bool useHeight)
		{
			bool match = false;
			if (useHeight && (e10 != null && e20 != null &&
				(e1.level - e10.level != e2.level - e20.level))) return false;
			if ((e1.conntectingNodes[0].nodeType == e2.conntectingNodes[0].nodeType &&
				e1.conntectingNodes[1].nodeType == e2.conntectingNodes[1].nodeType ) ||
				(e1.conntectingNodes[0].nodeType == e2.conntectingNodes[1].nodeType &&
				e1.conntectingNodes[1].nodeType == e2.conntectingNodes[0].nodeType ))
				match = true;
			return match;
		}
		
		//////////////////// Add: 2014-09-03 ////////////////////
		
	}// End -- Model Graph

	public class Face
	{
		public List<ModelGraphNode> vertices = null;
		public List<ModelGraphEdge> edges = null;
		public Face()
		{
			vertices = new List<ModelGraphNode>();
			edges = new List<ModelGraphEdge>();
		}
		public void AddElement(ModelGraphNode node, ModelGraphEdge edge)
		{
			this.vertices.Add(node);
			this.edges.Add(edge);
		}
		public bool Included(Face f)
		{
			//if (f.vertices.Count > this.vertices.Count)
			//    return false;
			if (this.vertices.Count >= f.vertices.Count)
			{
				foreach (ModelGraphNode mn in f.vertices)
				{
					if (!this.vertices.Contains(mn))
						return false;
				}
			}
			else
			{
				foreach (ModelGraphNode mn in this.vertices)
				{
					if (!f.vertices.Contains(mn))
						return false;
				}
			}
			return true;
		}
	
	}//End-Face

	public class TreeNode
	{
		public int index;
		public int eindex; //#edge*2
		public TreeNode parent;
		public List<TreeNode> children = null;
		public TreeNode(int idx, int eidx, TreeNode prt)
		{
			index = idx;
			eindex = eidx;
			parent = prt;
			children = new List<TreeNode>();
		}
	}//End-TreeNode

	public class GraphGroup
	{
		public List<ModelGraph> kidModelGraphs = null;
		public int nKidGraphs = 0;
		// least shared structure --- components which are included in each model
		// graph Idx + node type
		public List<int> sharedComponents = new List<int>();
		// partial shared structure --- components that are fixed but some do not have
		public List<int> subSharedComponents = new List<int>();
		public int numPreGroup = 0;
		// #edge types = MaxNumberNodeType.MAXNODETYPE * (MaxNumberNodeType.MAXNODETYPE-1)/2
		// (0,1) (0,2)....(1,2)....(n-2,n-1) --- (p, q)
		// map to n * p - p*(p-1)/2 + q - p - 1
		// edge pairs
		public List<List<List<int>>> edgeArray2D = new List<List<List<int>>>();

		public List<int> preGroupTypes = new List<int>();

		public GraphGroup()
		{
			kidModelGraphs = new List<ModelGraph>();
		}

		public GraphGroup(List<ModelGraph> graphs)
		{
			kidModelGraphs = graphs;
			for (int idx = 0; idx < graphs.Count; idx++)
			{
				kidModelGraphs[idx].graphIndex = idx;
				foreach (ModelGraphNode mn in kidModelGraphs[idx].graphNodes)
					mn.graphIdx = idx;
			}
			this.nKidGraphs = graphs.Count;
			ExtractSharedComponents();
		}

		

		//////////////////// 2014-08-24 ////////////////////

		public void FindMatchingSubgraphs(ModelGraph a_mg, ModelGraph b_mg)
		{ 
			// given two graphs, find matching subgraphs with equal boundaries and varied interior
			List<Pair<Subgraph, Subgraph>> potentialMatchingROIs = FindPotentialMatchingROI(a_mg, b_mg);

			// region growing to find matching subgraphs
			// 1. compare boundary, if match, exit
			// 2. add a node (select using BFS)
			// 3. update subgraph, goto step 1.
			int num = potentialMatchingROIs.Count;
			a_mg.equSubgraphs = new List<Subgraph>();
			b_mg.equSubgraphs = new List<Subgraph>();
			a_mg.equCutpairs = new List<List<ModelGraphEdge>>();
			b_mg.equCutpairs = new List<List<ModelGraphEdge>>();
			int maxTry = 6;
			for (int idx = 0; idx < num; ++idx)
			{
				Subgraph a_sg = potentialMatchingROIs[idx].Item1,
					b_sg = potentialMatchingROIs[idx].Item2;				
				// expand the subgraph
				int ntry = 0;
				while (ntry < maxTry)
				{
					bool isMatched = a_sg.IsMatchedSubgraph(b_sg);
					if (isMatched)
					{
						a_mg.equSubgraphs.Add(a_sg);
						b_mg.equSubgraphs.Add(b_sg);
						a_mg.equCutpairs.Add(a_sg.outgoingEdges);
						b_mg.equCutpairs.Add(b_sg.outgoingEdges);
						break;
					}					
					if (ntry % 2 == 0)
						RegionGrwoingSubgraph(ref a_sg, b_sg);
					else
						RegionGrwoingSubgraph(ref b_sg, a_sg);
					++ntry;
				}
			}
		}//FindMatchingSubgraphs

		private void RegionGrwoingSubgraph(ref Subgraph sg, Subgraph ref_sg)
		{
			List<int> nodeTypes = GetEdgeNodeTypes(sg),
				ref_nodeTypes = GetEdgeNodeTypes(ref_sg);
			IEnumerable<int> redun_nodeTypes = ref_nodeTypes.Except(nodeTypes),
				lack_nodeTypes = nodeTypes.Except(ref_nodeTypes);
			
			IEnumerable<ModelGraphNode> distinctNodes = sg.edgeNodes.Distinct();
			IEnumerable<int> intersection = lack_nodeTypes.Intersect(redun_nodeTypes);
			bool changed = false;
			foreach (ModelGraphNode mn in distinctNodes)
			{
				foreach (ModelGraphNode adjmn in mn.adjNodes)
				{  
					if(intersection.Contains(adjmn.nodeType))
					{
						AddNodeToSubgraph(ref sg, adjmn);
						changed = true;
						break;
					}
				}
				if (changed) break;
			}
			if (changed) return;
			foreach (ModelGraphNode mn in distinctNodes)
			{
				foreach (ModelGraphNode adjmn in mn.adjNodes)
				{
					if (lack_nodeTypes.Contains(adjmn.nodeType))
					{
						AddNodeToSubgraph(ref sg, adjmn);
						changed = true;
						break;
					}
				}
				if (changed) break;
			}
			if (changed) return;
			foreach (ModelGraphNode mn in distinctNodes)
			{
				foreach (ModelGraphNode adjmn in mn.adjNodes)
				{
					if (ref_nodeTypes.Contains(adjmn.nodeType))
					{
						AddNodeToSubgraph(ref sg, adjmn);
						changed = true;
						break;
					}
				}
				if (changed) break;
			}
		}//RegionGrwoingSubgraph

		private void AddNodeToSubgraph(ref Subgraph sg, ModelGraphNode node)
		{ 
			//1. all its edges connecting nodes in sg become ingoing edges (including existing outgoing edges)
			//2. all other edges become outgoing edges
			//3. add the node
			int nedges = node.edges.Count;
			bool[] labeled = new bool[nedges];
			for (int i = 0; i < nedges; ++i) labeled[i] = false;
			for(int i = 0; i < nedges; ++i)
			{
				ModelGraphEdge me = node.edges[i];
				int index = sg.outgoingEdges.IndexOf(me);
				if (index != -1)
				{
					//sg.outgoingEdges.RemoveAt(index);
					labeled[i] = true;
					sg.ingoingEdges.Add(me);
				}
			}
			// keep ordering, insert outgoing edges
			// find the one following an labeled edge
			int eindex = 0;
			bool found = false;
			while(true)
			{
				ModelGraphEdge me = node.edges[eindex];
				if (labeled[eindex] || (!found && !labeled[eindex]))
				{
					if(labeled[eindex])
						found = true;
					eindex = (eindex + 1) % nedges;
					continue;
				}
				// find the first edge following labeled edges
				int idx = (eindex - 1 + nedges) % nedges;
				int idxInOutEdges = sg.outgoingEdges.IndexOf(node.edges[idx]);
				if (idxInOutEdges == -1)
				{
					Program.OutputText("Wrong order!", true);
					break;
				}
				// insert edges
				while (true)
				{
					me = node.edges[eindex];
					if (labeled[eindex]) break;
					sg.outgoingEdges.Insert(idxInOutEdges, me);
					eindex = (eindex + 1) % nedges;
				}
				break;
			}
			// remove
			for (int i = 0; i < nedges; ++i)
			{
				if(!labeled[i]) continue;
				ModelGraphEdge me = node.edges[i];
				sg.outgoingEdges.Remove(me);
			}
			sg.nodeSets.Add(node);
		}//AddNodeToSubgraph

		private List<int> GetEdgeNodeTypes(Subgraph sg)
		{
			List<int> types = new List<int>();
			foreach (ModelGraphNode mn in sg.edgeNodes)
				types.Add(mn.nodeType);
			return types;
		}//GetEdgeNodeTypes

		private List<Pair<Subgraph, Subgraph>> FindPotentialMatchingROI(ModelGraph a_mg, ModelGraph b_mg)
		{
			// calculate similarities between any two ROI, to find out similar subgraphs for further matching

			// 1. find unmatched node sets
			List<ModelGraphNode> a_unmatched_nodes, b_unmatched_nodes;
			IdentifyUnmatchedNodes(a_mg, b_mg, out a_unmatched_nodes, out b_unmatched_nodes);

			// 2. region growing to find ROIs
			List<Subgraph> a_ROIs, b_ROIs;
			a_ROIs = RegionGrowingROI(a_mg, a_unmatched_nodes);
			b_ROIs = RegionGrowingROI(b_mg, b_unmatched_nodes);
			//a_mg.equCutpairs = new List<List<ModelGraphEdge>>();
			//b_mg.equCutpairs = new List<List<ModelGraphEdge>>();
			//a_mg.equSubgraphs = new List<Subgraph>();
			//b_mg.equSubgraphs = new List<Subgraph>();
			//foreach (Subgraph sg in a_ROIs)
			//{
			//    a_mg.equSubgraphs.Add(sg);
			//    a_mg.equCutpairs.Add(sg.outgoingEdges);
			//}
			//foreach (Subgraph sg in b_ROIs)
			//{
			//    b_mg.equCutpairs.Add(sg.outgoingEdges);
			//    b_mg.equSubgraphs.Add(sg);
			//}
			//return null;
			// 3. compare ROIs to find potential matching areas
			// --a. functional nodes
			List<List<int>> a_conntypes = new List<List<int>>(),
				b_conntypes = new List<List<int>>();
			for (int a = 0; a < a_ROIs.Count; ++a)
			{
				List<int> types = new List<int>();
				foreach (ModelGraphNode mn in a_ROIs[a].nodeSets)
				{
					if (!types.Contains(mn.nodeType))
						types.Add(mn.nodeType);
				}
				a_conntypes.Add(types);
			}
			for (int b = 0; b < b_ROIs.Count; ++b)
			{
				List<int> types = new List<int>();
				foreach (ModelGraphNode mn in b_ROIs[b].nodeSets)
				{
					if (!types.Contains(mn.nodeType))
						types.Add(mn.nodeType);
				}
				b_conntypes.Add(types);
			}
			List<Pair<Subgraph, Subgraph>> potentialMatchingROIs = new List<Pair<Subgraph, Subgraph>>();
			for (int a = 0; a < a_ROIs.Count; ++a)
			{
				for (int b = 0; b < b_ROIs.Count; ++b)
				{
					if (a_conntypes[a].Except(b_conntypes[b]).Count() == 0 ||
						a_conntypes[a].Except(b_conntypes[b]).Count() == 0)
					{
						Pair<Subgraph, Subgraph> pair = new Pair<Subgraph, Subgraph>(
							new Subgraph(a_ROIs[a]), new Subgraph(b_ROIs[b]));
						potentialMatchingROIs.Add(pair);
					}
				}
			}
			return potentialMatchingROIs;
		}//FindPotentialMatchingROI

		private List<Subgraph> RegionGrowingROI(ModelGraph mg, List<ModelGraphNode> unmatched_nodes)
		{
			// start from labeled nodes (unmatched), do region growing to find region of interest
			List<Subgraph> ROIs = new List<Subgraph>();
			int nnodes = unmatched_nodes.Count;
			bool[] grouped = new bool[nnodes];
			for (int i = 0; i < nnodes; ++i)
				grouped[i] = false;
			for (int i = 0; i < nnodes; ++i)
			{
				if (grouped[i]) continue;
				List<ModelGraphNode> group = new List<ModelGraphNode>();
				group.Add(unmatched_nodes[i]);
				int cursor = 0;
				while (cursor < group.Count)
				{
					ModelGraphNode mn = group[cursor++];
					foreach (ModelGraphNode adjmn in mn.adjNodes)
					{
						// --include isolated nodes (only has one edge, to mn)
						int idx = unmatched_nodes.IndexOf(adjmn);
						if ( (idx != -1 && grouped[idx]) || (idx == -1 && adjmn.edges.Count > 1))
							continue;
						group.Add(adjmn);
						if(idx != -1)
							grouped[idx] = true;
					}
				}// region growing
				Subgraph sg = mg.GetSubgraph(group);
				sg.OrderEdgeNodes();
				ROIs.Add(sg);
			}
			return ROIs;
		}//RegionGrowingROI

		private void IdentifyUnmatchedNodes(ModelGraph a_mg, ModelGraph b_mg,
			out List<ModelGraphNode> a_unmatched_nodes,
			out List<ModelGraphNode> b_unmatched_nodes)
		{
			// given two graphs, label the nodes that cannot find exact matches in the other graph
			List<ModelGraphNode> bmatchedNodes = new List<ModelGraphNode>();
			a_unmatched_nodes = new List<ModelGraphNode>();
			b_unmatched_nodes = new List<ModelGraphNode>();
			
			foreach (ModelGraphNode mn1 in a_mg.graphNodes)
			{
				bool hasmatched = false;
				foreach (ModelGraphNode mn2 in b_mg.graphNodes)
				{
					if (MatchingNodes(mn1, mn2))
					{
						hasmatched = true;
						bmatchedNodes.Add(mn2);
						break;
					}
				}
				if (!hasmatched) a_unmatched_nodes.Add(mn1);
			}
			foreach (ModelGraphNode mn2 in b_mg.graphNodes)
			{
				if (bmatchedNodes.Contains(mn2)) continue;
				bool hasmatched = false;
				foreach (ModelGraphNode mn1 in a_mg.graphNodes)
				{
					if (MatchingNodes(mn2, mn1))
					{
						hasmatched = true;
						break;
					}
				}
				if (!hasmatched) b_unmatched_nodes.Add(mn2);
			}
		}//IdentifyUnmatchedNodes

		private bool MatchingNodes(ModelGraphNode mn1, ModelGraphNode mn2)
		{
			bool isMatched = true;
			if (mn1.nodeType != mn2.nodeType) return false;
			if (mn1.edges.Count != mn2.edges.Count) return false;
			int nedges = mn1.edges.Count;
			// 1. check support
			List<int> upper1 = new List<int>(), lower1 = new List<int>(),
				upper2 = new List<int>(), lower2 = new List<int>();
			FindSupportNodes(mn1, out upper1, out lower1);
			FindSupportNodes(mn2, out upper2, out lower2);
			if (upper1.Count != upper2.Count || lower1.Count != lower2.Count)
				return false;
			if (upper1.Except(upper2).Count() > 0 || upper2.Except(upper1).Count() > 0)
				return false;
			if (lower1.Except(lower2).Count() > 0 || lower2.Except(lower1).Count() > 0)
				return false;
			// 2. planar order of edges, counter clockwise: X+, Z-, X-, Z+		
			List<int> eorder1, eorder2, conntypes1, conntypes2;
			OrderEdgeByDir(mn1, out eorder1, out conntypes1);
			OrderEdgeByDir(mn2, out eorder2, out conntypes2);
			// distinct direction indices			
			IEnumerable<int> distinctDir1 = eorder1.Distinct(),
				distinctDir2 = eorder2.Distinct();
			if (distinctDir1.Count() != distinctDir2.Count()) return false;
			int ndistincts = distinctDir1.Count();
			// check direction + connecting node types
			switch (ndistincts)
			{
 				case 1:
					IEnumerable<int> diffs = conntypes1.Except(conntypes2);
					if (diffs.Count() > 0) return false;
					diffs = conntypes2.Except(conntypes1);
					if (diffs.Count() > 0) return false;
					break;
				case 2:
					// match order
					if (distinctDir1.ElementAt(1) - distinctDir1.ElementAt(0) !=
						distinctDir2.ElementAt(1) - distinctDir2.ElementAt(0))
						return false;
					// match node types
					List<int> types1_1 = new List<int>(), types1_2 = new List<int>(),
						types2_1 = new List<int>(), types2_2 = new List<int>();
					for (int i = 0; i < conntypes1.Count; ++i)
					{
						if (eorder1[i] == distinctDir1.First())
							types1_1.Add(conntypes1[i]);
						else
							types1_2.Add(conntypes1[i]);
					}
					for (int i = 0; i < conntypes2.Count; ++i)
					{
						if (eorder1[i] == distinctDir2.First())
							types2_1.Add(conntypes2[i]);
						else
							types2_2.Add(conntypes2[i]);
					}
					if ((IsEqualSets(types1_1, types2_1) &&
						IsEqualSets(types1_2, types2_2)) ||
						(IsEqualSets(types1_1, types2_2) &&
						IsEqualSets(types1_2, types2_1)))
						isMatched = true;
					break;
				case 3:
					// order mush match, check connecting nodes' types
					AlignAdjNodes(eorder1, conntypes1);
					AlignAdjNodes(eorder2, conntypes2);
					List<List<int>> types1 = new List<List<int>>(),
						types2 = new List<List<int>>();
					for (int i = 0; i < 3; ++i)
					{
						List<int> empty = new List<int>();
						types1.Add(empty);
						empty = new List<int>();
						types2.Add(empty);
					}
					IEnumerable<int> eorder1_d = eorder1.Distinct(),
						eorder2_d = eorder2.Distinct();
					for (int i = 0; i < conntypes1.Count; ++i)
					{
						for (int j = 0; j < 3; ++j)
						{
							if (eorder1[i] == eorder1_d.ElementAt(j))
								types1[j].Add(conntypes1[i]);
						}
					}
					for (int i = 0; i < conntypes2.Count; ++i)
					{
						for (int j = 0; j < 3; ++j)
						{
							if (eorder2[i] == eorder2_d.ElementAt(j))
								types2[j].Add(conntypes2[i]);
						}
					}
					if(!IsEqualSets(types1[1], types2[1])) return false;
					if ((IsEqualSets(types1[0], types2[0]) &&
						IsEqualSets(types1[2], types2[2])) ||
						(IsEqualSets(types1[0], types2[2]) &&
						IsEqualSets(types1[2], types2[0])))
						isMatched = true;
					break;
			}
			return isMatched;
		}//MatchingNodes

		private void OrderEdgeByDir(ModelGraphNode mn, out List<int> eorder, out List<int> conntypes)
		{
			eorder = new List<int>();
			conntypes = new List<int>();
			foreach(ModelGraphEdge me in mn.edges)
			{
				if (me.support) continue; // already checked
				if (me.conntectingNodes[0] == mn)
				{
					eorder.Add( MapXYZTOOrder(me.faceIndex[0]) );
					conntypes.Add( me.conntectingNodes[1].nodeType );
				}
				else
				{
					eorder.Add( MapXYZTOOrder(me.faceIndex[1]) );
					conntypes.Add( me.conntectingNodes[0].nodeType );
				}
			}
			int nedges = eorder.Count;
			// non-descending order
			for (int i = 0; i < nedges - 1; ++i)
			{
				for (int j = i + 1; j < nedges; j++)
				{
					if (eorder[j] < eorder[i])
					{
						int tmp = eorder[j];
						eorder[j] = eorder[i];
						eorder[i] = tmp;
						tmp = conntypes[j];
						conntypes[j] = conntypes[i];
						conntypes[i] = tmp;
					}
				}
			}
		}//OrderEdgeByDir

		private void FindSupportNodes(ModelGraphNode mn, out List<int> upper, out List<int> lower)
		{
			upper = new List<int>();
			lower = new List<int>();
			foreach (ModelGraphEdge me in mn.edges)
			{
				if (!me.support) continue;
				if (me.faceIndex[0] == 1 && me.conntectingNodes[0] == mn)
					upper.Add(me.conntectingNodes[1].nodeType);
				else
					lower.Add(me.conntectingNodes[0].nodeType);
			}
		}//FindSupportNodes

		private void AlignAdjNodes(List<int> eorder, List<int> conntype)
		{
			// align 3-conn node to continuous order to facilitate further comparison
			bool isChecked = true;
			while (isChecked)
			{
				isChecked = false;
				for (int i = 0; i < eorder.Count; ++i)
				{
					if ((eorder[i] + 1) % 4 != eorder[i + 1])
					{
						int tmp = eorder[0];
						while (eorder[0] == tmp)
						{
							eorder.RemoveAt(0);
							eorder.Add(tmp);
							int tmpc = conntype[0];
							conntype.RemoveAt(0);
							conntype.Add(tmpc);
						}
						isChecked = true;
						break;
					}
				}
			}
		}//AlignAdjNodes

		private bool IsEqualSets(List<int> set1, List<int> set2)
		{
			return (set1.Except(set2).Count() == 0 && set2.Except(set1).Count() == 0);
		}//IsEqualSets

		private int MapXYZTOOrder(int direction)
		{
			int index = -1;
			switch (direction)
			{
				case 1:
					index = 1;
					break;
				case -1:
					index = 3;
					break;
				case 2:
					index = 5;
					break;
				case -2:
					index = 6;
					break;
				case 3:
					index = 4;
					break;
				case -3:
					index = 2;
					break;
			}
			return index;
		}//MapXYZTOOrder

		//////////////////// end-2014-08-24 ////////////////////
		
		//////////////////// previous search using dfs ////////////////////
		public void ExtractSharedComponents()
		{
			//---Extract shared components among all models---//
			if (this.kidModelGraphs == null || this.kidModelGraphs.Count == 0) return;
			this.sharedComponents = new List<int>();
			this.subSharedComponents = new List<int>();
			// how many model got this same type with same number
			int[] numRepetition = new int[MaxNumberNodeType.MAXNODETYPE];
			for (int i = 0; i < MaxNumberNodeType.MAXNODETYPE; i++)
				numRepetition[i] = 0;
			for (int itype = 0; itype < MaxNumberNodeType.MAXNODETYPE; itype++)
			{
				List<int> numTypes = new List<int>();
				foreach (ModelGraph mg in this.kidModelGraphs)
				{
					// already initialized when graph was loaded
					int nitype = mg.nodesByType[itype].Count;
					if(!numTypes.Contains(nitype))
						numTypes.Add(nitype);
				}
				if (numTypes.Count == 1 && numTypes[0] != 0)
					sharedComponents.Add(itype);
				else if (numTypes.Count < 3 && !(numTypes.Count == 1 && numTypes[0] == 0))
					subSharedComponents.Add(itype);
			}
		}//End-construct graph group		

		public List<ModelGraphEdge> a_route, b_route;

		public int ISACut(ModelGraph mg, List<ModelGraphEdge> edges, int tag, out Subgraph sg)
		{
			//bool iscut = false;
			int ncut = 0;
			// a cut split the graph into several ( at least 2 ) independent sets
			int nEdges = mg.graphEdges.Count;
			List<ModelGraphNode> activeNodes = new List<ModelGraphNode>();
			int nActNodes;
			// label inactive edges
			bool[] isActEdges = new bool[nEdges];
			for (int i = 0; i < nEdges; ++i)
				isActEdges[i] = true;
			for (int i = 0; i < edges.Count; i++)
			{
				ModelGraphNode mn1 = edges[i].conntectingNodes[0];
				ModelGraphNode mn2 = edges[i].conntectingNodes[1];
				int index = mg.graphEdges.IndexOf(edges[i]);
				isActEdges[index] = false;
				if (!activeNodes.Contains(mn1))
					activeNodes.Add(mn1);
				if (!activeNodes.Contains(mn2))
					activeNodes.Add(mn2);
			}
			nActNodes = activeNodes.Count;
			// DFS to find out inaccessible nodes
			bool[,] isAccNodes = new bool[nActNodes, nActNodes];
			List<bool> accNodes = new List<bool>();
			int nNodes = mg.meshComponents.Count;
			bool[,] accessNodesForIndependentSet = new bool[nNodes, nNodes];
			// start from each node (according to the edges), 
			// find all nodes that can be accessed from it by removing all edges in the input
			for (int n = 0; n < nActNodes; n++)
			{
				bool[] isaccess = new bool[nNodes];
				for (int i = 0; i < mg.meshComponents.Count; ++i)
					isaccess[i] = false;
				List<ModelGraphNode> queue = new List<ModelGraphNode>();
				queue.Add(activeNodes[n]);
				while (queue.Count > 0)
				{
					ModelGraphNode currNode = queue[0];
					queue.RemoveAt(0);
					foreach (ModelGraphEdge c in currNode.edges)
					{
						ModelGraphNode mn = null;
						if (c.conntectingNodes[0] == currNode)
							mn = c.conntectingNodes[1];
						else
							mn = c.conntectingNodes[0];
						int index = mg.graphEdges.IndexOf(c);
						if (!isaccess[mn.nodeIdx] && isActEdges[index])
						{
							queue.Add(mn);
							isaccess[mn.nodeIdx] = true;
						}
					}
				}//end-while
				// record inaccessible nodes
				isAccNodes[n, n] = true;
				for (int i = 0; i < nActNodes; i++)
					isAccNodes[n, i] = isaccess[activeNodes[i].nodeIdx];
				for (int i = 0; i < nNodes; ++i)
					accessNodesForIndependentSet[n, i] = isaccess[i];
			}//end-while

			// stat. the independent node sets
			List<ModelGraphNode> nodeset, partset;
			bool[] visited = new bool[nActNodes];
			for (int i = 0; i < nActNodes; ++i)
				visited[i] = false;
			List<List<ModelGraphNode>> indepNodeSets = new List<List<ModelGraphNode>>();
			for (int i = 0; i < nActNodes; i++)
			{
				if (visited[i])
					continue;
				visited[i] = true;
				nodeset = new List<ModelGraphNode>();
				nodeset.Add(activeNodes[i]);
				partset = new List<ModelGraphNode>();
				partset.Add(activeNodes[i]);
				List<int> queue = new List<int>(); // DFS to define an independent set
				queue.Add(i);
				while (queue.Count > 0)
				{
					int currActNode = queue[0];
					queue.RemoveAt(0);
					for (int j = 0; j < nActNodes; j++)
					{
						if (!visited[j] && isAccNodes[currActNode, j])
						{
							visited[j] = true;
							nodeset.Add(activeNodes[j]);
							queue.Add(j);
						}
					}
				}
				indepNodeSets.Add(nodeset);
			}
			ncut = indepNodeSets.Count;

			if (tag == 0)
			{
				sg = null;
				return ncut;
			}

			List<Subgraph> subgraphs = new List<Subgraph>();
			int sgIdx = -1;
			for (int i = 0; i < ncut; ++i)
			{
				List<ModelGraphEdge> outEdges = mg.GetOutgoingEdges(indepNodeSets[i]);
				bool isSplitSG = true;
				List<ModelGraphEdge> cts = new List<ModelGraphEdge>();
				for(int j = 0; j < outEdges.Count; ++j)
				{
					cts.Add(outEdges[j]);
				}
				foreach(ModelGraphEdge e in edges)
				{
				if (!cts.Contains(e))
					{
						isSplitSG = false;
						break;
					}
				}
				if (!isSplitSG) continue;
				List<ModelGraphEdge> inEdges = mg.GetIngoingEdges(indepNodeSets[i]);
				Subgraph subgraph = new Subgraph(mg.graphIndex, indepNodeSets[i], outEdges, inEdges);
				subgraphs.Add(subgraph);
				sgIdx = i;
			}
			//if (subgraphs.Count == 1)
			if(subgraphs.Count > 0)
			{
				sg = subgraphs[0];
				List<ModelGraphNode> allMCs = GetAllConnectedMCs(mg, edges, sg.nodeSets);
				List<ModelGraphEdge> outEdges = mg.GetOutgoingEdges(allMCs);
				List<ModelGraphEdge> inEdges = mg.GetIngoingEdges(allMCs);
				sg = new Subgraph(mg.graphIndex, allMCs, outEdges, inEdges);
			}
			else
				sg = null;
			//if (nindepSet == 2)
			//    iscut = true;
			return ncut;
		}

		public List<ModelGraphNode> GetAllConnectedMCs(ModelGraph mg, List<ModelGraphEdge> edges, List<ModelGraphNode> Mcs)
		{
			List<ModelGraphNode> connectedMCs = new List<ModelGraphNode>();
			connectedMCs.AddRange(Mcs);
			int nadded = 0;
			while (true)
			{
				nadded = 0;
				foreach (ModelGraphNode mn in mg.graphNodes)
				{
					if (connectedMCs.Contains(mn)) continue;
					bool included = false;
					if (mn.edges == null) continue;
					foreach (ModelGraphEdge c in mn.edges)
					{
						// edge nodes (not impossible included by two parallel cuts
						if (edges.Contains(c)) continue;
						ModelGraphNode mn1 = c.conntectingNodes[0];
						ModelGraphNode mn2 = c.conntectingNodes[1];
						if ((connectedMCs.Contains(mn1) && !connectedMCs.Contains(mn2))||
							(connectedMCs.Contains(mn2) && !connectedMCs.Contains(mn1)))
						{
							included = true;
							break;
						}
					}
					if (included)
					{
						connectedMCs.Add(mn);
						nadded++;
					}
				}
				if (nadded == 0)
					break;
			}
			return connectedMCs;
		}

		public void CollectSubgraphs(int graphIDx)
		{
			ModelGraph mg = this.kidModelGraphs[graphIDx];
			mg.inModelReplaceableSubgraphs = new List<Subgraph>();
			List<List<ModelGraphNode>> subgraphMCs = new List<List<ModelGraphNode>>();
			for (int ic = 0; ic < mg.equCutpairs.Count; ic += 2)
			{
				if (mg.equCutpairs[ic].Count < 2)
				{
					mg.equCutpairs.RemoveAt(ic + 1);
					mg.equCutpairs.RemoveAt(ic);
					ic -= 2;
					continue;
				}
				List<ModelGraphEdge> edges = new List<ModelGraphEdge>();
				edges.AddRange(mg.equCutpairs[ic]);
				edges.AddRange(mg.equCutpairs[ic+1]);
				Subgraph sg = null;
				ISACut(mg, edges, 1, out sg);
				// for extreme case, not cut through 
				if (sg == null)// || sg.nodeSets.Count > mg.meshComponents.Count / 2)
				{
					mg.equCutpairs.RemoveAt(ic + 1);
					mg.equCutpairs.RemoveAt(ic);
					ic -= 2;
					continue;
				}
				
				// direction
				double maxx = double.MinValue, maxy = double.MinValue;
				ModelGraphEdge c1 = mg.equCutpairs[ic][0];
				for (int i = 1; i < mg.equCutpairs[ic].Count; ++i)
				{
					ModelGraphEdge c2 = mg.equCutpairs[ic][i];
					maxx = Math.Max(maxx, Math.Abs(c2.pos.x - c1.pos.x));
					maxy = Math.Max(maxy, Math.Abs(c2.pos.y - c1.pos.y));
				}
				if (maxx > maxy)
					sg.isvertical = true;
				//double diffx = mg.equCutpairs[ic][0].pos.x - mg.equCutpairs[ic + 1][0].pos.x;
				//double diffy = mg.equCutpairs[ic][0].pos.y - mg.equCutpairs[ic + 1][0].pos.y;
				//if (Math.Abs(diffx) > Math.Abs(diffy))
				//    sg.isvertical = true;
				if (sg.isvertical)
				{
					if (mg.equCutpairs[ic][0].pos.y < mg.equCutpairs[ic + 1][0].pos.y)
					{
						sg.connEdges1 = mg.equCutpairs[ic];
						sg.connEdges2 = mg.equCutpairs[ic + 1];
					}
					else
					{
						sg.connEdges1 = mg.equCutpairs[ic + 1];
						sg.connEdges2 = mg.equCutpairs[ic];
					}
				}
				else
				{
					if (mg.equCutpairs[ic][0].pos.x < mg.equCutpairs[ic + 1][0].pos.x)
					{
						sg.connEdges1 = mg.equCutpairs[ic];
						sg.connEdges2 = mg.equCutpairs[ic + 1];
					}
					else
					{
						sg.connEdges1 = mg.equCutpairs[ic + 1];
						sg.connEdges2 = mg.equCutpairs[ic];
					}
				}
				mg.inModelReplaceableSubgraphs.Add(sg);
				sg.connNodes1 = new List<ModelGraphNode>();
				sg.connNodes2 = new List<ModelGraphNode>();
				foreach (ModelGraphEdge c in sg.connEdges1)
				{
					ModelGraphNode mn1 = c.conntectingNodes[0];
					ModelGraphNode mn2 = c.conntectingNodes[1];
					int ind1 = sg.nodeSets.IndexOf(mn1);
					int ind2 = sg.nodeSets.IndexOf(mn2);
					if ((ind1 == -1 && ind2 == -1) || (ind1 != -1 && ind2 != -1))
					{
						Program.OutputText("Wrong!", true);
						break;
					}
					if (ind1 != -1)
						sg.connNodes1.Add(mn1);
					else if (ind2 != -1)
						sg.connNodes1.Add(mn2);
				}
				foreach (ModelGraphEdge c in sg.connEdges2)
				{
					ModelGraphNode mn1 = c.conntectingNodes[0];
					ModelGraphNode mn2 = c.conntectingNodes[1];
					int ind1 = sg.nodeSets.IndexOf(mn1);
					int ind2 = sg.nodeSets.IndexOf(mn2);
					if ((ind1 == -1 && ind2 == -1) || (ind1 != -1 && ind2 != -1))
					{
						Program.OutputText("Wrong!", true);
						break;
					}
					if (ind1 != -1)
						sg.connNodes2.Add(mn1);
					else if (ind2 != -1)
						sg.connNodes2.Add(mn2);
				}

				//// not cut through
				//List<ModelGraphNode> allconnMCs = GetAllConnectedMCs(graphIDx, mg.equCutpairs[ic], connMCs);
				//if (allconnMCs.Count + sg.connMNs1.Count >= mg.meshComponents.Count)
				//{
				//    //not cut through
				//    mg.inModelReplaceableSubgraphs.Remove(sg);
				//    mg.equCutpairs.RemoveAt(ic+1);
				//    mg.equCutpairs.RemoveAt(ic);
				//    ic -= 2;
				//    continue;
				//}
				
				subgraphMCs.Add(sg.nodeSets);
			}//end-iterate
			// if any subgraph include the other, 
			// mark the smallest for deform
			List<Subgraph> allSubgraphs = new List<Subgraph>();
			foreach (Subgraph sg in mg.inModelReplaceableSubgraphs)
				allSubgraphs.Add(sg);
			for (int i = 0; i < subgraphMCs.Count-1; ++i)
			{
				bool i_included = false, j_included = false;
				for (int j = i+1; j < subgraphMCs.Count; ++j)
				{
					if (subgraphMCs[i].Count > subgraphMCs[j].Count)
					{
						i_included = true;
						foreach (ModelGraphNode mn in subgraphMCs[j])
						{
							if (!subgraphMCs[i].Contains(mn))
							{
								i_included = false;
								break;
							}
						}
						if (i_included)
						{
							subgraphMCs.RemoveAt(i);
							allSubgraphs.RemoveAt(i);
							//mg.equCutpairs.RemoveAt(2 * i + 1);
							//mg.equCutpairs.RemoveAt(2 * i);
							--i;
						}
					}
					else
					{
						foreach (ModelGraphNode mn in subgraphMCs[i])
						{
							j_included = true;
							if (!subgraphMCs[j].Contains(mn))
							{
								j_included = false;
								break;
							}
						}
						if (j_included)
						{
							subgraphMCs.RemoveAt(j);
							allSubgraphs.RemoveAt(j);
							//mg.equCutpairs.RemoveAt(2 * j + 1);
							//mg.equCutpairs.RemoveAt(2 * j);
							--j;
						}
					}
					if (i_included) break;
				}//end-j
			}//end-i
		}//end-collect-subgraph

		public void Calspaces(ModelGraph mg)
		{
			foreach (Subgraph sg in mg.inModelReplaceableSubgraphs)
			{
				// min of max, max of min
				int dir = 0;
				if (sg.isvertical)
					dir = 1;
				Vector3d minCoord = new Vector3d(double.MaxValue, double.MaxValue, double.MaxValue),
					maxCoord = new Vector3d(double.MinValue, double.MinValue, double.MinValue);
				double mindir = 0, maxdir = 0;
				Vector3d vec1 = new Vector3d(double.MaxValue, double.MaxValue, double.MaxValue),
					vec2 = new Vector3d(double.MinValue, double.MinValue, double.MinValue);
				foreach (ModelGraphNode mn in sg.nodeSets)
				{
					minCoord = Vector3d.Min(minCoord, mn.meshComponent.MinCoord());
					maxCoord = Vector3d.Max(maxCoord, mn.meshComponent.MaxCoord());
					mindir += mn.meshComponent.MinCoord()[dir];
					maxdir += mn.meshComponent.MaxCoord()[dir];
					if (mn.nodeType <8)
					{
						vec1 = Vector3d.Min(vec1, mn.meshComponent.MinCoord());
						vec2 = Vector3d.Max(vec2, mn.meshComponent.MaxCoord());
					}
				}
				sg.widheig[0] = maxCoord.x - minCoord.x;
				sg.widheig[1] = maxCoord.y - minCoord.y;

				sg.widheig[0] = vec2.x - vec1.x;
				sg.widheig[1] = vec2.y - vec1.y;
				//sg.widheig[dir] = maxdir / sg.nodeSets.Count - mindir/sg.nodeSets.Count;
				sg.moveLimit[1] = maxCoord.y;
				sg.moveLimit[0] = maxCoord.x;

				double dist1 = 0, dist2 = 0;
				
				foreach (ModelGraphEdge c1 in sg.connEdges1)
				{
					ModelGraphNode mn1 = c1.conntectingNodes[0];
					ModelGraphNode mn2 = c1.conntectingNodes[1];
					double len1 = mn1.meshComponent.MaxCoord()[dir] - mn1.meshComponent.MinCoord()[dir];
					double len2 = mn2.meshComponent.MaxCoord()[dir] - mn2.meshComponent.MinCoord()[dir];
					double len = Math.Max(mn1.meshComponent.MaxCoord()[dir], mn2.meshComponent.MaxCoord()[dir])
						- Math.Min(mn1.meshComponent.MinCoord()[dir], mn2.meshComponent.MinCoord()[dir]);
					dist1 += len - len1 - len2;
				}
				dist1 /= sg.connEdges1.Count;

				foreach (ModelGraphEdge c2 in sg.connEdges2)
				{
					ModelGraphNode mn1 = c2.conntectingNodes[0];
					ModelGraphNode mn2 = c2.conntectingNodes[1];
					double len1 = mn1.meshComponent.MaxCoord()[dir] - mn1.meshComponent.MinCoord()[dir];
					double len2 = mn2.meshComponent.MaxCoord()[dir] - mn2.meshComponent.MinCoord()[dir];
					double len = Math.Max(mn1.meshComponent.MaxCoord()[dir], mn2.meshComponent.MaxCoord()[dir])
						- Math.Min(mn1.meshComponent.MinCoord()[dir], mn2.meshComponent.MinCoord()[dir]);
					dist2 += len - len1 - len2;
				}
				dist2 /= sg.connEdges1.Count;
				sg.distToNeigs[0] = dist1;
				sg.distToNeigs[1] = dist2;
			}
		}

		List<Subgraph> sampleSubgraphs = null;
		public void SampleCuts(ModelGraph mg)
		{
			// starting from an edge, iterate back to itself
			sampleSubgraphs = new List<Subgraph>();
			foreach (ModelGraphEdge me in mg.graphEdges)
			{
				a_route = new List<ModelGraphEdge>();
				foreach (ModelGraphEdge mge in mg.graphEdges)
					mge.visitedEdgeInSearch = false;

				if (mg.modelType == 10)
				{
					List<int> nodetypes = new List<int>();
					nodetypes.Add(me.conntectingNodes[0].nodeType);
					nodetypes.Add(me.conntectingNodes[1].nodeType);
					if (!nodetypes.Contains(9) || !nodetypes.Contains(3))
						continue;
				}
				a_route.Add(me);
				for (int i = 0; i <= me.edgeIdx; ++i)
					mg.graphEdges[i].visitedEdgeInSearch = true;
				DFSCut(mg, me);
			}
			// Remove repetition
			for (int i = 0; i < sampleSubgraphs.Count-1; ++i)
			{
				for (int j = i + 1; j < sampleSubgraphs.Count; ++j)
				{
					if (IsEqualCut(sampleSubgraphs[i].outgoingEdges,
						sampleSubgraphs[j].outgoingEdges))
					{
						sampleSubgraphs.RemoveAt(j);
						--j;
					}
				}
			}
			// order
			for (int s = 0; s < sampleSubgraphs.Count; ++s)
			{
				Subgraph a_sg = sampleSubgraphs[s];
				int numToCompare = a_sg.outgoingEdges.Count;
				List<ModelGraphEdge> orderedEdges = null;
				List<ModelGraphNode> orderedNodes = null;
				mg.OrderEdges(a_sg.outgoingEdges, a_sg.connNodes, out orderedEdges, out orderedNodes);
				a_sg.outgoingEdges = orderedEdges;
				a_sg.connNodes = orderedNodes;

				a_sg.edgeNodes = new List<ModelGraphNode>();
				for (int i = 0; i < a_sg.outgoingEdges.Count; ++i)
				{
					ModelGraphEdge me = a_sg.outgoingEdges[i];
					if (me.conntectingNodes[0] == a_sg.connNodes[i])
						a_sg.edgeNodes.Add(me.conntectingNodes[1]);
					else
						a_sg.edgeNodes.Add(me.conntectingNodes[0]);
				}
			}
		}
		public int numCutThresh = 8;
		public void DFSCut(ModelGraph mg, ModelGraphEdge edge)
		{
			if (a_route.Count > this.numCutThresh) return;

			//if (a_route.Count > this.minmumCutedges) return;

			List<ModelGraphEdge> neigs = new List<ModelGraphEdge>();

			foreach (ModelGraphEdge e in edge.adjEdges)
			{
				if (!e.visitedEdgeInSearch)
				{
					neigs.Add(e);
				}
			}

			if (neigs.Count == 0 || edge.adjEdges.Contains(a_route[0]))
				//&& a_route.Count >= minmumCutedges))
			{
				List<Subgraph> outsgs = null;
				int iscut = HasAvalidSubgraph(mg, a_route, out outsgs);
				if (iscut > 1)
				{
					for (int i = 0; i < outsgs.Count; ++i)
					{
						if (outsgs[i].outgoingEdges.Count == a_route.Count
							&& outsgs[i].nodeSets.Count > 1
							&& outsgs[i].nodeSets.Count < mg.graphNodes.Count / 2)
						{
							// order of edge
							//List<ModelGraphEdge> orderEdges = new List<ModelGraphEdge>(a_route);
							//Subgraph sg = new Subgraph(mg.graphIndex, outsgs[i].nodeSets,
							//    orderEdges, outsgs[i].ingoingEdges);
							//sampleSubgraphs.Add(sg);
							sampleSubgraphs.Add(outsgs[i]);
						}
					}
				}
				if (neigs.Count == 0) return;
			}
			
			foreach (ModelGraphEdge me in neigs)
			{
				// mark the edges in the same cycle as "visited"
				// to avoid meaningless search
				//int nb = 0;
				//if (a_route.Count == 3 && a_route[0].edgeIdx == 0
				//    && a_route[1].edgeIdx == 2 && a_route[2].edgeIdx == 4
				//    && me.edgeIdx == 6)
				//    nb = 1;;
				if (mg.modelType == 10)
				{
					List<int> nodetypes = new List<int>();
					nodetypes.Add(me.conntectingNodes[0].nodeType);
					nodetypes.Add(me.conntectingNodes[1].nodeType);
					if (!nodetypes.Contains(9) || !nodetypes.Contains(3))
						continue;
				}
				List<int> sects = edge.cycleIndices.Intersect(me.cycleIndices).ToList();
				if (sects.Count() > 1) continue; // both in two cycles, meaningless (cut a node)
				List<int> labeledEidxs = new List<int>();
				if (sects.Count() == 1)
				{
					int cycleIdx = sects[0];
					foreach (ModelGraphEdge cme in mg.faceCycles[cycleIdx].edges)
					{
						cme.visitedEdgeInSearch = true;
						labeledEidxs.Add(cme.edgeIdx);
					}
				}
				else
				{
					// put *a_edge*'s neighbors are labeled
					foreach (ModelGraphEdge adje in neigs)
					{
						int adjindex = edge.adjEdges.IndexOf(adje);
						if (adje.cycleIndices.Count == 0 && edge.trueAdjEdges[adjindex])
						{
							adje.visitedEdgeInSearch = true;
							labeledEidxs.Add(adje.edgeIdx);
						}
					}
				}
				a_route.Add(me);
				me.visitedEdgeInSearch = true;
				DFSCut(mg, me);
				me.visitedEdgeInSearch = false;
				a_route.Remove(me);
				for (int i = 0; i < labeledEidxs.Count; ++i)
					mg.graphEdges[labeledEidxs[i]].visitedEdgeInSearch = false;
			}
		}

		public void GetSymmetryCuts(ModelGraph a_mg, ModelGraph b_mg)
		{
			// initialization
			this.SetNodeIndex(a_mg);
			this.SetNodeIndex(b_mg);
			a_mg.equCutpairs = new List<List<ModelGraphEdge>>();
			b_mg.equCutpairs = new List<List<ModelGraphEdge>>();
			a_mg.equSubgraphs = new List<Subgraph>();
			b_mg.equSubgraphs = new List<Subgraph>();
			bool[] a_visited = new bool[a_mg.graphEdges.Count],
				b_visited = new bool[b_mg.graphEdges.Count];
			for (int ai = 0; ai < a_mg.graphEdges.Count; ++ai)
			{
				if (a_visited[ai]) continue;
				ModelGraphEdge a_e = a_mg.graphEdges[ai];
				a_route = new List<ModelGraphEdge>();
				a_route.Add(a_e);
				// collect all same types of edges
				for (int aj = ai + 1; aj < a_mg.graphEdges.Count; ++aj)
				{
					if (IsEqualEdge(a_mg.graphEdges[aj], a_e))
						a_route.Add(a_mg.graphEdges[aj]);
				}
				b_route = new List<ModelGraphEdge>();
				for (int bi = 0; bi < b_mg.graphEdges.Count; ++bi)
				{
					ModelGraphEdge b_e = b_mg.graphEdges[bi];
					if (IsEqualEdge(a_e, b_e))
						b_route.Add(b_e);
				}

				List<Subgraph> a_outsgs = null, b_outsgs = null;
				int iscut_a = HasAvalidSubgraph(a_mg, a_route, out a_outsgs);
				int iscut_b = HasAvalidSubgraph(b_mg, b_route, out b_outsgs);
				foreach (ModelGraphEdge me in a_route)
					a_visited[me.edgeIdx] = true;
				foreach (ModelGraphEdge me in b_route)
					b_visited[me.edgeIdx] = true;
				if (iscut_a <= 1 || iscut_b <= 1) continue;

				for (int ia = 0; ia < a_outsgs.Count; ++ia)
				{
					if (a_outsgs[ia].outgoingEdges.Count != a_route.Count) continue;
					bool valid = true;
					for (int j = 0; j < a_route.Count; ++j)
					{
						if (!a_outsgs[ia].outgoingEdges.Contains(a_route[j]))
						{
							valid = false;
							break;
						}
					}
					if (!valid) continue;
					int edgetype = a_outsgs[ia].edgeNodes[0].nodeType;
					for (int ib = 0; ib < b_outsgs.Count; ++ib)
					{
						if (b_outsgs[ib].outgoingEdges.Count != b_route.Count) continue;
						valid = true;
						for (int j = 0; j < b_route.Count; ++j)
						{
							if (!b_outsgs[ib].outgoingEdges.Contains(b_route[j]))
							{
								valid = false;
								break;
							}
						}
						if (!valid || b_outsgs[ib].edgeNodes[0].nodeType != edgetype)
							continue;
						a_mg.equSubgraphs.Add(a_outsgs[ia]);
						b_mg.equSubgraphs.Add(b_outsgs[ib]);
						a_mg.equCutpairs.Add(a_outsgs[ia].outgoingEdges);
						b_mg.equCutpairs.Add(b_outsgs[ib].outgoingEdges);
						// check the left graph
						Subgraph a_sg = null, b_sg = null;
						List<ModelGraphNode> a_nodeset = new List<ModelGraphNode>(),
							b_nodeset = new List<ModelGraphNode>();
						foreach (ModelGraphNode mn in a_mg.graphNodes)
						{
							if (!a_outsgs[ia].nodeSets.Contains(mn))
								a_nodeset.Add(mn);
						}
						foreach (ModelGraphNode mn in b_mg.graphNodes)
						{
							if (!b_outsgs[ib].nodeSets.Contains(mn))
								b_nodeset.Add(mn);
						}
						List<ModelGraphEdge> outEdges = a_mg.GetOutgoingEdges(a_nodeset);
						List<ModelGraphEdge> inEdges = a_mg.GetIngoingEdges(a_nodeset);
						a_sg = new Subgraph(a_mg.graphIndex, a_nodeset, outEdges, inEdges);

						outEdges = b_mg.GetOutgoingEdges(b_nodeset);
						inEdges = b_mg.GetIngoingEdges(b_nodeset);
						b_sg = new Subgraph(b_mg.graphIndex, b_nodeset, outEdges, inEdges);

							a_mg.equSubgraphs.Add(a_sg);
							b_mg.equSubgraphs.Add(b_sg);
							a_mg.equCutpairs.Add(a_sg.outgoingEdges);
							b_mg.equCutpairs.Add(b_sg.outgoingEdges);
					}
				}// collect	
			}//search			
		}//end-get-symmetry-edges

		// search replaceable subgraphs from two graph
		public void SearchReplaceableSubgraphs_old(ModelGraph a_mg, ModelGraph b_mg, 
			int maxdistanceOfdummyedges)
		{
			// initialization
			this.SetNodeIndex(a_mg);
			this.SetNodeIndex(b_mg);
			a_mg.equCutpairs = new List<List<ModelGraphEdge>>();
			b_mg.equCutpairs = new List<List<ModelGraphEdge>>();
			a_mg.equSubgraphs = new List<Subgraph>();
			b_mg.equSubgraphs = new List<Subgraph>();
			// search

			if (maxdistanceOfdummyedges == 0)
			{
				this.SearchEqualCuts(a_mg, b_mg, true);
				return;
			}

			//*********With dummy edge***********//
			int start = 0;
			// make the adjedges of adjedges of each edge become its own adjedges
			// in every iteration, just repeat this step without any removal
			//a_mg.AddDummyEdgesBydistance();
			//b_mg.AddDummyEdgesBydistance();
			// add one dummy edge each time
			for (int ai = 0; ai < a_mg.graphEdges.Count; ++ai)
			{
				ModelGraphEdge a_e = a_mg.graphEdges[ai];
				// all possible dummy edges for a_e
				List<ModelGraphEdge> adjEdges_a_e = new List<ModelGraphEdge>();
				start = 0;
				int startIdx = 0;
				adjEdges_a_e.AddRange(a_e.adjEdges);
				int dist_1_adj_edges = a_e.adjEdges.Count;
				while (start < maxdistanceOfdummyedges)
				{
					int ncurrent = adjEdges_a_e.Count;
					for (int i = startIdx; i < ncurrent; ++i)
					{
						ModelGraphEdge adjEdge = adjEdges_a_e[i];
						foreach (ModelGraphEdge a_me in adjEdge.adjEdges)
						{
							if (!a_e.adjEdges.Contains(a_me)&& !adjEdges_a_e.Contains(a_me))
							{
									adjEdges_a_e.Add(a_me);
							}
						}
					}
					++start;
					startIdx += ncurrent;
				}
				for (int i = 0; i < dist_1_adj_edges; ++i)
					adjEdges_a_e.RemoveAt(0);
				int nadjedges_a = adjEdges_a_e.Count;
				// all possible adj edges of a_e
				for (int nadj_a = 0; nadj_a < nadjedges_a; ++nadj_a)
				{
					ModelGraphEdge edge_nadj_a = adjEdges_a_e[nadj_a];
					List<ModelGraphEdge> a_candEdges = new List<ModelGraphEdge>();
					foreach(ModelGraphEdge me_adj in a_e.adjEdges)
					{
							a_candEdges.Add(me_adj);
					}
					a_e.adjEdges.Add(edge_nadj_a);
					if (a_e.trueAdjEdges != null && a_e.trueAdjEdges.Count > 0)
						a_e.trueAdjEdges.Add(true);
					edge_nadj_a.adjEdges.Add(a_e);
					for (int bi = 0; bi < b_mg.graphEdges.Count; ++bi)
					{
						// check whether two edges can connect
						ModelGraphEdge b_e = b_mg.graphEdges[bi];
						if (IsEqualEdge(a_e, b_e))
						{
							//add dummy edge for b_e
							List<ModelGraphEdge> adjEdges_b_e = new List<ModelGraphEdge>();
							start = 0;
							startIdx = 0;
							adjEdges_b_e.AddRange(b_e.adjEdges);
							dist_1_adj_edges = b_e.adjEdges.Count;
							while (start < maxdistanceOfdummyedges)
							{
								int ncurrent = adjEdges_b_e.Count;
								for (int i = startIdx; i < ncurrent; ++i)
								{
									ModelGraphEdge adjEdge = adjEdges_b_e[i];
									foreach (ModelGraphEdge b_me in adjEdge.adjEdges)
									{
										if (!b_e.adjEdges.Contains(b_me) && !adjEdges_b_e.Contains(b_me))
										{
												adjEdges_b_e.Add(b_me);
										}
									}
								}
								++start;
								startIdx += ncurrent;
							}//
							for (int i = 0; i < dist_1_adj_edges; ++i)
								adjEdges_b_e.RemoveAt(0);
							int nadjedges_b = adjEdges_b_e.Count;
							for (int nadj_b = 0; nadj_b < nadjedges_b; ++nadj_b)
							{
								ModelGraphEdge edge_nadj_b = adjEdges_b_e[nadj_b];
								// to save computation -- only  add the same dummy edge
								if (!IsEqualEdge(edge_nadj_a, edge_nadj_b))
									continue;
								// start from adjacent edges only
								List<ModelGraphEdge> b_candEdges = new List<ModelGraphEdge>();
								foreach (ModelGraphEdge me_adj in b_e.adjEdges)
								{
									b_candEdges.Add(me_adj);
								}
								b_e.adjEdges.Add(edge_nadj_b);
								if (b_e.trueAdjEdges != null && b_e.trueAdjEdges.Count > 0)
									b_e.trueAdjEdges.Add(false);
								edge_nadj_b.adjEdges.Add(b_e);
								foreach (ModelGraphEdge cand_a in a_candEdges)
								{
									foreach (ModelGraphEdge cand_b in b_candEdges)
									{
										if (IsEqualEdge(cand_a, cand_b))
										{
											a_route = new List<ModelGraphEdge>();
											b_route = new List<ModelGraphEdge>();
											a_route.Add(cand_a);
											b_route.Add(cand_b);
											foreach (ModelGraphEdge e in a_mg.graphEdges)
												e.visitedEdgeInSearch = false;
											foreach (ModelGraphEdge e in b_mg.graphEdges)
												e.visitedEdgeInSearch = false;
											cand_a.visitedEdgeInSearch = true;
											cand_b.visitedEdgeInSearch = true;
											//DFSAnEdge(a_mg, b_mg, cand_a, cand_b);
											DFSAnEdge_old(a_mg, b_mg, cand_a, cand_b);
										}
									}
								}//end-possible-match
								b_e.adjEdges.Remove(edge_nadj_b);
								if (b_e.trueAdjEdges != null && b_e.trueAdjEdges.Count > 0)
									b_e.trueAdjEdges.RemoveAt(b_e.trueAdjEdges.Count-1);
								edge_nadj_b.adjEdges.Remove(b_e);
							}//end-b-dummy
						}//serach if equal
					}
					a_e.adjEdges.Remove(edge_nadj_a);
					if (a_e.trueAdjEdges != null && a_e.trueAdjEdges.Count > 0)
						a_e.trueAdjEdges.RemoveAt(a_e.trueAdjEdges.Count - 1);
					edge_nadj_a.adjEdges.Remove(a_e);
				}//end-a-dummy
			}
			Filtersoultions(a_mg, b_mg);
		}//end-searc-replaceable

		public void SearchReplaceableSubgraphs(ModelGraph a_mg, ModelGraph b_mg,
			int maxdistanceOfdummyedges, bool isIterative)
		{
			// return the subgraph index with the maximum scale change
			// initialization
			this.SetNodeIndex(a_mg);
			this.SetNodeIndex(b_mg);

			a_route = new List<ModelGraphEdge>();
			b_route = new List<ModelGraphEdge>();
			a_mg.equCutpairs = new List<List<ModelGraphEdge>>();
			b_mg.equCutpairs = new List<List<ModelGraphEdge>>();
			a_mg.equSubgraphs = new List<Subgraph>();
			b_mg.equSubgraphs = new List<Subgraph>();
			//*********With dummy edge***********//
			List<List<ModelGraphEdge>> adjEdgesBydistance = new List<List<ModelGraphEdge>>();
			// make the adjedges of adjedges of each edge become its own adjedges
			// in every iteration, just repeat this step without any removal
			if (maxdistanceOfdummyedges > 0)
			a_mg.AddDummyEdgesBydistance(maxdistanceOfdummyedges);
			//a_mg.AddIsoAdjEdges();
			if (maxdistanceOfdummyedges > 0)
				b_mg.AddDummyEdgesBydistance(maxdistanceOfdummyedges);
			//b_mg.AddIsoAdjEdges();

			////sample cuts for a
			SampleCuts(a_mg);
			int nSample = sampleSubgraphs.Count;		

			Subgraph givenSG = null;

			for (int idx = 0; idx < nSample; ++idx)
			{
				givenSG = sampleSubgraphs[idx];
				int nedge = givenSG.outgoingEdges.Count;
				foreach (ModelGraphEdge me in b_mg.graphEdges)
				{
					if (!(a_mg.graphIndex == b_mg.graphIndex &&
						givenSG.outgoingEdges[0].edgeIdx == me.edgeIdx) &&
						IsEqualEdge(givenSG.outgoingEdges[0], me))
					{
						b_route = new List<ModelGraphEdge>();
						b_route.Add(me);
						foreach (ModelGraphEdge mge in b_mg.graphEdges)
							mge.visitedEdgeInSearch = false;
						me.visitedEdgeInSearch = true;
						DFSMatchedCut(a_mg, b_mg, givenSG, 1, me, isIterative);
					}
				}
			}
			
			Filtersoultions(a_mg, b_mg);
		
			
		}//end-searc-replaceable

		public void SelectExpandGraph(ModelGraph a_mg, ModelGraph b_mg)
		{
			for (int i = 0; i < a_mg.equSubgraphs.Count; ++i)
			{
				if (a_mg.equSubgraphs[i].nodeSets.Count > b_mg.equSubgraphs[i].nodeSets.Count)
				{
					Removesolution(a_mg, b_mg, i);
					--i;
				}
			}
		}
		List<List<ModelGraphEdge>> aEquCutPairs = null, bEquCutPairs = null;
		public void SearchReplaceableSubgraphsWithLevel(ModelGraph a_mg, ModelGraph b_mg, bool useHeight)
		{
			// initialization
			this.SetNodeIndex(a_mg);
			this.SetNodeIndex(b_mg);
			aEquCutPairs = new List<List<ModelGraphEdge>>();
			bEquCutPairs = new List<List<ModelGraphEdge>>();
			a_mg.equCutpairs = new List<List<ModelGraphEdge>>();
			b_mg.equCutpairs = new List<List<ModelGraphEdge>>();
			a_mg.equSubgraphs = new List<Subgraph>();
			b_mg.equSubgraphs = new List<Subgraph>();

			// 
			bool[] a_edge_cut = new bool[a_mg.graphEdges.Count],
				b_edge_cut = new bool[b_mg.graphEdges.Count];
			// only edges in cycles cannot lead to a cut
			for (int ai = 0; ai < a_mg.graphEdges.Count; ++ai)
				a_edge_cut[ai] = true;
			for (int bi = 0; bi < b_mg.graphEdges.Count; ++bi)
				b_edge_cut[bi] = true;

			foreach (ModelGraphEdge e in a_mg.graphEdges)
				e.adjEdges = new List<ModelGraphEdge>();
			foreach (ModelGraphEdge e in b_mg.graphEdges)
				e.adjEdges = new List<ModelGraphEdge>();

			foreach (Face face in a_mg.faceCycles)
			{
				foreach (ModelGraphEdge e in face.edges)
				{
					a_edge_cut[e.edgeIdx] = false;
				}
				for (int i = 0; i < face.edges.Count - 1; ++i)
				{
					for (int j = i + 1; j < face.edges.Count; ++j)
					{
						face.edges[i].adjEdges.Add(face.edges[j]);
						face.edges[j].adjEdges.Add(face.edges[i]);
					}
				}
			}

				foreach (Face face in b_mg.faceCycles)
				{
					foreach (ModelGraphEdge e in face.edges)
					{
						b_edge_cut[e.edgeIdx] = false;
					}
					for (int i = 0; i < face.edges.Count - 1; ++i)
					{
						for (int j = i + 1; j < face.edges.Count; ++j)
						{
							face.edges[i].adjEdges.Add(face.edges[j]);
							face.edges[j].adjEdges.Add(face.edges[i]);
						}
					}
				}

			// search
			for (int ai = 0; ai < a_mg.graphEdges.Count; ++ai)
			{
				ModelGraphEdge a_e = a_mg.graphEdges[ai];
				List<ModelGraphEdge> aes = new List<ModelGraphEdge>();
				aes.Add(a_e);
				for (int bi = 0; bi < b_mg.graphEdges.Count; ++bi)
				{
					// check whether two edges can connect					
					ModelGraphEdge b_e = b_mg.graphEdges[bi];
					if (!(a_mg.graphIndex == b_mg.graphIndex && a_e.edgeIdx == b_e.edgeIdx)
						&& IsEqualEdge(a_e, b_e))
					{
						a_route = new List<ModelGraphEdge>();
						b_route = new List<ModelGraphEdge>();
						a_route.Add(a_e);
						b_route.Add(b_e);
						foreach (ModelGraphEdge e in a_mg.graphEdges)
							e.visitedEdgeInSearch = false;
						foreach (ModelGraphEdge e in b_mg.graphEdges)
							e.visitedEdgeInSearch = false;
						a_e.visitedEdgeInSearch = true;
						b_e.visitedEdgeInSearch = true;
						this.DFSMatchedCutWithLevel(a_mg, b_mg, a_e, b_e, useHeight);
						List<ModelGraphEdge> bes = new List<ModelGraphEdge>();
						bes.Add(b_e);
						if (a_edge_cut[ai] && b_edge_cut[bi])
						{
							aEquCutPairs.Add(aes);
							bEquCutPairs.Add(bes);
						}
					}
				}
				
			}//search
			// 1. One cut
			// filter repetition
			for (int i = 0; i < aEquCutPairs.Count - 1; ++i)
			{
				for (int j = i + 1; j < aEquCutPairs.Count; ++j)
				{
					bool aequal = this.IsEqualCut(aEquCutPairs[i], aEquCutPairs[j]);
					bool bequal = this.IsEqualCut(bEquCutPairs[i], bEquCutPairs[j]);
					if (aequal && bequal)
					{
						aEquCutPairs.RemoveAt(j);
						bEquCutPairs.RemoveAt(j);
						--j;
						continue;
					}
					aequal = this.IsEqualCutPair(aEquCutPairs[i], bEquCutPairs[j]);
					bequal = this.IsEqualCutPair(aEquCutPairs[i], bEquCutPairs[j]);
					if (aequal && bequal)
					{
						aEquCutPairs.RemoveAt(j);
						bEquCutPairs.RemoveAt(j);
						--j;
					}
				}
			}

            // TEST
            //List<ModelGraphEdge> ae = new List<ModelGraphEdge>(), be = new List<ModelGraphEdge>();
            //ae.Add(a_mg.graphEdges[35]);
            //ae.Add(a_mg.graphEdges[45]);
            //be.Add(b_mg.graphEdges[34]);
            //be.Add(b_mg.graphEdges[44]);
            //aEquCutPairs.Add(ae);
            //bEquCutPairs.Add(be);

			for (int cidx = 0; cidx < aEquCutPairs.Count; ++cidx)
			{
				List<Subgraph> a_outsgs = null, b_outsgs = null;
				int iscut_a = HasAvalidSubgraph(a_mg, aEquCutPairs[cidx], out a_outsgs);
				int iscut_b = HasAvalidSubgraph(b_mg, bEquCutPairs[cidx], out b_outsgs);


                //if (cidx == aEquCutPairs.Count - 1)
                //    cidx = aEquCutPairs.Count - 1;
				if (iscut_a > 1 && iscut_b > 1)
				{
					if (iscut_a == 2 && iscut_b == 2 && (a_outsgs[0].nodeSets.Count < 2
							|| b_outsgs[0].nodeSets.Count < 2 ||
							a_outsgs[1].nodeSets.Count < 2
							|| b_outsgs[1].nodeSets.Count < 2))
						continue;
					List<Subgraph> matchedSubgraphs = IsReplaceableSubgraph(a_mg, b_mg, a_outsgs, b_outsgs, aEquCutPairs[cidx].Count, false);
					if (matchedSubgraphs.Count > 0)
					{
						for (int i = 0; i < matchedSubgraphs.Count; i += 2)
						{
							a_mg.equSubgraphs.Add(matchedSubgraphs[i]);
							b_mg.equSubgraphs.Add(matchedSubgraphs[i + 1]);
							a_mg.equCutpairs.Add(matchedSubgraphs[i].outgoingEdges);
							b_mg.equCutpairs.Add(matchedSubgraphs[i + 1].outgoingEdges);
						}
					}
				}
				else
				{
					aEquCutPairs.RemoveAt(cidx);
					bEquCutPairs.RemoveAt(cidx);
					--cidx;
				}
			}
			
			// filter
			for (int i = 0; i < a_mg.equCutpairs.Count; ++i)
			{
				bool del = false;
				if (a_mg.equSubgraphs[i].outgoingEdges.Count < minmumCutedges ||
					a_mg.equSubgraphs[i].nodeSets.Count < 2 ||
					b_mg.equSubgraphs[i].nodeSets.Count < 2 
                    //||(a_mg.equSubgraphs[i].nodeSets.Count > a_mg.graphNodes.Count * rate &&
                    //b_mg.equSubgraphs[i].nodeSets.Count > b_mg.graphNodes.Count * rate)
                    )
					del = true;
				if (!del)
				{
					// different level connection

					for (int j = 0; j < a_mg.equSubgraphs[i].edgeNodes.Count; ++j)
					{
						int eca = a_mg.equSubgraphs[i].edgeNodes[j].level -
							a_mg.equSubgraphs[i].connNodes[j].level;
						int ecb = b_mg.equSubgraphs[i].edgeNodes[j].level -
							b_mg.equSubgraphs[i].connNodes[j].level;
						if (eca * ecb < 0)
						{
							del = true;
							break;
						}
					}
				}
				if (!del)
				{
					List<int> alevels = new List<int>(), blevels = new List<int>();
					for (int j = 0; j < a_mg.equSubgraphs[i].nodeSets.Count; ++j)
						alevels.Add(a_mg.equSubgraphs[i].nodeSets[j].level);
					for (int j = 0; j < b_mg.equSubgraphs[i].nodeSets.Count; ++j)
						blevels.Add(b_mg.equSubgraphs[i].nodeSets[j].level);
					if ((alevels.Contains(0) && !blevels.Contains(0)) ||
						(!alevels.Contains(0) && blevels.Contains(0)))
						del = true;
				}
				if (del)
				{
					Removesolution(a_mg, b_mg, i);
					--i;
				}

			}
			// remove the same subgraph
			for (int i = 0; i < a_mg.equCutpairs.Count; ++i)
			{
				int nnodes = a_mg.equSubgraphs[i].nodeSets.Count;
				if (b_mg.equSubgraphs[i].nodeSets.Count != nnodes) continue;
				int[] ntypes_a = new int[MaxNumberNodeType.MAXNODETYPE];
				foreach (ModelGraphNode mn in a_mg.equSubgraphs[i].nodeSets)
					ntypes_a[mn.nodeType]++;
				int[] ntypes_b = new int[MaxNumberNodeType.MAXNODETYPE];
				foreach (ModelGraphNode mn in b_mg.equSubgraphs[i].nodeSets)
					ntypes_b[mn.nodeType]++;
				int j = 0;
				for (; j < MaxNumberNodeType.MAXNODETYPE; ++j)
				{
					if (ntypes_a[j] != ntypes_b[j])
						break;
				}
				if (j >= MaxNumberNodeType.MAXNODETYPE)
				{
					Removesolution(a_mg, b_mg, i);
					--i;
				}
			}
       
		}//end-search-replaceable-with-level

		public void SearchEqualCuts(ModelGraph a_mg, ModelGraph b_mg, bool old)
		{
			
			for (int ai = 0; ai < a_mg.graphEdges.Count; ++ai)
			{
				ModelGraphEdge a_e = a_mg.graphEdges[ai];
				for (int bi = 0; bi < b_mg.graphEdges.Count; ++bi)
				{
					// check whether two edges can connect
					ModelGraphEdge b_e = b_mg.graphEdges[bi];
					if (a_e == b_e) 
						continue;
					if (IsEqualEdge(a_e, b_e))
					{
						a_route = new List<ModelGraphEdge>();
						b_route = new List<ModelGraphEdge>();
						a_route.Add(a_e);
						b_route.Add(b_e);
						foreach (ModelGraphEdge e in a_mg.graphEdges)
							e.visitedEdgeInSearch = false;
						foreach(ModelGraphEdge e in b_mg.graphEdges)
							e.visitedEdgeInSearch = false;
						a_e.visitedEdgeInSearch = true;
						b_e.visitedEdgeInSearch = true;
						if (old)
							DFSAnEdge_old(a_mg, b_mg, a_e, b_e);
						else
						{
							DFSAnEdge(a_mg, b_mg, a_e, b_e);
							//DFSAnEdge_old(a_mg, b_mg, a_e, b_e);
						}
					}
				}
			}//search
			Filtersoultions(a_mg, b_mg);			
		}//End-search-cuts

		public void Filtersoultions(ModelGraph a_mg, ModelGraph b_mg)
		{
			//remove repetition
			for (int i = 0; i < a_mg.equCutpairs.Count - 1; ++i)
			{
				for (int j = i + 1; j < a_mg.equCutpairs.Count; ++j)
				{
					bool aequal = this.IsEqualCut(a_mg.equCutpairs[i], a_mg.equCutpairs[j]);
					bool bequal = this.IsEqualCut(b_mg.equCutpairs[i], b_mg.equCutpairs[j]);
					if (aequal && bequal)
					{
						if (a_mg.equSubgraphs[i].nodeSets.Count < a_mg.equSubgraphs[j].nodeSets.Count)
						{
							Removesolution(a_mg, b_mg, j);
							--j;
						}
						else if (a_mg.equSubgraphs[i].nodeSets.Count == a_mg.equSubgraphs[j].nodeSets.Count)
						{
							if (b_mg.equSubgraphs[i].nodeSets.Count <= b_mg.equSubgraphs[j].nodeSets.Count)
							{
								Removesolution(a_mg, b_mg, j);
								--j;
							}
							else
							{
								Removesolution(a_mg, b_mg, i);
								--i;
								break;
							}
						}
						else
						{
							Removesolution(a_mg, b_mg, i);
							--i;
							break;
						}
					}
				}
			}

			for (int i = 0; i < a_mg.equCutpairs.Count; ++i)
			{

				// cut size and different node size threshhold
				if (a_mg.equCutpairs[i].Count < minmumCutedges ||
					(a_mg.modelType != 5 && (a_mg.equSubgraphs[i].nodeSets.Count < 2 ||
					b_mg.equSubgraphs[i].nodeSets.Count < 2 ))
					//|| (a_mg.modelType == 8 &&
					//(a_mg.equSubgraphs[i].nodeSets.Count > a_mg.graphNodes.Count / 2 ||
					//b_mg.equSubgraphs[i].nodeSets.Count > b_mg.graphNodes.Count / 2))
					)
				{
					Removesolution(a_mg, b_mg, i);
					--i;
					continue;
				}
			}

			if (a_mg.modelType != 8)
			{
				
				for (int i_s = 0; i_s < a_mg.equCutpairs.Count; ++i_s)
				{
					Subgraph aSG = a_mg.equSubgraphs[i_s],
						bSG = b_mg.equSubgraphs[i_s];
					bool valid = true;
					for (int i = 0; i < aSG.edgeNodes.Count - 1; ++i)
					{
						for (int j = i + 1; j < aSG.edgeNodes.Count; ++j)
						{
							if ((aSG.connNodes[i] == aSG.connNodes[j] &&
								bSG.connNodes[i] != bSG.connNodes[j]) ||
								(aSG.edgeNodes[i] == aSG.edgeNodes[j] &&
								bSG.edgeNodes[i] != bSG.edgeNodes[j]) ||
								(aSG.connNodes[i] != aSG.connNodes[j] &&
								bSG.connNodes[i] == bSG.connNodes[j]) ||
								(aSG.edgeNodes[i] != aSG.edgeNodes[j] &&
								bSG.edgeNodes[i] == bSG.edgeNodes[j]))
							{
								valid = false;
								break;
							}
							if (!valid) break;
						}
					}
					if (!valid)
					{
						Removesolution(a_mg, b_mg, i_s);
						--i_s;
					}
				}
			}
		}//End-of-filter
		public void Removesolution(ModelGraph a_mg, ModelGraph b_mg, int index)
		{
			a_mg.equCutpairs.RemoveAt(index);
			a_mg.equSubgraphs.RemoveAt(index);
			b_mg.equCutpairs.RemoveAt(index);
			b_mg.equSubgraphs.RemoveAt(index);
		}//End-reomove-solution

		public void DFSMatchedCutWithLevel(ModelGraph a_mg, ModelGraph b_mg,
			ModelGraphEdge a_edge, ModelGraphEdge b_edge, bool useHeight)
		{
			List<ModelGraphEdge> a_neig = new List<ModelGraphEdge>(),
				b_neig = new List<ModelGraphEdge>();

			foreach (ModelGraphEdge e in a_edge.adjEdges)
            {
				if (!e.visitedEdgeInSearch)
				{
					a_neig.Add(e);
					e.visitedEdgeInSearch = true;
				}
            }
			foreach (ModelGraphEdge e in b_edge.adjEdges)
            {
				if (!e.visitedEdgeInSearch)
				{
					b_neig.Add(e);
					e.visitedEdgeInSearch = true;
				}
            }

			if (a_neig.Count == 0 || b_neig.Count == 0)
			{
				if (a_route.Count < minmumCutedges || b_route.Count < minmumCutedges) return;
				List<ModelGraphEdge> aedges = new List<ModelGraphEdge>(a_route);
				List<ModelGraphEdge> bedges = new List<ModelGraphEdge>(b_route);
				aEquCutPairs.Add(aedges);
				bEquCutPairs.Add(bedges);
			}

			foreach (ModelGraphEdge ae in a_neig)
			{
				a_route.Add(ae);
				foreach (ModelGraphEdge be in b_neig)
				{
					if ((!useHeight && IsEqualEdge(ae, be)) ||
						(useHeight && IsEqualEdgeWithLevel(ae, be, a_edge,b_edge)))
					{
						b_route.Add(be);
						this.DFSMatchedCutWithLevel(a_mg, b_mg, ae, be, useHeight);
						b_route.Remove(be);
					}
				}
				a_route.Remove(ae);
			}

			//if (a_neig.Count >= minmumCutedges || b_neig.Count >= minmumCutedges)
			//{
			//    List<ModelGraphEdge> aedges = new List<ModelGraphEdge>(a_route);
			//    List<ModelGraphEdge> bedges = new List<ModelGraphEdge>(b_route);
			//    aEquCutPairs.Add(aedges);
			//    bEquCutPairs.Add(bedges);
			//}

		}//DFS-match-cut-with-level

		public void DFSAnEdge_old(ModelGraph a_mg, ModelGraph b_mg, ModelGraphEdge a_edge, ModelGraphEdge b_edge)
		{
			List<ModelGraphEdge> a_neig = new List<ModelGraphEdge>(),
				b_neig = new List<ModelGraphEdge>();
			foreach (ModelGraphEdge e in a_edge.adjEdges)
			{
				if (!e.visitedEdgeInSearch)
				{
					a_neig.Add(e);
					if (a_mg.modelType != 5)
						e.visitedEdgeInSearch = true;
				}
			}
			foreach (ModelGraphEdge e in b_edge.adjEdges)
			{
				if (!e.visitedEdgeInSearch)
				{
					b_neig.Add(e);
					if(b_mg.modelType !=5)
						e.visitedEdgeInSearch = true;
				}
			}
			if (a_neig.Count == 0 || b_neig.Count == 0 ||
				a_route.Count > numCutThresh)
			{
				// a match or no solution
				List<Subgraph> a_outsgs = null, b_outsgs = null;
				foreach (ModelGraphEdge e in a_neig)
					e.visitedEdgeInSearch = false;
				foreach (ModelGraphEdge e in b_neig)
					e.visitedEdgeInSearch = false;
				if (a_route.Count < minmumCutedges || b_route.Count < minmumCutedges) return;

				int iscut_a = HasAvalidSubgraph(a_mg, a_route, out a_outsgs);
				int iscut_b = HasAvalidSubgraph(b_mg, b_route, out b_outsgs);
				//if (a_mg.modelType == 8 && ( a_outsgs.Count != 2 || b_outsgs.Count != 2))
				//    return;
				if (iscut_a > 1 && iscut_b > 1)
				{
					if ((a_mg.modelType == 2 || a_mg.modelType == 9) &&
						 iscut_a > 4 && iscut_b > 4) // discrete cut
						return;

					List<Subgraph> matchedSubgraphs = IsReplaceableSubgraph(a_mg, b_mg, a_outsgs, b_outsgs, a_route.Count, false);
					if (matchedSubgraphs.Count > 0)
					{
						for (int i = 0; i < matchedSubgraphs.Count; i += 2)
						{
							if (a_mg.modelType == 8 && (
								matchedSubgraphs[i].outgoingEdges.Count != a_route.Count))
								continue;
							a_mg.equSubgraphs.Add(matchedSubgraphs[i]);
							b_mg.equSubgraphs.Add(matchedSubgraphs[i + 1]);
							a_mg.equCutpairs.Add(matchedSubgraphs[i].outgoingEdges);
							b_mg.equCutpairs.Add(matchedSubgraphs[i + 1].outgoingEdges);
						}
					}
				}
				return;
			}
			foreach (ModelGraphEdge ae in a_neig)
			{
				// mark the edges in the same cycle as "visited"
				// to avoid meaningless search
				//IEnumerable<int> sects = a_edge.cycleIndices.Intersect(ae.cycleIndices);
				a_route.Add(ae);
				ae.visitedEdgeInSearch = true;
				foreach (ModelGraphEdge be in b_neig)
				{
					b_route.Add(be);
					be.visitedEdgeInSearch = true;
					if (this.IsEqualEdge(ae, be))
					{
						this.DFSAnEdge_old(a_mg, b_mg, ae, be);
					}
					be.visitedEdgeInSearch = false;
					b_route.Remove(be);
				}
				a_route.Remove(ae);
				ae.visitedEdgeInSearch = false;
			}
			foreach (ModelGraphEdge e in a_neig)
				e.visitedEdgeInSearch = false;
			foreach (ModelGraphEdge e in b_neig)
				e.visitedEdgeInSearch = false;
			if (a_mg.modelType == 5 || a_mg.modelType == 2)
			{
				List<Subgraph> a_outsgs = null, b_outsgs = null;
				int iscut_a = HasAvalidSubgraph(a_mg, a_route, out a_outsgs);
				int iscut_b = HasAvalidSubgraph(b_mg, b_route, out b_outsgs);
				if (iscut_a > 1 && iscut_b > 1)
				{
					List<Subgraph> matchedSubgraphs = IsReplaceableSubgraph(a_mg, b_mg, a_outsgs, b_outsgs, a_route.Count, false);
					if (matchedSubgraphs.Count > 0)
					{
						for (int i = 0; i < matchedSubgraphs.Count; i += 2)
						{
							a_mg.equSubgraphs.Add(matchedSubgraphs[i]);
							b_mg.equSubgraphs.Add(matchedSubgraphs[i + 1]);
							a_mg.equCutpairs.Add(matchedSubgraphs[i].outgoingEdges);
							b_mg.equCutpairs.Add(matchedSubgraphs[i + 1].outgoingEdges);
						}
					}
				}
			}
		}//End-DFS

		public void DFSAnEdge(ModelGraph a_mg, ModelGraph b_mg, ModelGraphEdge a_edge, ModelGraphEdge b_edge)
		{
			List<ModelGraphEdge> a_neig = new List<ModelGraphEdge>(),
				b_neig = new List<ModelGraphEdge>();
			foreach (ModelGraphEdge e in a_edge.adjEdges)
			{
				if (!e.visitedEdgeInSearch )
				{
					a_neig.Add(e);
				}
			}
			foreach (ModelGraphEdge e in b_edge.adjEdges)
			{
				if (!e.visitedEdgeInSearch )
				{
					b_neig.Add(e);
				}
			}
			if (a_neig.Count == 0 || b_neig.Count == 0)
			//|| a_neig.Contains(a_route[0]) || b_neig.Contains(b_route[0]))
			{
				// a match or no solution				
				if (a_route.Count < minmumCutedges || b_route.Count < minmumCutedges) return;
				List<Subgraph> a_outsgs = null, b_outsgs = null;
				int iscut_a = HasAvalidSubgraph(a_mg, a_route, out a_outsgs);
				int iscut_b = HasAvalidSubgraph(b_mg, b_route, out b_outsgs);
			// filter the subgraph that do not match the number of subgset = 2
				//if (a_outsgs.Count != 2 && b_outsgs.Count != 2)
				//    return;
				if (iscut_a > 1 && iscut_b > 1)
				{
					List<Subgraph> matchedSubgraphs = IsReplaceableSubgraph(a_mg, b_mg, a_outsgs, b_outsgs, a_route.Count, false);
					if (matchedSubgraphs.Count > 0) // two subsets
					{
						for (int i = 0; i < matchedSubgraphs.Count; i += 2)
						{
							if (matchedSubgraphs[i].outgoingEdges.Count != a_route.Count)
								continue;
							a_mg.equSubgraphs.Add(matchedSubgraphs[i]);
							b_mg.equSubgraphs.Add(matchedSubgraphs[i + 1]);
							a_mg.equCutpairs.Add(matchedSubgraphs[i].outgoingEdges);
							b_mg.equCutpairs.Add(matchedSubgraphs[i + 1].outgoingEdges);
						}
					}
				}
				return;
			}
			foreach (ModelGraphEdge ae in a_neig)
			{
				// mark the edges in the same cycle as "visited"
				// to avoid meaningless search
				List<int> sects = a_edge.cycleIndices.Intersect(ae.cycleIndices).ToList();
				if (sects.Count() > 1) return; // both in two cycles, meaningless (cut a node)
				a_route.Add(ae);
				List<int> a_labeledEidxs = new List<int>();
				if (sects.Count() == 1)
				{
					int a_cycleIdx = sects[0];
					foreach (ModelGraphEdge me in a_mg.faceCycles[a_cycleIdx].edges)
					{
						me.visitedEdgeInSearch = true;
						a_labeledEidxs.Add(me.edgeIdx);
					}
				}
				else
				{
					// put *a_edge*'s neighbors are labeled
					foreach (ModelGraphEdge adje in a_neig)
					{
						int adjindex = a_edge.adjEdges.IndexOf(adje);
						if (adje.cycleIndices.Count == 0 && a_edge.trueAdjEdges[adjindex])
						{
							adje.visitedEdgeInSearch = true;
							a_labeledEidxs.Add(adje.edgeIdx);
						}
					}
				}
				// for b
				foreach (ModelGraphEdge be in b_neig)
				{
					if (this.IsEqualEdge(ae, be))
					{
						sects = b_edge.cycleIndices.Intersect(be.cycleIndices).ToList();
						if (sects.Count() > 1) return;
						List<int> b_labeledEidxs = new List<int>();
						if (sects.Count() == 1)
						{
							int b_cycleIdx = sects[0];
							foreach (ModelGraphEdge me in b_mg.faceCycles[b_cycleIdx].edges)
							{
								me.visitedEdgeInSearch = true;
								b_labeledEidxs.Add(me.edgeIdx);
							}
						}
						else
						{
							// put *a_edge*'s neighbors are labeled
							foreach (ModelGraphEdge adje in b_neig)
							{
								int adjindex = b_edge.adjEdges.IndexOf(adje);
								if (adje.cycleIndices.Count == 0 && b_edge.trueAdjEdges[adjindex])
								{
									adje.visitedEdgeInSearch = true;
									b_labeledEidxs.Add(adje.edgeIdx);
								}
							}
						}
						b_route.Add(be);
						this.DFSAnEdge(a_mg, b_mg, ae, be);
						b_route.Remove(be);
						for (int i = 0; i < b_labeledEidxs.Count; ++i)
							b_mg.graphEdges[b_labeledEidxs[i]].visitedEdgeInSearch = false;
					}
				}
				a_route.Remove(ae);
				for (int i = 0; i < a_labeledEidxs.Count; ++i)
					a_mg.graphEdges[a_labeledEidxs[i]].visitedEdgeInSearch = false;
			}

		}//End-DFS

		public void DFSMatchedCut(ModelGraph a_mg, ModelGraph b_mg, Subgraph aSG, 
			int index, ModelGraphEdge b_edge, bool isIterative)
		{
			List<ModelGraphEdge> b_neig = new List<ModelGraphEdge>();
			foreach (ModelGraphEdge e in b_edge.adjEdges)
			{
				if (!e.visitedEdgeInSearch)
				{
					b_neig.Add(e);
				}
			}
			if (index >= aSG.outgoingEdges.Count || b_neig.Count == 0 || b_route.Count == aSG.outgoingEdges.Count)
			{
				// a match or no solution				
				if (b_route.Count != aSG.outgoingEdges.Count) return;
				List<Subgraph> a_outsgs = new List<Subgraph>(), b_outsgs = null;
				a_outsgs.Add(aSG);
				int iscut_b = HasAvalidSubgraph(b_mg, b_route, out b_outsgs);
				if (iscut_b > 1)
				{
					List<Subgraph> matchedSubgraphs = IsReplaceableSubgraph(a_mg, b_mg, a_outsgs, b_outsgs, b_route.Count, true);
					if (matchedSubgraphs.Count > 0) // two subsets
					{
						for (int i = 0; i < matchedSubgraphs.Count; i += 2)
						{
							//if (matchedSubgraphs[i + 1].nodeSets.Count
							//    > b_mg.graphNodes.Count / 2) continue;
							a_mg.equSubgraphs.Add(matchedSubgraphs[i]);
							b_mg.equSubgraphs.Add(matchedSubgraphs[i + 1]);
							a_mg.equCutpairs.Add(matchedSubgraphs[i].outgoingEdges);
							b_mg.equCutpairs.Add(matchedSubgraphs[i + 1].outgoingEdges);
						}
					}
				}
				return;
			}
			ModelGraphEdge ae = aSG.outgoingEdges[index];
			foreach (ModelGraphEdge be in b_neig)
			{
				//int nb = 0;
				//if (b_route.Count == 1 && b_route[0].edgeIdx == 0 && be.edgeIdx == 2)
				//    //&& b_route[1].edgeIdx == 21 )//&& b_route[2].edgeIdx == 39)
				//    //&& b_route[3].edgeIdx == 4)// && b_route[4].edgeIdx == 1)
				//    //&& b_route[5].edgeIdx == 34 && b_route[6].edgeIdx == 49)
					//nb = 4;

				if (!(a_mg.graphIndex == b_mg.graphIndex &&
					ae.edgeIdx == be.edgeIdx) && this.IsEqualEdge(ae, be))
				{
					List<int> sects = b_edge.cycleIndices.Intersect(be.cycleIndices).ToList();
					if (sects.Count() > 1) continue;
					// check the validity since the subgraph to match is given
					List<ModelGraphNode> b_connNodes = new List<ModelGraphNode>(),
						b_edgeNodes = new List<ModelGraphNode>();

					b_route.Add(be);
					for(int i = 0; i < index+1; ++i)
					{
						if (b_route[i].conntectingNodes[0].nodeType == aSG.edgeNodes[i].nodeType)
						{
							b_edgeNodes.Add(b_route[i].conntectingNodes[0]);
							b_connNodes.Add(b_route[i].conntectingNodes[1]);
						}
						else
						{
							b_edgeNodes.Add(b_route[i].conntectingNodes[1]);
							b_connNodes.Add(b_route[i].conntectingNodes[0]);
						}
					}
					bool valid = true;
					for (int i = 0; i < index; ++i )
					{
						for (int j = i + 1; j < index+1; ++j)
						{
							if ((aSG.connNodes[i] == aSG.connNodes[j] &&
								b_connNodes[i] != b_connNodes[j]) ||
								(aSG.edgeNodes[i] == aSG.edgeNodes[j] &&
								b_edgeNodes[i] != b_edgeNodes[j]) ||
								(aSG.connNodes[i] != aSG.connNodes[j] &&
								b_connNodes[i] == b_connNodes[j]) ||
								(aSG.edgeNodes[i] != aSG.edgeNodes[j] &&
								b_edgeNodes[i] == b_edgeNodes[j]))
							{
								valid = false;
								break;
							}
							if (!valid) break;
						}
					}
					if (!valid)
					{
						b_route.Remove(be);
						continue;
					}
					// check the angle
					if (!isIterative)
					{
						Vector3d v1_a = aSG.connNodes[index].meshComponent.boundingBox.center.pos
							- aSG.edgeNodes[index].meshComponent.boundingBox.center.pos,
							v2_a = aSG.connNodes[index - 1].meshComponent.boundingBox.center.pos
							- aSG.edgeNodes[index - 1].meshComponent.boundingBox.center.pos;
						double acos_a = v1_a.Dot(v2_a) / (v1_a.Length() * v2_a.Length());
						Vector3d v1_b = b_connNodes[index].meshComponent.boundingBox.center.pos
							- b_edgeNodes[index].meshComponent.boundingBox.center.pos,
							v2_b = b_connNodes[index - 1].meshComponent.boundingBox.center.pos
							- b_edgeNodes[index - 1].meshComponent.boundingBox.center.pos;
						double acos_b = v1_b.Dot(v2_b) / (v1_b.Length() * v2_b.Length());
						double acosv_a = Math.Acos(acos_a), acosv_b = Math.Acos(acos_b);
						double diff = Math.Abs(acos_a - acos_b);
						if (diff > 0.8)
							valid = false;

						if (!valid)
						{
							b_route.Remove(be);
							continue;
						}
					}
					List<int> b_labeledEidxs = new List<int>();
					if (sects.Count() == 1)
					{
						int b_cycleIdx = sects[0];
						foreach (ModelGraphEdge me in b_mg.faceCycles[b_cycleIdx].edges)
						{
							if (!me.visitedEdgeInSearch)
							{
								me.visitedEdgeInSearch = true;
								b_labeledEidxs.Add(me.edgeIdx);
							}
						}
					}
					else
					{
						// put *a_edge*'s neighbors are labeled
						foreach (ModelGraphEdge adje in b_neig)
						{
							int adjindex = b_edge.adjEdges.IndexOf(adje);
							if (adje.cycleIndices.Count == 0 && b_edge.trueAdjEdges[adjindex])
							{
								if (!adje.visitedEdgeInSearch)
								{
									adje.visitedEdgeInSearch = true;
									b_labeledEidxs.Add(adje.edgeIdx);
								}
							}
						}
					}
					//b_route.Add(be);
					be.visitedEdgeInSearch = true;
					this.DFSMatchedCut(a_mg, b_mg, aSG, ++index, be, isIterative);
					b_route.Remove(be);
					be.visitedEdgeInSearch = false;
					--index;
					for (int i = 0; i < b_labeledEidxs.Count; ++i)
						b_mg.graphEdges[b_labeledEidxs[i]].visitedEdgeInSearch = false;
				}
			}
		}//End-DFS-MatchedCut

		public bool IsExist(List<ModelGraphEdge> edges, ModelGraphEdge e)
		{
			//return false;
			bool exist = false;
			foreach (ModelGraphEdge mge in edges)
			{
				if ((mge.conntectingNodes[0].nodeType == e.conntectingNodes[0].nodeType &&
					mge.conntectingNodes[1].nodeType == e.conntectingNodes[1].nodeType) ||
				(mge.conntectingNodes[0].nodeType == e.conntectingNodes[1].nodeType) &&
					(mge.conntectingNodes[1].nodeType == e.conntectingNodes[0].nodeType))
				{
					exist = true;
					break;
				}
			}
			return exist;
		}

		public void SetNodeIndex(ModelGraph mg)
		{
			int nidx = 0;
			foreach (ModelGraphNode mgn in mg.graphNodes)
				mgn.nodeIdx = nidx++;
			int eidx = 0;
			foreach (ModelGraphEdge mge in mg.graphEdges)
				mge.edgeIdx = eidx++;
		}//end-set-node-index
		
		public int HasAvalidSubgraph(ModelGraph mg, List<ModelGraphEdge> edges, out List<Subgraph> subgraphs)
		{
			// check independent node set, only keep the subgraph with all outgoing edges,
			// that is, a connected (sub)graph
			//bool iscut = false;
			int nIndepSets = 0;
			// a cut split the graph into several ( at least 2 ) independent sets
			int nEdges = mg.graphEdges.Count;
			List<ModelGraphNode> activeNodes = new List<ModelGraphNode>();
			int nActNodes = 0;
			// label cut edges as inAccesible edges
			bool[] isAccessibleEdges = new bool[nEdges];
			for (int i = 0; i < nEdges; ++i)
				isAccessibleEdges[i] = true;
			List<ModelGraphEdge> realcutEdges = new List<ModelGraphEdge>();
			for (int i = 0; i < mg.graphEdges.Count; i++)
			{
				if (!edges.Contains(mg.graphEdges[i]) || mg.graphEdges[i].isDummyEdge)
					continue;
				ModelGraphNode mn1 = mg.graphEdges[i].conntectingNodes[0];
				ModelGraphNode mn2 = mg.graphEdges[i].conntectingNodes[1];
				isAccessibleEdges[i] = false;
				if (!activeNodes.Contains(mn1))
					activeNodes.Add(mn1);
				if (!activeNodes.Contains(mn2))
					activeNodes.Add(mn2);
				realcutEdges.Add(mg.graphEdges[i]);
			}
			nActNodes = activeNodes.Count;
			// DFS to find out inaccessible nodes
			bool[,] isAccNodes = new bool[nActNodes, nActNodes];
			List<bool> accNodes = new List<bool>();
			int nNodes = mg.graphNodes.Count;
			bool[,] accessNodesForIndependentSet = new bool[nNodes, nNodes];
			// start from each node (according to the edges), 
			// find all nodes that can be accessed from it by removing all edges in the input
			for (int n = 0; n < nActNodes; n++)
			{
				bool[] isaccess = new bool[nNodes];
				List<ModelGraphNode> queue = new List<ModelGraphNode>();
				queue.Add(activeNodes[n]);
				while (queue.Count > 0)
				{
					ModelGraphNode currNode = queue[0];
					queue.RemoveAt(0);
					foreach (ModelGraphEdge e in currNode.edges)
					{
						int index = e.edgeIdx; // already re-ordered
						if (!isAccessibleEdges[index] || e.isDummyEdge)
							continue;
						ModelGraphNode mn = null;
						if (e.conntectingNodes[0] == currNode)
							mn = e.conntectingNodes[1];
						else
							mn = e.conntectingNodes[0];
						//if (mn.canStretch) continue;
						if (!isaccess[mn.nodeIdx])
						{
							queue.Add(mn);
							isaccess[mn.nodeIdx] = true;
						}
					}
				}//end-while
				// record inaccessible nodes
				isAccNodes[n, n] = true;
				for (int i = 0; i < nActNodes; i++)
					isAccNodes[n, i] = isaccess[activeNodes[i].nodeIdx];
				for (int i = 0; i < nNodes; ++i)
					accessNodesForIndependentSet[activeNodes[n].nodeIdx, i] = isaccess[i];
			}//end-while

			// stat. the independent node sets
			List<ModelGraphNode> nodeset = null;
			bool[] visited = new bool[nNodes];
			List<List<ModelGraphNode>> indepNodeSets = new List<List<ModelGraphNode>>();
			for (int i = 0; i < nActNodes; i++)
			{
				int nidx = activeNodes[i].nodeIdx;
				if (visited[nidx])
					continue;
				visited[nidx] = true;
				nodeset = new List<ModelGraphNode>();
				nodeset.Add(activeNodes[i]);
				List<ModelGraphNode> queue = new List<ModelGraphNode>(); // DFS to define an independent set
				queue.Add(activeNodes[i]);
				while (queue.Count > 0)
				{
					ModelGraphNode currActNode = queue[0];
					queue.RemoveAt(0);
					for (int j = 0; j < nNodes; j++)
					{
						if (!visited[j] && accessNodesForIndependentSet[currActNode.nodeIdx, j])
						{
							visited[j] = true;
							nodeset.Add(mg.graphNodes[j]);
							queue.Add(mg.graphNodes[j]);
						}
					}
				}
				indepNodeSets.Add(nodeset);
			}
			nIndepSets = indepNodeSets.Count;
			subgraphs = new List<Subgraph>();
			if (nIndepSets < 2) return nIndepSets;

			List<ModelGraphEdge> outgoingEdges = new List<ModelGraphEdge>();
			for (int i = 0; i < nIndepSets; ++i)
			{
				List<ModelGraphEdge> outEdges = mg.GetOutgoingEdges(indepNodeSets[i]);
				outgoingEdges.AddRange(outEdges);
				//if (outEdges.Count < edges.Count) continue;
				//outEdges = edges;
				List<ModelGraphEdge> inEdges = mg.GetIngoingEdges(indepNodeSets[i]);
				Subgraph subgraph = new Subgraph(mg.graphIndex, indepNodeSets[i], outEdges, inEdges);
				subgraphs.Add(subgraph);				
			}
			// in some cases, there are redundancy cut edges, then not all cut edges
			// belong to the outgoing edges of subgraphs, remove such cases
			bool included = true;
			foreach (ModelGraphEdge e in realcutEdges)
			{
				if (!outgoingEdges.Contains(e))
				{
					included = false;
					break;
				}
			}
			if (!included)
			{
				nIndepSets = 0;
				subgraphs = new List<Subgraph>();
			}

			return nIndepSets;
		}//End-A-Cut

		public bool IsEqualCut(List<ModelGraphEdge> edges1, List<ModelGraphEdge> edges2)
		{ 
	
			// check whether two cuts are exactly the same
			bool isIden = true;
			bool iden1 = true, iden2 = true;
			foreach (ModelGraphEdge e1 in edges1)
			{
				if(e1.isDummyEdge) continue;
				if (!edges2.Contains(e1))
				{
					iden1 = false;
					break;
				}
			}
			foreach (ModelGraphEdge e2 in edges2)
			{
				if (e2.isDummyEdge) continue;
				if (!edges1.Contains(e2))
				{
					iden2 = false;
					break;
				}
			}
			isIden = iden1 && iden2;

			return isIden;
		}

		public bool IsEqualCutPair(List<ModelGraphEdge> edges1, List<ModelGraphEdge> edges2)
		{

			// check whether two cuts are exactly the same
			bool isIden = true;
			bool iden1 = true, iden2 = true;
			List<int> eidx1 = new List<int>(), eidx2 = new List<int>();
			foreach (ModelGraphEdge e1 in edges1)
			{
				eidx1.Add(e1.edgeIdx);
			}
			foreach (ModelGraphEdge e2 in edges2)
			{
				eidx2.Add(e2.edgeIdx);
			}
			for (int i = 0; i < eidx1.Count; ++i)
			{
				if (!eidx2.Contains(eidx1[i]))
				{
					iden1 = false;
					break;
				}
			}
			for (int i = 0; i < eidx2.Count; ++i)
			{
				if (!eidx1.Contains(eidx2[i]))
				{
					iden2 = false;
					break;
				}
			}
			isIden = iden1 && iden2;

			return isIden;
		}

		public List<Subgraph> IsReplaceableSubgraph(ModelGraph a_mg, ModelGraph b_mg,
				List<Subgraph> a_sgs, List<Subgraph> b_sgs, int edgeLength, bool a_order)
		{
			// edgeLength == #cut edge
			List<Subgraph> matchedsubgs = new List<Subgraph>();
			// check whether the docking sites match for subgraph replacement
			for (int i = 0; i < a_sgs.Count; ++i)
			{
				if (a_sgs[i].outgoingEdges.Count != edgeLength) continue;
				for (int j = 0; j < b_sgs.Count; ++j)
				{
					if (b_sgs[j].outgoingEdges.Count != edgeLength) continue;
					bool matched = IsMatchedSubgraph(a_mg, b_mg, a_sgs[i], b_sgs[j], a_order);
					if (matched)
					{
						matchedsubgs.Add(a_sgs[i]);
						matchedsubgs.Add(b_sgs[j]);
					}
				}
			}
			return matchedsubgs;
		}//End-MatchDockingsites
		public int minmumCutedges = 2;
		public bool IsMatchedSubgraph(ModelGraph a_mg, ModelGraph b_mg,
			Subgraph a_sg, Subgraph b_sg, bool a_order)
		{
			bool conn_matched = false;
			if (a_sg.connNodes.Count != b_sg.connNodes.Count ||
				a_sg.edgeNodes.Count != b_sg.edgeNodes.Count ||
				a_sg.outgoingEdges.Count != a_sg.connNodes.Count ||
				b_sg.outgoingEdges.Count != b_sg.connNodes.Count)
				return false;
			int numToCompare = a_sg.connNodes.Count;
			List<ModelGraphEdge> orderedEdges = null;
			List<ModelGraphNode> orderedNodes = null;
			if (numToCompare > 2)
			{
				if (!a_order)
				{
					a_mg.OrderEdges(a_sg.outgoingEdges, a_sg.connNodes, out orderedEdges, out orderedNodes);
					a_sg.outgoingEdges = orderedEdges;
					a_sg.connNodes = orderedNodes;
					orderedEdges = null;
					orderedNodes = null;
				}
				
				b_mg.OrderEdges(b_sg.outgoingEdges, b_sg.connNodes, out orderedEdges, out orderedNodes);
				b_sg.outgoingEdges = orderedEdges;
				b_sg.connNodes = orderedNodes;
			}
			if (!a_order)
			{
				a_sg.edgeNodes = new List<ModelGraphNode>();
				for (int i = 0; i < numToCompare; ++i)
				{
					ModelGraphEdge me = a_sg.outgoingEdges[i];
					if (me.conntectingNodes[0] == a_sg.connNodes[i])
						a_sg.edgeNodes.Add(me.conntectingNodes[1]);
					else
						a_sg.edgeNodes.Add(me.conntectingNodes[0]);
				}
			}
			b_sg.edgeNodes = new List<ModelGraphNode>();
			for (int i = 0; i < numToCompare; ++i)
			{
				ModelGraphEdge me = b_sg.outgoingEdges[i];
				if (me.conntectingNodes[0] == b_sg.connNodes[i])
					b_sg.edgeNodes.Add(me.conntectingNodes[1]);
				else
					b_sg.edgeNodes.Add(me.conntectingNodes[0]);
			}
			// match
			int b_startNidx = 0;
			while (true)
			{
				conn_matched = false;
				int b_nidx = -1;
				for (int i = 0; i < numToCompare; ++i)
				{
					for (int j = b_startNidx; j < numToCompare; ++j)
					{
						if (a_sg.connNodes[i].nodeType == b_sg.connNodes[j].nodeType &&
							a_sg.edgeNodes[i].nodeType == b_sg.edgeNodes[j].nodeType)
						{
							b_nidx = j;
							break;
						}
					}
					if (b_nidx != -1) break;
				}
				if (b_nidx == -1) break;
				// begin compare
				conn_matched = true;
				for (int i = 1; i < numToCompare; ++i)
				{
					if ((a_sg.connNodes[(i) % numToCompare].nodeType !=
						b_sg.connNodes[(b_nidx + i) % numToCompare].nodeType) ||
						(a_sg.edgeNodes[(i) % numToCompare].nodeType !=
						b_sg.edgeNodes[(b_nidx + i) % numToCompare].nodeType))
					{
						conn_matched = false;
						break;
					}
				}
				b_startNidx = b_nidx;
				if (conn_matched)
					break;
				b_startNidx = b_nidx + 1;
				if (b_startNidx >= numToCompare)
					break;
			}//end-while

			if (!conn_matched) return false;

			// correct the matched order
			List<ModelGraphEdge> newOrderedEdges = new List<ModelGraphEdge>();
			List<ModelGraphNode> orderedNodes_conn = new List<ModelGraphNode>(),
				orderedNodes_edge = new List<ModelGraphNode>();
			for (int i = 0; i < b_sg.outgoingEdges.Count; ++i)
			{
				ModelGraphEdge me = b_sg.outgoingEdges[(b_startNidx + i) % numToCompare];
				newOrderedEdges.Add(me);
				if (b_sg.nodeSets.Contains(me.conntectingNodes[0]))
				{
					orderedNodes_conn.Add(me.conntectingNodes[1]);
					orderedNodes_edge.Add(me.conntectingNodes[0]);
				}
				else if (b_sg.nodeSets.Contains(me.conntectingNodes[1]))
				{
					orderedNodes_conn.Add(me.conntectingNodes[0]);
					orderedNodes_edge.Add(me.conntectingNodes[1]);
				}
			}
			b_sg.outgoingEdges = newOrderedEdges;
			b_sg.connNodes = orderedNodes_conn;
			b_sg.edgeNodes = orderedNodes_edge;

			return true;
		}//End-Match-subgraph
	
		public bool IsEqualEdge(ModelGraphEdge e1, ModelGraphEdge e2)
		{
			bool match = false;
			if ((e1.conntectingNodes[0].nodeType == e2.conntectingNodes[0].nodeType &&
				e1.conntectingNodes[1].nodeType == e2.conntectingNodes[1].nodeType) ||
				(e1.conntectingNodes[0].nodeType == e2.conntectingNodes[1].nodeType &&
				e1.conntectingNodes[1].nodeType == e2.conntectingNodes[0].nodeType))
				match = true;
			return match;
		}

		public bool IsEqualEdgeWithLevel(ModelGraphEdge e1, ModelGraphEdge e2, 
			ModelGraphEdge e10, ModelGraphEdge e20)
		{
			bool match = false;
			if (e1.level - e10.level != e2.level - e20.level) return false;
			if ((e1.conntectingNodes[0].nodeType == e2.conntectingNodes[0].nodeType &&
				e1.conntectingNodes[1].nodeType == e2.conntectingNodes[1].nodeType ) ||
				(e1.conntectingNodes[0].nodeType == e2.conntectingNodes[1].nodeType &&
				e1.conntectingNodes[1].nodeType == e2.conntectingNodes[0].nodeType ))
				match = true;
			return match;
		}

		//////////////////// End - previous search using dfs ////////////////////
	}// End -- GraphGroup

}
